/***************************************************************************
 *   Implementation of the Hierarchical RLE level set                      *
 *   Copyright (C) 2007 by Holger Gerth                                    *
 *   Chemnitz University of Technology                                     *
 *   holger.gerth@informatik.tu-chemnitz.de                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef HIERARCHICALRLE_H
#define HIERARCHICALRLE_H

#include <vector>
#include "includes.h"
#include "voxelobject3d.h"
#include "structures.h"

#define RT_NEGATIVE     -1
#define RT_POSITIVE     2147483647
#define RT_DEFINED(x)      ((RT_NEGATIVE < (x)) && ((x) < RT_POSITIVE))

/**
    * @short Hierarchical Run Lenght Encoded Level Set for binary 3D voxel images
    * @author Holger Gerth <holger.gerth@informatik.tu-chemnitz.de>
    * @version 0.3
*/

class CHierarchicalRLE{
public:
    unsigned int                dimension;                           // dimension of the hrle level set

    vector<unsigned int>        start_indices;          // start indices and extends for the segments
    short int                   min;                    // minimal extend along the axis of encoding
    short                       max;                    // maximum+1 extend along the axis of encoding
    vector<short int>           run_breaks;            // start of each run (exept of the first run)
    vector<int>                 run_types;              // how each segment is divided into different runs
    CHierarchicalRLE*           lower_hrle;             // hrle level set of dimension-1

    CHierarchicalRLE();
    CHierarchicalRLE( unsigned int dim );
    CHierarchicalRLE( unsigned int dim, CVoxelObject3D* vo );

    ~CHierarchicalRLE();

    // public classes
    bool convert_to_hrle( CVoxelObject3D* vo );                                                 // convert an 3D Voxel Grid into a HRLE level set
    int randomAccess( vector<int> qv, int segmentIndex = 0);                               // random access operator
    int randomAccess( int X, int Y, int Z, int segmentIndex = 0);
    CHierarchicalRLE* getSegmentAt( int si, bool withSubSegments = true ); // return this segment as seperate H-RLE
    bool addToHRLE( CHierarchicalRLE *hrleObject );                      // add another HRLE Level Set to the current
    CHierarchicalRLE* dilation( vector<bool> structureElement );         // dilation algorithm
    CHierarchicalRLE* advancedDilation( CHierarchicalRLE* structureElement, vector<short int> qv );
    bool computeMedialAxis( bool bTopHold = false, bool bBottomHold = false );
    bool computeMedialSurface( bool bTopHold = false, bool bBottomHold = false );
    CHierarchicalRLE* getMedialSurface( bool bTopHold = false, bool bBottomHold = false );
    CHierarchicalRLE* getMedialAxis( bool bTopHold = false, bool bBottomHold = false );
    bool copy( CHierarchicalRLE* value );
    int getRunByCoordinate( int qi, int segmentIndex = 0, short int* startCoord = NULL );    // get run type by coordinate & segment index
    bool thinObject( short int checkPlaneValue, borderPointType type );
    CHierarchicalRLE* getDilatedHRLE( vector<bool> structureElement );   // returns the dilated HRLE level set
    CHierarchicalRLE* getDilatedHRLE( CHierarchicalRLE* structureElement, vector<short int> mpv );
    CHierarchicalRLE* getErodedHRLE( CHierarchicalRLE* structureElement, vector<short int> mpv );

///////// for testing only /////////////
        // for time measurement
        double accessTimeRandomAccess, accessAverageTimeRandomAccess, accessTimeSequentialAccess;
        // for draw cubes
        CVertex3D                       minRange, maxRange;                  // range

        void drawCubesUsingSequentialAccess();                               // for testing sequential access
        void drawCubes();                                                    // for testing random access
        void printDataStructure();                                           // prints the full data structure to std out
        void printNeighborOfGridPoint(int x, int y, int z, int axis = 0);    // print the values of the neighbors of grid point (x,y,z)
        double getMemorySize();

        void drawACube(CVertex3D *p1, CVertex3D *p2, double colorValue);    // draw the voxels as cubes

        // get all border points
        vector<definedPoint> getBorderPoints();
        vector<definedPoint> getBorderPoints( vector<int> parentCoord, CHierarchicalRLE* topHRLE, int segmentIndex = 0 );
///////////////////////////////////////

private:

    vector<adjacency>           adjancenyTableN26, adjancenyTableN6, adjancenyTableN8;

    bool create_hrle_from_voxel_object( CVoxelObject3D* vo );            // transform voxel object into an hrle level set
    int accessBinSearch( int qi, unsigned int indexFirstRun, unsigned int indexFirstBreak, int segmentLength,
                         short int *min, short int *max );       // binary search for the random access operator
    // the old dilation
    CHierarchicalRLE* dilation( CHierarchicalRLE* dilatedHRLE, vector<bool> structureElement, bool left, bool middle, bool right, int si = 0 );
    CHierarchicalRLE* dilateSegment( int si, bool left, bool middle, bool right ); // 1D dilation
    CHierarchicalRLE* getDilatedSubsegment( int qi, int si, vector<bool> structureElement );
    // erosions algorithm
    CHierarchicalRLE* erosion( CHierarchicalRLE* structureElement, vector<short int> qv, int si = 0 );
    CHierarchicalRLE* erosion1D( int si, CHierarchicalRLE* structureElement, short int nCoord );
    bool coordinateIsSignificant( CHierarchicalRLE *structElement, int si = 0 );
    CHierarchicalRLE* hrleUnion( vector<CHierarchicalRLE*> hrleVector ); // union of some HRLE Level Sets
    CHierarchicalRLE* hrleUnion( CHierarchicalRLE* first, CHierarchicalRLE* second );
    CHierarchicalRLE* hrleIntersection( CHierarchicalRLE* first, CHierarchicalRLE* second );
    CHierarchicalRLE* hrleIntersection( vector<CHierarchicalRLE*> hrleVector );
    vector<bool> neighbor_N6( vector<int> point, int segmentIndex = 0 );
    vector<bool> neighbor_N26( vector<int> point, int segmentIndex = 0 );
    vector<bool> neighbor_N18( vector<int> point, int segmentIndex );
    bool sameRunType( int a, int b );
    vector<definedPoint> getDefinedPoints();
    vector<definedPoint> getDefinedPoints( vector<int> parentCoord, int segmentIndex = 0 );
    bool isFinalPoint( vector<int> bp );
    int getSegmentIndex( vector<int> qv, int segmentIndex = 0 );
    bool deleteBorderPoints_bool( vector<coordValue> deleteablePoints );
    CHierarchicalRLE* deleteBorderPoints( vector<coordValue> deleteablePoints );
    CHierarchicalRLE* dilation1D( int si, CHierarchicalRLE* structureElement, short int nCoord );
    CHierarchicalRLE* uniteSegments( CHierarchicalRLE* currentSegment, CHierarchicalRLE* structureElement );
    CHierarchicalRLE* advancedDilation( CHierarchicalRLE* dilatedHRLE, CHierarchicalRLE* structureElement, vector<short int> qv, int si = 0  );
    bool addToDeleteablePoints( vector<int> qv, vector<coordValue> *deleteablePoints );
    bool checkingPlaneCondition( vector<int> bp, int numRemainingVoxels, borderPointType type );
    short int connectedComponets2D( vector<bool> array2D );
    bool isBorderPointOfDirection( vector<int> coord, borderPointType type );
    vector<short int> connectedComponentsN3D_N26( vector<bool> inArray, connectivity con );
    short int countConnectedComponentsN3D_N26( vector<bool> inArray, connectivity con );
    bool isSimplePoint( vector<int> coord );
    bool isBorderPoint( vector<int> coord );
    bool isHoldPoint( vector<int> coord, borderPointType bpType, bool bTopHold, bool bBottomHold );
    void printCoordValue( vector<coordValue> value );
    void createAdjacencyTables();

};

#endif
