/***************************************************************************
 *   Copyright (C) 2007 by Holger Gerth                                    *
 *   holger.gerth@informatik.tu-chemnitz.de                                *
 *                                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "qopenglwidget.h"

CqOpenGLwidget::CqOpenGLwidget(QWidget *parent) : QGLWidget(parent) {
    // constructor
    this->camera = new CoglCamera3D();
    this->myMouse = CMouse();

    // OpenGL status flags
    this->bLighting = false;
    this->bCoordinates = false;
    this->bWireFrame = false;
    this->bColorMaterial = false;
    this->bCullBackFace = false;
    this->bTransparency = false;

    this->bSmfObjectStatus = false;
    this->bVoxelObjectStatus = false;
    this->bHrleObjectStatus = false;
    this->bHrleBObjectStatus = false;

    this->bSmfObjectChanged = false;
    this->bVoxelObjectChanged = false;
    this->bHrleObjectChanged = false;
    this->bHrleBObjectChanged = false;

    this->bDrawVoxelObject = false;
    this->bDrawSmfObject = false;
    this->bDrawHrleRand = false;
    this->bDrawHrleSeq = false;

    this->bHrleBObjectStatus = false;
    this->bHrleBObjectChanged = false;
    this->bDrawHrleB = false;

    this->iCameraMode = 0;

    this->myHRLE = NULL;
    this->mySmfObject = NULL;
    this->myVoxelObject = NULL;
    this->myHRLEB = NULL;

    // start the idle timer
    this->idleTimer = new QTimer (this);
    connect( idleTimer, SIGNAL( timeout() ), SLOT( idleFuncGL() ) );
    this->idleTimer->start(0);
}

CqOpenGLwidget::~CqOpenGLwidget() {

    delete idleTimer;
    delete camera;

    if ( this->myHRLE != NULL ) delete this->myHRLE;
    if ( this->mySmfObject != NULL ) delete this->mySmfObject;
    if ( this->myVoxelObject != NULL ) delete this->myVoxelObject;
    if ( this->myHRLEB != NULL ) delete this->myHRLEB;

}

void CqOpenGLwidget::initializeGL() {

    // GL state initializations

    // set background color to black
    glClearColor(0.01, 0.01, 0.1, 1.0);

    // shading model
    glShadeModel(GL_SMOOTH);

    // clear Z-buffer
    glClear(GL_DEPTH_BUFFER_BIT);

    // Z-buffer
    glEnable(GL_DEPTH_TEST);

    // normalize normals
    glEnable(GL_NORMALIZE);

    glFrontFace(GL_CW);

    // initialize viewing values
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

    // set global ambient light
    float glob_amb[] = {0.2,0.2,0.2};
    glLightModelfv(GL_LIGHT_MODEL_AMBIENT, glob_amb);

    // LightModel Local Viewer
//      glLightModelf(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE);
//      glLightModelf(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);

    // set mode for color material
    glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);

    // set clipping planes
    glFrustum(
            -0.5,               // left Plane
            +0.5,               // right Plane
            -0.5,               // upper Plane
            +0.5,               // lower Plane
            +1.0,               // near
            +2000);             // far

    // switch to modelview matrix
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    // load static gl draw lists
    staticListsGL();

}

void CqOpenGLwidget::paintGL() {
    // draw scene

    // clear color and depth buffer
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    //draw wireframe
    if (this->bWireFrame)
        glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
    else
        glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

    // switch to modelview matrix
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    camera->setCameraMode( iCameraMode );
    camera->setCamPos();

///////////////////////////////////////////////////////////////////////////////////////////
//      light definitions
///////////////////////////////////////////////////////////////////////////////////////////

    // Lightsource 0
    float light0_pos[] = {5.0, 0.0, -2.0, 0.0};
    float light0_amb[] = {0.1, 0.1, 0.1};
    float light0_dif[] = {0.5, 0.5, 0.5};
    float light0_spec[] = {0.5, 0.5, 0.5};

    glLightfv(GL_LIGHT0, GL_POSITION, light0_pos);
    glLightfv(GL_LIGHT0, GL_AMBIENT, light0_amb);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, light0_dif);
    glLightfv(GL_LIGHT0, GL_SPECULAR, light0_spec);

    // Lightsource 1
    float light1_pos[] = {0.0, 5.0, 2.0, 0.0};
    float light1_amb[] = {0.0, 0.0, 0.0};
    float light1_dif[] = {0.7, 0.7, 0.7};
    float light1_spec[] = {0.5, 0.5, 0.5};

    glLightfv(GL_LIGHT1, GL_POSITION, light1_pos);
    glLightfv(GL_LIGHT1, GL_AMBIENT, light1_amb);
    glLightfv(GL_LIGHT1, GL_DIFFUSE, light1_dif);
    glLightfv(GL_LIGHT1, GL_SPECULAR, light1_spec);

    // Lightsource 2
    float light2_pos[] = {-1.0, 1.0, 1.0, 0.0};
    float light2_amb[] = {0.0, 0.0, 0.0};
    float light2_dif[] = {0.5, 0.5, 0.5};
    float light2_spec[] = {0.5, 0.5, 0.5};

    glLightfv(GL_LIGHT2, GL_POSITION, light2_pos);
    glLightfv(GL_LIGHT2, GL_AMBIENT, light2_amb);
    glLightfv(GL_LIGHT2, GL_DIFFUSE, light2_dif);
    glLightfv(GL_LIGHT2, GL_SPECULAR, light2_spec);

    // Lightsource 3
    float light3_pos[] = {1.0, 1.0, -1.0, 0.0};
    float light3_amb[] = {0.0, 0.0, 0.0};
    float light3_dif[] = {0.5, 0.5, 0.5};
    float light3_spec[] = {0.5, 0.5, 0.5};

    glLightfv(GL_LIGHT3, GL_POSITION, light3_pos);
    glLightfv(GL_LIGHT3, GL_AMBIENT, light3_amb);
    glLightfv(GL_LIGHT3, GL_DIFFUSE, light3_dif);
    glLightfv(GL_LIGHT3, GL_SPECULAR, light3_spec);

///////////////////////////////////////////////////////////////////////////////////////////
//		change OpenGL state
///////////////////////////////////////////////////////////////////////////////////////////	

    // set OpenGL status
        // Lighting
        if (this->bLighting)
            glEnable(GL_LIGHTING);
        else
            glDisable(GL_LIGHTING);

        glEnable(GL_LIGHT0);
        glEnable(GL_LIGHT1);
//         glEnable(GL_LIGHT2);
//         glEnable(GL_LIGHT3);

    // front face to counter clockwise
    glFrontFace(GL_CCW);

    // glColor-Modus
    if (this->bColorMaterial)
        glEnable(GL_COLOR_MATERIAL);
    else
        glDisable(GL_COLOR_MATERIAL);

    // backface Culling
    if (this->bCullBackFace) 
        glEnable(GL_CULL_FACE);
    else 
        glDisable(GL_CULL_FACE);

    // Alpha Blending
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    if (this->bTransparency)
        glEnable(GL_BLEND);
    else
        glDisable(GL_BLEND);

    // draw coordinates
    if (this->bCoordinates)
        glCallList((GLint) COORDINATES);

///////////////////////////////////////////////////////////////////////////////////////////
//		Draw the object
///////////////////////////////////////////////////////////////////////////////////////////

//     CWavingFlag flag;
//     flag.drawWavingFlag();

////////////////
// --> fr Studienarbeit
///////////////

    glPushMatrix();

        glTranslatef(0, 1, 0);

        if ( this->bSmfObjectStatus && this->bDrawSmfObject ) {
            glCallList((GLint) SMFOBJECT);
        }

        if ( this->bVoxelObjectStatus && this->bDrawVoxelObject ) {
            glCallList((GLint) VOXELOBJECT);
        }

        if ( this->bHrleObjectStatus && this->bDrawHrleRand ) {
            glCallList((GLint) HRLE_RANDOM);
        }

        if ( this->bHrleBObjectStatus && this->bDrawHrleB ) {
            glCallList((GLint) HRLEB_RANDOM);
        }

    glPopMatrix();

///////////////////////////////////////////////////////////////////////////////////////////
//		end of drawing
///////////////////////////////////////////////////////////////////////////////////////////

    swapBuffers();
    glFlush();

}

/*!
    \fn CqOpenGLwidget::mouseMoveEvent( QMouseEvent *ev )
*/
void CqOpenGLwidget::mouseMoveEvent( QMouseEvent *ev )
{
// mouse movement callback function

    if ( QEvent::MouseMove ) {

        myMouse.setCurX( ev->globalX() );
        myMouse.setCurY( ev->globalY() );

        switch (myMouse.getLastState()) {
            case (Qt::LeftButton):
                myMouse.setMoveX( myMouse.getCurX() - myMouse.getOldX() );
                myMouse.setMoveY( myMouse.getCurY() - myMouse.getOldY() );
                break;
            case (Qt::RightButton):
                myMouse.setRadius( myMouse.getRadius() + myMouse.getCurY() - myMouse.getOldY() );
                myMouse.setMoveY( myMouse.getCurY() - myMouse.getOldY() );
                break;

            default: break;
        }

        if (myMouse.getRadius() < 1.0) myMouse.setRadius( 1.0 );

        camera->updateMouseData( myMouse.getMoveX(), myMouse.getMoveY(), myMouse.getRadius(), myMouse.getLastState());

        myMouse.setOldX( ev->globalX() );
        myMouse.setOldY( ev->globalY() );

        updateGL();
    }
}


/*!
    \fn CqOpenGLwidget::mousePressEvent( QMouseEvent *ev )
*/
void CqOpenGLwidget::mousePressEvent( QMouseEvent *ev )
{
    // if mouse button is pressed

    if (ev->button() != Qt::NoButton) myMouse.setLastState( ev->button() );
    else myMouse.setLastState( -1 );

    myMouse.setOldX( ev->globalX() );
    myMouse.setOldY( ev->globalY() );
}


/*!
    \fn CqOpenGLwidget::mouseReleaseEvent( QMouseEvent *ev )
*/
void CqOpenGLwidget::mouseReleaseEvent( QMouseEvent *ev )
{
    // if mouse button is released

    if (ev->button() == Qt::NoButton)myMouse.setLastState( -1 );

}


void CqOpenGLwidget::resizeGL(int w, int h) {
    // handle resize events

    // set viewport to window size
    glViewport(0, 0, (GLsizei) w, (GLsizei) h);

    updateGL();

}

/*!
    \fn CqOpenGLwidget::staticListsGL()
*/
void CqOpenGLwidget::staticListsGL()
{
    // static lists loaded at startup

    // draw coordinate system
    glNewList((GLint) COORDINATES, GL_COMPILE);
            glPushAttrib(GL_LIGHTING_BIT);
            glPushMatrix();
                    glDisable(GL_LIGHTING);
                    glLineWidth(3);
                    glBegin(GL_LINES);
                            //X axis (red)
                            glColor4f(1.0,0.0,0.0,1.0);
                            glVertex3d(0.0,0.0,0.0);
                            glVertex3d(5.0,0.0,0.0);

                            // Y axis (green)
                            glColor4f(0.0,1.0,0.0,1.0);
                            glVertex3d(0.0,0.0,0.0);
                            glVertex3d(0.0,5.0,0.0);

                            // Z axis (blue)
                            glColor4f(0.0,0.0,1.0,1.0);
                            glVertex3d(0.0,0.0,0.0);
                            glVertex3d(0.0,0.0,5.0);
                    glEnd();
                    glLineWidth(1);
            glPopMatrix();
            glPopAttrib();
    glEndList();

}


/*!
    \fn CqOpenGLwidget::dynListsGL()
*/
//non-static GL-Lists
void CqOpenGLwidget::dynListsGL()
{
    // GL lists which can be changed during execution

    if ( this->bSmfObjectStatus && this->bSmfObjectChanged ) {
        glNewList((GLint) SMFOBJECT, GL_COMPILE);
            glPushMatrix();
                mySmfObject->draw();
            glPopMatrix();
        glEndList();
    }

    if ( this->bHrleObjectStatus && this->bHrleObjectChanged ) {
        glNewList((GLint) HRLE_RANDOM, GL_COMPILE);
            glPushMatrix();
                glTranslatef(0, 0, -1);
                myHRLE->drawCubes();
            glPopMatrix();
        glEndList();
    }

    if ( this->bHrleBObjectStatus && this->bHrleBObjectChanged ) {
        glNewList((GLint) HRLEB_RANDOM, GL_COMPILE);
            glPushMatrix();
//                 glTranslatef(0, 0, 0);
                myHRLEB->drawTheObject();
            glPopMatrix();
        glEndList();
    }

    this->bSmfObjectChanged = false;
    this->bVoxelObjectChanged = false;
    this->bHrleObjectChanged = false;
    this->bHrleBObjectChanged = false;

    updateGL();

}

/*!
    \fn CqOpenGLwidget::idleFuncGL()
*/
// idle function
void CqOpenGLwidget::idleFuncGL()
{

        updateGL();
}

/*!
    \fn CqOpenGLwidget::loadVoxelObject( int numCubes )
 */
// create a voxel object from an triangulated object
bool CqOpenGLwidget::loadVoxelObject( int numCubes )
{

    if ( this->bVoxelObjectStatus )
        delete this->myVoxelObject;

    this->myVoxelObject = new CVoxelObject3D();
    this->myVoxelObject->TransformTriangleMeshToVoxelSet( this->mySmfObject, numCubes );
    this->bVoxelObjectStatus = true;
    this->myVoxelObject->countDefinedVoxels();

//     this->bVoxelObjectChanged = true;

    this->dynListsGL();

    return true;
}

/*!
    \fn CqOpenGLwidget::loadSmfObject( char* filename )
 */
// load triangulated object from a .smf file
bool CqOpenGLwidget::loadSmfObject( const char* filename )
{
    if ( this->bSmfObjectStatus )
        delete this->mySmfObject;

    this->mySmfObject = new CsmfObject( filename );
    this->bSmfObjectStatus = true;

    this->bSmfObjectChanged = true;

    this->dynListsGL();

    return true;
}

/*!
    \fn CqOpenGLwidget::saveVoxelObjectAsHRLEB( QString directory )
 */
// convert a voxel object into a HRLE binary file
bool CqOpenGLwidget::saveVoxelObjectAsHRLEB( QString directory )
{
    if ( this->bVoxelObjectStatus ) {

      this->myVoxelObject->convertToHRLEBfile( directory.toStdString() );

      return true;
    }

    return false;
}

/*!
    \fn CqOpenGLwidget::loadHrleObjectFromVoxelObject()
 */
// create a HRLE object from a voxel object
bool CqOpenGLwidget::loadHrleObjectFromVoxelObject()
{
    if ( this->bHrleObjectStatus )
        delete this->myHRLE;

    this->myHRLE = new CHierarchicalRLE( 3 );
    this->myHRLE->convert_to_hrle( myVoxelObject );
    this->bHrleObjectStatus = true;

    cout << "oooooooooooooooooooooooooooooooooooooooooooooo H-RLE object created oooooooooooooooooooooooooooooooooooooo" << endl;
    this->myHRLE->printDataStructure();
    cout << "oooooooooooooooooooooooooooooooooooooooooooooo oooooooooooooooooooo oooooooooooooooooooooooooooooooooooooo" << endl;

    this->bHrleObjectChanged = true;

    this->dynListsGL();

    return true;
}

/*!
    \fn CqOpenGLwidget::dilateHRLEObject( int dilationPoints )
 */
// old dilation algorithm
bool CqOpenGLwidget::dilateHRLEObject( vector<bool> structureElement ) {

    CHierarchicalRLE* tmpHRLE = myHRLE->getDilatedHRLE( structureElement );

// cout << "oooooooooooooooooooooooooooooooooooooooooooooo after Dilation oooooooooooooooooooooooooooooooooooooo" << endl;
// tmpHRLE->printDataStructure();
// cout << "oooooooooooooooooooooooooooooooooooooooooooooo oooooooooooooo oooooooooooooooooooooooooooooooooooooo" << endl;

    delete myHRLE;
    myHRLE = tmpHRLE;

    this->bHrleObjectChanged = true;

    this->dynListsGL();

    return true;
}

bool CqOpenGLwidget::dilateHRLEObject2( vector<bool> structureElement ) {

    CHierarchicalRLE* se/* = new CHierarchicalRLE( 3 )*/;
    vector<short int> middlePoint = this->encodeStructElement( structureElement, &se );

// 7x7x7 structure element
// se = new CHierarchicalRLE( 3 );
// se->min = 0;
// se->max = 7;
// se->lower_hrle->min = 0;
// se->lower_hrle->max = 7;
// se->lower_hrle->lower_hrle->min = 0;
// se->lower_hrle->lower_hrle->max = 7;
// 
// se->start_indices.push_back(0);
// se->run_types.push_back(0);
// 
// se->lower_hrle->start_indices.push_back(0);
// se->lower_hrle->start_indices.push_back(1);
// se->lower_hrle->start_indices.push_back(2);
// se->lower_hrle->start_indices.push_back(3);
// se->lower_hrle->start_indices.push_back(4);
// se->lower_hrle->start_indices.push_back(5);
// se->lower_hrle->start_indices.push_back(6);
// se->lower_hrle->run_types.push_back(0);
// se->lower_hrle->run_types.push_back(7);
// se->lower_hrle->run_types.push_back(14);
// se->lower_hrle->run_types.push_back(21);
// se->lower_hrle->run_types.push_back(28);
// se->lower_hrle->run_types.push_back(35);
// se->lower_hrle->run_types.push_back(42);
// 
// se->lower_hrle->lower_hrle->start_indices.push_back(0);
// se->lower_hrle->lower_hrle->start_indices.push_back(1);
// se->lower_hrle->lower_hrle->start_indices.push_back(2);
// se->lower_hrle->lower_hrle->start_indices.push_back(3);
// se->lower_hrle->lower_hrle->start_indices.push_back(4);
// se->lower_hrle->lower_hrle->start_indices.push_back(5);
// se->lower_hrle->lower_hrle->start_indices.push_back(6);
// 
// se->lower_hrle->lower_hrle->start_indices.push_back(7);
// se->lower_hrle->lower_hrle->start_indices.push_back(8);
// se->lower_hrle->lower_hrle->start_indices.push_back(9);
// se->lower_hrle->lower_hrle->start_indices.push_back(10);
// se->lower_hrle->lower_hrle->start_indices.push_back(11);
// se->lower_hrle->lower_hrle->start_indices.push_back(12);
// se->lower_hrle->lower_hrle->start_indices.push_back(13);
// 
// se->lower_hrle->lower_hrle->start_indices.push_back(14);
// se->lower_hrle->lower_hrle->start_indices.push_back(15);
// se->lower_hrle->lower_hrle->start_indices.push_back(16);
// se->lower_hrle->lower_hrle->start_indices.push_back(17);
// se->lower_hrle->lower_hrle->start_indices.push_back(18);
// se->lower_hrle->lower_hrle->start_indices.push_back(19);
// se->lower_hrle->lower_hrle->start_indices.push_back(20);
// 
// se->lower_hrle->lower_hrle->start_indices.push_back(21);
// se->lower_hrle->lower_hrle->start_indices.push_back(22);
// se->lower_hrle->lower_hrle->start_indices.push_back(23);
// se->lower_hrle->lower_hrle->start_indices.push_back(24);
// se->lower_hrle->lower_hrle->start_indices.push_back(25);
// se->lower_hrle->lower_hrle->start_indices.push_back(26);
// se->lower_hrle->lower_hrle->start_indices.push_back(27);
// 
// se->lower_hrle->lower_hrle->start_indices.push_back(28);
// se->lower_hrle->lower_hrle->start_indices.push_back(29);
// se->lower_hrle->lower_hrle->start_indices.push_back(30);
// se->lower_hrle->lower_hrle->start_indices.push_back(31);
// se->lower_hrle->lower_hrle->start_indices.push_back(32);
// se->lower_hrle->lower_hrle->start_indices.push_back(33);
// se->lower_hrle->lower_hrle->start_indices.push_back(34);
// 
// se->lower_hrle->lower_hrle->start_indices.push_back(35);
// se->lower_hrle->lower_hrle->start_indices.push_back(36);
// se->lower_hrle->lower_hrle->start_indices.push_back(37);
// se->lower_hrle->lower_hrle->start_indices.push_back(38);
// se->lower_hrle->lower_hrle->start_indices.push_back(39);
// se->lower_hrle->lower_hrle->start_indices.push_back(40);
// se->lower_hrle->lower_hrle->start_indices.push_back(41);
// 
// se->lower_hrle->lower_hrle->start_indices.push_back(42);
// se->lower_hrle->lower_hrle->start_indices.push_back(43);
// se->lower_hrle->lower_hrle->start_indices.push_back(44);
// se->lower_hrle->lower_hrle->start_indices.push_back(45);
// se->lower_hrle->lower_hrle->start_indices.push_back(46);
// se->lower_hrle->lower_hrle->start_indices.push_back(47);
// se->lower_hrle->lower_hrle->start_indices.push_back(48);
// 
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// 
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// 
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// 
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// 
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// 
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// 
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// se->lower_hrle->lower_hrle->run_types.push_back(0);
// 
// se->printDataStructure();
// 
// vector<short int> middlePoint;
// middlePoint.push_back(3);
// middlePoint.push_back(3);
// middlePoint.push_back(3);

    CHierarchicalRLE* tmpHRLE = myHRLE->getDilatedHRLE( se, middlePoint );

// cout << "oooooooooooooooooooooooooooooooooooooooooooooo after Dilation oooooooooooooooooooooooooooooooooooooo" << endl;
// tmpHRLE->printDataStructure();
// cout << "oooooooooooooooooooooooooooooooooooooooooooooo oooooooooooooooooooooooooooooooooooooooooooooooooooo" << endl;

    delete myHRLE;
    myHRLE = tmpHRLE;

    this->bHrleObjectChanged = true;

    this->dynListsGL();

    return true;
}

bool CqOpenGLwidget::erodeHRLEObject( vector<bool> structureElement ) {

    CHierarchicalRLE* se/* = new CHierarchicalRLE( 3 )*/;
    vector<short int> middlePoint = this->encodeStructElement( structureElement, &se );

    CHierarchicalRLE* tmpHRLE = myHRLE->getErodedHRLE( se, middlePoint );

// cout << "oooooooooooooooooooooooooooooooooooooooooooooo after Erosion  oooooooooooooooooooooooooooooooooooooo" << endl;
// tmpHRLE->printDataStructure();
// cout << "oooooooooooooooooooooooooooooooooooooooooooooo oooooooooooooo oooooooooooooooooooooooooooooooooooooo" << endl;

    delete myHRLE;
    myHRLE = tmpHRLE;

    this->bHrleObjectChanged = true;

    this->dynListsGL();

    return true;
}

bool CqOpenGLwidget::loadHRLEB( QString directory ) {

    if ( this->myHRLEB != NULL ) delete this->myHRLEB;

    this->myHRLEB = new CAdvancedHRLE();

    this->bHrleBObjectChanged = true;

    return this->myHRLEB->loadObjectFile( directory.toStdString() );

}

bool CqOpenGLwidget::drawHRLEB() {

    if ( this->bHrleBObjectStatus ) {

        this->myHRLEB->loadCompleteData();

        if ( this->bHrleBObjectChanged ) {

            cout << "Drawing the object." << endl;
            this->dynListsGL();

        }

        this->updateGL();

        return true;

    }
    else cout << "There's no object loaded!" << endl;

    return false;
}

bool CqOpenGLwidget::dilateHRLEB( vector<bool> structureElement ) {

    CHierarchicalRLE*   se = NULL;
    vector<short int>   qv = this->encodeStructElement( structureElement, &se );

    if ( myHRLEB->morphOperation( se, qv, 10, DILATION ) ) {

        this->bHrleBObjectChanged = true;

        this->dynListsGL();

        return true;
    }

    return false;
}

bool CqOpenGLwidget::erodeHRLEB( vector<bool> structureElement ) {

    CHierarchicalRLE*   se = NULL;
    vector<short int>   qv = this->encodeStructElement( structureElement, &se );

    if ( myHRLEB->morphOperation( se, qv, 10, EROSION ) ) {

        this->bHrleBObjectChanged = true;

        this->dynListsGL();

        return true;
    }

    return false;
}

bool CqOpenGLwidget::closeHRLEB( vector<bool> structureElement ) {

    CHierarchicalRLE*   se = NULL;
    vector<short int>   qv = this->encodeStructElement( structureElement, &se );

    if ( myHRLEB->morphOperation( se, qv, 10, CLOSING ) ) {

        this->bHrleBObjectChanged = true;

        this->dynListsGL();

        return true;
    }

    return false;
}

bool CqOpenGLwidget::openHRLEB( vector<bool> structureElement ) {

    CHierarchicalRLE*   se = NULL;
    vector<short int>   qv = this->encodeStructElement( structureElement, &se );

    if ( myHRLEB->morphOperation( se, qv, 10, OPENING ) ) {

        this->bHrleBObjectChanged = true;

        this->dynListsGL();

        return true;
    }

    return false;
}

bool CqOpenGLwidget::medialAxisHRLEB() {

    if ( myHRLEB->medialAxis() ) {

        this->bHrleBObjectChanged = true;

        this->dynListsGL();

        return true;
    }

    return false;
}

bool CqOpenGLwidget::medialSurfaceHRLEObject() {

    if ( this->bHrleObjectStatus == false ) return false;

    if ( myHRLE->computeMedialSurface() ) {

// cout << " ****************************************** The medial surface *********************************** " << endl;
// myHRLE->printDataStructure();
// cout << " ****************************************** __________________ *********************************** " << endl;

        this->bHrleObjectChanged = true;

        this->dynListsGL();

        return true;

    }

    return false;
}

bool CqOpenGLwidget::medialAxisHRLEObject() {

    if ( this->bHrleObjectStatus == false ) return false;

    if ( myHRLE->computeMedialAxis() ) {

        this->bHrleObjectChanged = true;

        this->dynListsGL();

        return true;

    }

    return false;
}

vector<short int> CqOpenGLwidget::encodeStructElement( vector<bool> structureElement, CHierarchicalRLE** se )
{
    *se = new CHierarchicalRLE( 3 );

    vector<bool>    yPlane, zPlane;
    int             i;
    short int       coord, si;
    bool            defCoords = false;

    (*se)->min = 0;
    (*se)->max = 3;
    (*se)->lower_hrle->min = 0;
    (*se)->lower_hrle->max = 3;
    (*se)->lower_hrle->lower_hrle->min = 0;
    (*se)->lower_hrle->lower_hrle->max = 3;

    for ( i = 0; i < (structureElement.size()+1); i++ ) {

// for ( int j = 0; j < (*se)->lower_hrle->lower_hrle->run_types.size(); j++ )
// cout << (*se)->lower_hrle->lower_hrle->run_types.at(j) << " | ";
// cout << endl;

        if ( i == 0 ) coord = 0;
        else coord = i % 3;

        if ( coord == 0) {

            if (defCoords) {
                yPlane.push_back( true );
            }
            else {
                if ((*se)->lower_hrle->lower_hrle->run_types.size() > 0 )
                    (*se)->lower_hrle->lower_hrle->run_types.pop_back();

                if ( i > 0 ) {
                    (*se)->lower_hrle->lower_hrle->start_indices.pop_back();
                    yPlane.push_back( false );
                }
            }

            (*se)->lower_hrle->lower_hrle->start_indices.push_back((*se)->lower_hrle->lower_hrle->run_types.size());

            defCoords = false;
        }

        if ( i < structureElement.size() ) {
            if ( structureElement.at(i) ) {
                defCoords = true;
                if ( (coord == 0) || !RT_DEFINED((*se)->lower_hrle->lower_hrle->run_types.back()) ) {
                      (*se)->lower_hrle->lower_hrle->run_types.push_back(1);
                      if ( (coord != 0) ) (*se)->lower_hrle->lower_hrle->run_breaks.push_back( coord );
                }
            }
            else {
                if ( (coord == 0) || RT_DEFINED((*se)->lower_hrle->lower_hrle->run_types.back()) ) {
                      (*se)->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
                      if ( coord != 0 ) (*se)->lower_hrle->lower_hrle->run_breaks.push_back( coord );
                }
            }
        }
    }
    if ( (*se)->lower_hrle->lower_hrle->start_indices.back() >= (*se)->lower_hrle->lower_hrle->run_types.size() )
        (*se)->lower_hrle->lower_hrle->start_indices.pop_back();

    si = 0;
    for ( i = 0; i < (yPlane.size()+1); i++ ) {
        if ( i == 0 ) coord = 0;
        else coord = i % 3;

        if ( coord == 0) {

            (*se)->lower_hrle->start_indices.push_back((*se)->lower_hrle->run_types.size());

            if (defCoords) {
                zPlane.push_back( true );
            }
            else {
                if ((*se)->lower_hrle->run_types.size() > 0 )
                    (*se)->lower_hrle->run_types.pop_back();

                if ( i > 0 ) {
                    (*se)->lower_hrle->start_indices.pop_back();
                    zPlane.push_back( false );
                }
            }

            defCoords = false;
        }

        if ( i < yPlane.size() ) {
            if ( yPlane.at(i) ) {
                defCoords = true;
                if ( (coord == 0) || !RT_DEFINED((*se)->lower_hrle->run_types.back()) ) {
                      (*se)->lower_hrle->run_types.push_back(si);
                      if ( (coord != 0) ) (*se)->lower_hrle->run_breaks.push_back( coord );
                      si++;
                }
            }
            else {
                if ( (coord == 0) || RT_DEFINED((*se)->lower_hrle->run_types.back()) ) {
                      (*se)->lower_hrle->run_types.push_back( RT_POSITIVE );
                      if ( coord != 0 ) (*se)->lower_hrle->run_breaks.push_back( coord );
                }
            }
        }
    }
    if ( (*se)->lower_hrle->start_indices.back() >= (*se)->lower_hrle->run_types.size() )
        (*se)->lower_hrle->start_indices.pop_back();

    si = 0;
    (*se)->start_indices.push_back(0);
    for ( i = 0; i < zPlane.size(); i++ ) {
        if ( i == 0 ) coord = 0;
        else coord = i % 3;

        if ( zPlane.at(i) ) {
            if ( (coord == 0) || !RT_DEFINED((*se)->run_types.back()) ) {
                  (*se)->run_types.push_back(si);
                  if ( (coord != 0) ) (*se)->run_breaks.push_back( coord );
                  si++;
            }
        }
        else {
            if ( (coord == 0) || RT_DEFINED((*se)->run_types.back()) ) {
                  (*se)->run_types.push_back( RT_POSITIVE );
                  if ( coord != 0 ) (*se)->run_breaks.push_back( coord );
            }
        }
    }

cout << "oooooooooooooooooooooooooooooooooooooooooo structure element oooooooooooooooooooooooooooooooooooooo" << endl;
(*se)->printDataStructure();
cout << "ooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo" << endl;

    vector<short int> middlePoint;
    middlePoint.push_back( 1 );
    middlePoint.push_back( 1 );
    middlePoint.push_back( 1 );

    return middlePoint;
}

// for testing
void CqOpenGLwidget::createTestingHRLE() {

//     short int newRun;

    myHRLE = new CHierarchicalRLE( 3 );

    // dimension 3
    myHRLE->minRange = CVertex3D(0,0,0);
    myHRLE->maxRange = CVertex3D(2.5,4.5,8);
    myHRLE->min = 0;
    myHRLE->max = 16;
//     myHRLE->max = 8;

    myHRLE->run_types.push_back( RT_POSITIVE );
    myHRLE->run_types.push_back( 0 );
    myHRLE->run_types.push_back( RT_POSITIVE );

    myHRLE->run_breaks.push_back( 1 );
    myHRLE->run_breaks.push_back( 15 );

    myHRLE->start_indices.push_back( 0 );

    // dimension 2
    myHRLE->lower_hrle->min = 0;
    myHRLE->lower_hrle->max = 10;

    // 1. segment
    myHRLE->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->run_types.push_back( 0 );
    myHRLE->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 2. segment
    myHRLE->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->run_types.push_back( 3 );
    myHRLE->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 3. segment
    myHRLE->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->run_types.push_back( 6 );
    myHRLE->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 4. segment
    myHRLE->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->run_types.push_back( 9 );
    myHRLE->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 5. segment
    myHRLE->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->run_types.push_back( 12 );
    myHRLE->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 6. segment
    myHRLE->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->run_types.push_back( 15 );
    myHRLE->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 7. segment
    myHRLE->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->run_types.push_back( 18 );
    myHRLE->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 8. segment
    myHRLE->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->run_types.push_back( 26 );
    myHRLE->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 9. segment
    myHRLE->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->run_types.push_back( 34 );
    myHRLE->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 10. segment
    myHRLE->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->run_types.push_back( 42 );
    myHRLE->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 11. segment
    myHRLE->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->run_types.push_back( 45 );
    myHRLE->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 12. segment
    myHRLE->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->run_types.push_back( 48 );
    myHRLE->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 13. segment
    myHRLE->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->run_types.push_back( 51 );
    myHRLE->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 14. segment
    myHRLE->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->run_types.push_back( 54 );
    myHRLE->lower_hrle->run_types.push_back( RT_POSITIVE );

    myHRLE->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->run_breaks.push_back( 4 );
    myHRLE->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->run_breaks.push_back( 4 );
    myHRLE->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->run_breaks.push_back( 4 );
    myHRLE->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->run_breaks.push_back( 4 );
    myHRLE->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->run_breaks.push_back( 4 );
    myHRLE->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->run_breaks.push_back( 4 );

    myHRLE->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->run_breaks.push_back( 9 );
    myHRLE->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->run_breaks.push_back( 9 );
    myHRLE->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->run_breaks.push_back( 9 );

    myHRLE->lower_hrle->run_breaks.push_back( 6 );
    myHRLE->lower_hrle->run_breaks.push_back( 9 );
    myHRLE->lower_hrle->run_breaks.push_back( 6 );
    myHRLE->lower_hrle->run_breaks.push_back( 9 );
    myHRLE->lower_hrle->run_breaks.push_back( 6 );
    myHRLE->lower_hrle->run_breaks.push_back( 9 );
    myHRLE->lower_hrle->run_breaks.push_back( 6 );
    myHRLE->lower_hrle->run_breaks.push_back( 9 );
    myHRLE->lower_hrle->run_breaks.push_back( 6 );
    myHRLE->lower_hrle->run_breaks.push_back( 9 );

    myHRLE->lower_hrle->start_indices.push_back( 0 );
    myHRLE->lower_hrle->start_indices.push_back( 3 );
    myHRLE->lower_hrle->start_indices.push_back( 6 );
    myHRLE->lower_hrle->start_indices.push_back( 9 );
    myHRLE->lower_hrle->start_indices.push_back( 12 );
    myHRLE->lower_hrle->start_indices.push_back( 15 );
    myHRLE->lower_hrle->start_indices.push_back( 18 );
    myHRLE->lower_hrle->start_indices.push_back( 21 );
    myHRLE->lower_hrle->start_indices.push_back( 24 );
    myHRLE->lower_hrle->start_indices.push_back( 27 );
    myHRLE->lower_hrle->start_indices.push_back( 30 );
    myHRLE->lower_hrle->start_indices.push_back( 33 );
    myHRLE->lower_hrle->start_indices.push_back( 36 );
    myHRLE->lower_hrle->start_indices.push_back( 39 );

    // dimension 1
    myHRLE->lower_hrle->lower_hrle->min = 0;
    myHRLE->lower_hrle->lower_hrle->max = 6;

    // 1. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 2. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 3. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 4. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 5. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 6. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 7. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 8. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 9. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 10. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 11. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 12. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 13. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 14. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 15. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 16. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 17. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 18. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 19. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 20. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 21. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 22. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 23. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 24. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 25. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 26. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 27. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 28. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 29. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 30. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 31. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 32. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 33. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 34. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 35. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 36. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 37. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 38. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 39. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 40. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 41. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 42. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 43. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 44. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 45. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 46. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 47. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 48. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 49. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 50. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 51. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 52. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 53. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 54. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 55. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 56. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    // 57. segment
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );

    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );

    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );

    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );

    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );

    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );

    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );

    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );

    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );

    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );

    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );

    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );

    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );

    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );

    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
    myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 5 );

    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 0 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 3 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 6 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 9 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 12 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 15 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 18 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 21 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 24 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 27 );

    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 30 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 33 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 36 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 39 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 42 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 45 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 48 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 51 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 54 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 57 );

    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 60 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 63 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 66 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 69 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 72 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 75 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 78 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 81 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 84 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 87 );

    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 90 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 93 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 96 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 99 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 102 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 105 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 108 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 111 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 114 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 117 );

    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 120 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 123 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 126 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 129 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 132 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 135 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 138 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 141 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 144 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 147 );

    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 150 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 153 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 156 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 159 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 162 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 165 );
    myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 168 );

    this->bHrleObjectStatus = true;

    this->myHRLE->printDataStructure();

    this->bHrleObjectChanged = true;

//     this->myHRLE->printNeighborOfGridPoint( 1, 2, 2 , 0 );

    this->dynListsGL();
}
// void CqOpenGLwidget::createTestingHRLE() {
//
// //     short int newRun;
// 
//     myHRLE = new CHierarchicalRLE( 3 );
// 
//     // dimension 3
//     myHRLE->minRange = CVertex3D(0,0,0);
//     myHRLE->maxRange = CVertex3D(5,5,5);
//     myHRLE->min = 0;
//     myHRLE->max = 7;
// //     myHRLE->max = 8;
// 
//     myHRLE->run_types.push_back( RT_POSITIVE );
//     myHRLE->run_types.push_back( 0 );
// //     myHRLE->run_types.push_back( RT_POSITIVE );
// 
//     myHRLE->run_breaks.push_back( 1 );
// //     myHRLE->run_breaks.push_back( 7 );
// 
//     myHRLE->start_indices.push_back( 0 );
// 
//     // dimension 2
//     myHRLE->lower_hrle->min = 0;
//     myHRLE->lower_hrle->max = 7;
// 
//     // 1. segment
//     myHRLE->lower_hrle->run_types.push_back( RT_POSITIVE );
//     myHRLE->lower_hrle->run_types.push_back( 0 );
//     myHRLE->lower_hrle->run_types.push_back( RT_POSITIVE );
//     myHRLE->lower_hrle->run_types.push_back( 1 );
//     myHRLE->lower_hrle->run_types.push_back( RT_POSITIVE );
// 
//     // 2. segment
//     myHRLE->lower_hrle->run_types.push_back( RT_POSITIVE );
//     myHRLE->lower_hrle->run_types.push_back( 2 );
//     myHRLE->lower_hrle->run_types.push_back( RT_POSITIVE );
// 
//     // 3. segment
//     myHRLE->lower_hrle->run_types.push_back( RT_POSITIVE );
//     myHRLE->lower_hrle->run_types.push_back( 6 );
//     myHRLE->lower_hrle->run_types.push_back( RT_POSITIVE );
// 
//     // 4. segment
//     myHRLE->lower_hrle->run_types.push_back( RT_POSITIVE );
//     myHRLE->lower_hrle->run_types.push_back( 10 );
//     myHRLE->lower_hrle->run_types.push_back( RT_POSITIVE );
// 
//     // 5. segment
//     myHRLE->lower_hrle->run_types.push_back( RT_POSITIVE );
//     myHRLE->lower_hrle->run_types.push_back( 15 );
//     myHRLE->lower_hrle->run_types.push_back( RT_POSITIVE );
// 
//     // 6. segment
//     myHRLE->lower_hrle->run_types.push_back( RT_POSITIVE );
//     myHRLE->lower_hrle->run_types.push_back( 19 );
//     myHRLE->lower_hrle->run_types.push_back( RT_POSITIVE );
// 
//     myHRLE->lower_hrle->run_breaks.push_back( 2 );
//     myHRLE->lower_hrle->run_breaks.push_back( 3 );
//     myHRLE->lower_hrle->run_breaks.push_back( 4 );
//     myHRLE->lower_hrle->run_breaks.push_back( 5 );
//     myHRLE->lower_hrle->run_breaks.push_back( 1 );
//     myHRLE->lower_hrle->run_breaks.push_back( 5 );
//     myHRLE->lower_hrle->run_breaks.push_back( 2 );
//     myHRLE->lower_hrle->run_breaks.push_back( 6 );
//     myHRLE->lower_hrle->run_breaks.push_back( 1 );
//     myHRLE->lower_hrle->run_breaks.push_back( 6 );
//     myHRLE->lower_hrle->run_breaks.push_back( 2 );
//     myHRLE->lower_hrle->run_breaks.push_back( 6 );
//     myHRLE->lower_hrle->run_breaks.push_back( 2 );
//     myHRLE->lower_hrle->run_breaks.push_back( 5 );
// 
//     myHRLE->lower_hrle->start_indices.push_back( 0 );
//     myHRLE->lower_hrle->start_indices.push_back( 5 );
//     myHRLE->lower_hrle->start_indices.push_back( 8 );
//     myHRLE->lower_hrle->start_indices.push_back( 11 );
//     myHRLE->lower_hrle->start_indices.push_back( 14 );
//     myHRLE->lower_hrle->start_indices.push_back( 17 );
// 
//     // dimension 1
//     myHRLE->lower_hrle->lower_hrle->min = 0;
//     myHRLE->lower_hrle->lower_hrle->max = 7;
// 
//     // 1. segment
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
// 
//     // 2. segment
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
// 
//     // 3. segment
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
// 
//     // 4. segment
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
// 
//     // 5. segment
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
// 
//     // 6. segment
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
// 
//     // 7. segment
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
// 
//     // 8. segment
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
// 
//     // 9. segment
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
// 
//     // 10. segment
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
// 
//     // 11. segment
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
// 
//     // 12. segment
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
// 
//     // 13. segment
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
// //     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
// //     myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
// 
//     // 14. segment
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
// 
//     // 15. segment
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
// 
//     // 16. segment
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
// 
//     // 17. segment
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
// 
//     // 18. segment
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
// 
//     // 19. segment
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
// 
//     // 20. segment
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
// 
//     // 21. segment
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
// 
//     // 22. segment
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( 1 );
//     myHRLE->lower_hrle->lower_hrle->run_types.push_back( RT_POSITIVE );
// 
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 2 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 2 );
// 
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 2 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 2 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 2 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 3 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 4 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 2 );
// 
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 4 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 6 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 4 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 3 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 4 );
// 
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 2 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 4 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
// //     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 2 );
// //     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 3 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 4 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 4 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 2 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 3 );
// 
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 4 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 4 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 4 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 3 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 4 );
// 
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 2 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 3 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 4 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 1 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 2 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 3 );
//     myHRLE->lower_hrle->lower_hrle->run_breaks.push_back( 4 );
// 
//     myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 0 );
//     myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 3 );
//     myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 6 );
//     myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 9 );
//     myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 12 );
//     myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 17 );
//     myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 20 );
//     myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 23 );
//     myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 26 );
//     myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 29 );
//     myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 32 );
//     myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 35 );
//     myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 38 );
//     myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 41 );
//     myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 44 );
//     myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 47 );
//     myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 50 );
//     myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 53 );
//     myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 56 );
//     myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 59 );
//     myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 62 );
//     myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 65 );
// //     myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 43 );
// //     myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 46 );
// //     myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 49 );
// //     myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 52 );
// //     myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 55 );
// //     myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 58 );
// //     myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 61 );
// //     myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 64 );
// //     myHRLE->lower_hrle->lower_hrle->start_indices.push_back( 67 );
// 
//     this->bHrleObjectStatus = true;
// 
//     this->myHRLE->printDataStructure();
// 
//     this->bHrleObjectChanged = true;
// 
// //     this->myHRLE->printNeighborOfGridPoint( 1, 2, 2 , 0 );
// 
//     this->dynListsGL();
// }

/////////////////////////////////////////////////////////////////////////////////////////
//          get and set methodes
/////////////////////////////////////////////////////////////////////////////////////////

bool CqOpenGLwidget::getColorMaterialStatus() const
{
  return bColorMaterial;
}


void CqOpenGLwidget::setColorMaterialStatus(bool theValue)
{
  bColorMaterial = theValue;
}


bool CqOpenGLwidget::getCoordinatesStatus() const
{
  return bCoordinates;
}


void CqOpenGLwidget::setCoordinatesStatus(bool theValue)
{
  bCoordinates = theValue;
}


bool CqOpenGLwidget::getCullBackFaceStatus() const
{
  return bCullBackFace;
}


void CqOpenGLwidget::setCullBackFaceStatus(bool theValue)
{
  bCullBackFace = theValue;
}


bool CqOpenGLwidget::getLightingStatus() const
{
  return bLighting;
}


void CqOpenGLwidget::setLightingStatus(bool theValue)
{
  bLighting = theValue;
}


bool CqOpenGLwidget::getTransparencyStatus() const
{
  return bTransparency;
}


void CqOpenGLwidget::setTransparencyStatus(bool theValue)
{
  bTransparency = theValue;
}


int CqOpenGLwidget::getCameraMode() const
{
  return iCameraMode;
}


void CqOpenGLwidget::setCameraMode(const int& theValue)
{
  iCameraMode = theValue;
}


bool CqOpenGLwidget::getWireFrameStatus() const
{
  return bWireFrame;
}


void CqOpenGLwidget::setWireFrameStatus(bool theValue)
{
  bWireFrame = theValue;
}


bool CqOpenGLwidget::getSmfObjectStatus() const
{
  return bSmfObjectStatus;
}


void CqOpenGLwidget::setSmfObjectStatus(bool theValue)
{
  bSmfObjectStatus = theValue;
}


bool CqOpenGLwidget::getVoxelObjectStatus() const
{
  return bVoxelObjectStatus;
}


void CqOpenGLwidget::setVoxelObjectStatus(bool theValue)
{
  bVoxelObjectStatus = theValue;
}


bool CqOpenGLwidget::getHrleObjectStatus() const
{
  return bHrleObjectStatus;
}


void CqOpenGLwidget::setHrleObjectStatus(bool theValue)
{
  bHrleObjectStatus = theValue;
}

bool CqOpenGLwidget::getDrawVoxelObject() const
{
  return bDrawVoxelObject;
}

void CqOpenGLwidget::setDrawVoxelObject(bool theValue)
{
  bDrawVoxelObject = theValue;
}

bool CqOpenGLwidget::getDrawSmfObject() const
{
  return bDrawSmfObject;
}

void CqOpenGLwidget::setDrawSmfObject(bool theValue)
{
  bDrawSmfObject = theValue;
}

bool CqOpenGLwidget::getDrawHrleSeq() const
{
  return bDrawHrleSeq;
}

void CqOpenGLwidget::setDrawHrleSeq(bool theValue)
{
  bDrawHrleSeq = theValue;
}

bool CqOpenGLwidget::getDrawHrleRand() const
{
  return bDrawHrleRand;
}

void CqOpenGLwidget::setDrawHrleRand(bool theValue)
{
  bDrawHrleRand = theValue;
}

double CqOpenGLwidget::getMemoryUsageHRLE()
{
//     if ( this->bHrleObjectStatus )
//         return this->myHRLE->getMemorySize();

    return 0.0;
}


double CqOpenGLwidget::getAccessTimeRA() {

    if ( this->bHrleObjectStatus )
        return this->myHRLE->accessTimeRandomAccess;

    return 0.0;

}

double CqOpenGLwidget::getAccessAverageTimeRA() {

    if ( this->bHrleObjectStatus )
        return this->myHRLE->accessAverageTimeRandomAccess;

    return 0.0;

}

double CqOpenGLwidget::getAccessTimeSA() {

    if ( this->bHrleObjectStatus )
        return this->myHRLE->accessTimeSequentialAccess;

    return 0.0;

}

double CqOpenGLwidget::getMemoryUsageVoxelobject()
{
    if ( this->bVoxelObjectStatus )
        return this->myVoxelObject->getMemorySize();

    return 0.0;
}

double CqOpenGLwidget::getAccessTimeRA_voxelobject() {

    if ( this->bVoxelObjectStatus )
        return this->myVoxelObject->accessTimeRandomAccess;

    return 0.0;

}

double CqOpenGLwidget::getAccessAverageTimeRA_voxelobject() {

    if ( this->bVoxelObjectStatus )
        return this->myVoxelObject->accessAverageTimeRandomAccess;

    return 0.0;

}

double CqOpenGLwidget::getAccessTimeSA_voxelobject() {

    if ( this->bVoxelObjectStatus )
        return this->myVoxelObject->accessTimeSequentialAccess;

    return 0.0;

}

void CqOpenGLwidget::setHrleBObjectStatus(bool theValue) {
    this->bHrleBObjectStatus = theValue;
}

bool CqOpenGLwidget::getHrleBObjectStatus() const {
    return this->bHrleBObjectStatus;
}

void CqOpenGLwidget::setDrawHrleB(bool theValue) {
    this->bDrawHrleB = theValue;
}

bool CqOpenGLwidget::getDrawHrleB() const {
    return this->bDrawHrleB;
}

int CqOpenGLwidget::getNumDefinedVoxels() {
    if ( this->bVoxelObjectStatus )
        return this->myVoxelObject->numDefVoxels;

    return 0;
}

CPoint3D CqOpenGLwidget::getVoxelobjectResolution() {
    if ( this->bVoxelObjectStatus ) {
        return CPoint3D(
            this->myVoxelObject->m_ScopeX,
            this->myVoxelObject->m_ScopeY,
            this->myVoxelObject->m_ScopeZ );
    }

    return CPoint3D(0,0,0);
}

