/***************************************************************************
 *   Copyright (C) 2005 Chemnitz University of Technology                  *
 *   Copyright (C) 2006 Holger Gerth                                       *
 *   holger.gerth@informatik.tu-chemnitz.de                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "voxelobject3d.h"

#include "moments.h"
#include "hierarchicalrle.h"

CVoxelObject3D::CVoxelObject3D() {

    // set scopes
    this->m_ScopeX = 0;
    this->m_ScopeY = 0;
    this->m_ScopeZ = 0;

    // set center
    this->m_CenterX = 0;
    this->m_CenterY = 0;
    this->m_CenterZ = 0;

    // set centroid
    this->m_CentroidX = -1;
    this->m_CentroidY = -1;
    this->m_CentroidZ = -1;

    // set standard cube color
    this->m_StandardCubeColor = CColor4f(0.9f, 0.9f, 0.9f);

    // set file import and export
//      FileImport.SetParentVoxelObject(this);
//      FileExport.SetParentVoxelObject(this);

    // set drawing options
    this->drawCubes = true;
    this->drawPrincipalAxes = true;

    // free arrays
    this->m_CubeArray.free();
    this->m_CubeColorArray.free();
    this->m_DistanceMapArray.free();

//     this->m_pAxisX = CVector3D();
//     this->m_pAxisY = CVector3D();
//     this->m_pAxisZ = CVector3D();

    this->numDefVoxels = 0;
    this->accessAverageTimeRandomAccess = 0.0;
    this->accessTimeRandomAccess = 0.0;
    this->accessTimeSequentialAccess = 0.0;

}

CVoxelObject3D::~CVoxelObject3D() {
}

//////////////////////////////////////////////////////////////////////
// Drawing
//////////////////////////////////////////////////////////////////////

void CVoxelObject3D::Draw() {

    long            numGridPoints = 0, accessTime = 0, totalAccessTime = 0, averageAccessTime = 0;
//     struct timeval  tp1,tp2;            // process cpu time in seconds (tv_sec) and micro seconds (tv_usec)

    // extensions of a cube
    float CubeW = (m_MaxRange.getX() - m_MinRange.getX()) / (float)m_ScopeX;
    float CubeH = (m_MaxRange.getY() - m_MinRange.getY()) / (float)m_ScopeY;
    float CubeD = (m_MaxRange.getZ() - m_MinRange.getZ()) / (float)m_ScopeZ;

//     cout << "Scopes: " << m_ScopeX << ", " << m_ScopeY << ", " << m_ScopeZ << endl;

    // draw cubes and cube edges
    // -------------------------

    if (this->drawCubes) {

        for (int w = 0; w < m_ScopeX; w++) {
            for (int h = 0; h < m_ScopeY; h++) {
                for (int d = 0; d < m_ScopeZ; d++) {

                    // check if cube is empty
                    //if (m_CubeArray(w,h,d).flag == CT_EMPTY) continue;

                    // check if cube is border cube
                    //if (m_CubeArray(w,h,d).flag != CT_BORDER) continue;

                    CVertex3D point1( m_MinRange.getX() + w * CubeW,
                                      m_MinRange.getY() + h * CubeH,
                                      m_MinRange.getZ() + d * CubeD );
                    CVertex3D point2( m_MinRange.getX() + (w+1) * CubeW,
                                      m_MinRange.getY() + (h+1) * CubeH,
                                      m_MinRange.getZ() + (d+1) * CubeD);

//                     gettimeofday( &tp1, NULL );

                    // check if cube color array is set
                    if (w < m_CubeColorArray.width() && h < m_CubeColorArray.height() && d < m_CubeColorArray.depth()) {
                        accessTime = m_CubeArray(w,h,d).Draw(&point1, &point2, &m_CubeColorArray(w,h,d));
                    }
                    else {
                        accessTime = m_CubeArray(w,h,d).Draw(&point1, &point2, &m_StandardCubeColor);
                    }

//                     gettimeofday( &tp2, NULL );

                    numGridPoints++;
//                     accessTime = (tp2.tv_sec -tp1.tv_sec) * 1000000 + (tp2.tv_usec - tp1.tv_usec);
                    totalAccessTime += accessTime;

                }
            }
        }
    }

    averageAccessTime = totalAccessTime / numGridPoints;

    cout << "Total time while accessing the voxel object was " << totalAccessTime << " micro seconds and the average time";
    cout << " for one element was " << averageAccessTime << " micro seconds." << endl;

    cout << "Voxelobject Memory Usage: " << this->m_ScopeX * this->m_ScopeY * this->m_ScopeZ * sizeof(CCube) / 1024 << " kbytes of system memory." << endl;

    this->accessTimeRandomAccess = (double) accessTime;
    this->accessAverageTimeRandomAccess = (double) totalAccessTime / (double) numGridPoints;
    this->accessTimeSequentialAccess = (double) totalAccessTime;


    // draw principal axes
    // -------------------

//     if (this->drawPrincipalAxes) {
// 
// 
//         float w = this->m_CentroidX + 0.5f;
//         float h = this->m_CentroidY + 0.5f;
//         float d = this->m_CentroidZ + 0.5f;
// 
//         CVertex3D pos( m_MinRange.getX() + w * CubeW, m_MinRange.getY() + h * CubeH, m_MinRange.getZ() + d * CubeD );
// 
//         this->m_pAxisX.Draw(pos, CColor4f(1.f, 0.f, 0.f));
//         this->m_pAxisY.Draw(pos, CColor4f(0.f, 1.f, 0.f));
//         this->m_pAxisZ.Draw(pos, CColor4f(0.f, 0.f, 1.f));
// 
// 
//     }

    return;
}


//////////////////////////////////////////////////////////////////////
// Checking neighbourhoods
//////////////////////////////////////////////////////////////////////

bool CVoxelObject3D::N26(int w1, int h1, int d1, int w2, int h2, int d2) {

    int distW = (w1 < w2) ? w2 - w1 : w1 - w2;
    int distH = (h1 < h2) ? h2 - h1 : h1 - h2;
    int distD = (d1 < d2) ? d2 - d1 : d1 - d2;

    return ((distW <= 1) && (distH <= 1) && (distD <= 1));

}

bool CVoxelObject3D::N18(int w1, int h1, int d1, int w2, int h2, int d2) {

    int distW = (w1 < w2) ? w2 - w1 : w1 - w2;
    int distH = (h1 < h2) ? h2 - h1 : h1 - h2;
    int distD = (d1 < d2) ? d2 - d1 : d1 - d2;

    return (((distW + distH + distD) <= 2) && (distW < 2) && (distH < 2) && (distD < 2));

}

bool CVoxelObject3D::N6(int w1, int h1, int d1, int w2, int h2, int d2) {

    int distW = (w1 < w2) ? w2 - w1 : w1 - w2;
    int distH = (h1 < h2) ? h2 - h1 : h1 - h2;
    int distD = (d1 < d2) ? d2 - d1 : d1 - d2;

    return ((distW + distH + distD) <= 1 );

}


//////////////////////////////////////////////////////////////////////
// Fill empty voxels
//////////////////////////////////////////////////////////////////////

void CVoxelObject3D::Fill() {

    for (int w = 0; w < this->m_ScopeX; w++) {
        for (int h = 0; h < this->m_ScopeY; h++) {
            for (int d = 0; d < this->m_ScopeZ; d++) {

                bool nXn = false;
                bool nXp = false;
                bool nYn = false;
                bool nYp = false;
                bool nZn = false;
                bool nZp = false;
                int i;

                //TRACE("searching neighbors for %d %d %d\n", w, h, d);

                // neighbor X negative?
                if (w > 0)
                for (i = 0; i < w; i++) {
                    if (this->m_CubeArray(i,h,d).flag != CT_EMPTY) {
                    //if (this->m_CubeArray(i,h,d).flag == CT_INNER) {
                        nXn = true;
                        //TRACE("found neg x neighbor for %d %d %d  at  %d %d %d\n", w, h, d, i, h, d);
                        break;
                    }
                }

                // neighbor X positive?
                if (w < this->m_ScopeX - 1)
                for (i = w + 1; i < this->m_ScopeX; i++) {
                    if (this->m_CubeArray(i,h,d).flag != CT_EMPTY) {
                    //if (this->m_CubeArray(i,h,d).flag == CT_INNER) {
                        nXp = true;
                        //TRACE("found pos x neighbor for %d %d %d  at  %d %d %d\n", w, h, d, i, h, d);
                        break;
                    }
                }

                // neighbor Y negative?
                if (h > 0)
                for (i = 0; i < h; i++) {
                    if (this->m_CubeArray(w,i,d).flag != CT_EMPTY) {
                    //if (this->m_CubeArray(w,i,d).flag == CT_INNER) {
                        nYn = true;
                        //TRACE("found neg y neighbor for %d %d %d  at  %d %d %d\n", w, h, d, w, i, d);
                        break;
                    }
                }

                // neighbor Y positive?
                if (h < this->m_ScopeY - 1)
                for (i = h + 1; i < this->m_ScopeY; i++) {
                    if (this->m_CubeArray(w,i,d).flag != CT_EMPTY) {
                    //if (this->m_CubeArray(w,i,d).flag == CT_INNER) {
                        nYp = true;
                        //TRACE("found pos y neighbor for %d %d %d  at  %d %d %d\n", w, h, d, w, i, d);
                        break;
                    }
                }

                // neighbor Z negative?
                if (d > 0)
                for (i = 0; i < d; i++) {
                    if (this->m_CubeArray(w,h,i).flag != CT_EMPTY) {
                    //if (this->m_CubeArray(w,h,i).flag == CT_INNER) {
                        nZn = true;
                        //TRACE("found neg z neighbor for %d %d %d  at  %d %d %d\n", w, h, d, w, h, i);
                        break;
                    }
                }

                // neighbor Z positive?
                if (d < this->m_ScopeZ - 1)
                for (i = d + 1; i < this->m_ScopeZ; i++) {
                    if (this->m_CubeArray(w,h,i).flag != CT_EMPTY) {
                    //if (this->m_CubeArray(w,h,i).flag == CT_INNER) {
                        nZp = true;
                        //TRACE("found pos z neighbor for %d %d %d  at  %d %d %d\n", w, h, d, w, h, h);
                        break;
                    }
                }

                //TRACE("pos %d %d %d --> %d %d %d %d %d %d\n", w, h, d, nXn, nXp, nYn, nYp, nZn, nZp);

                if ((nXn == true) && (nXp == true) && (nYn == true) && (nYp == true) && (nZn == true) && (nZp == true)) {
                    this->m_CubeArray(w,h,d).flag = CT_INNER;
                    //TRACE("filled\n");
                }

            }
        }
    }

    return;
}


//////////////////////////////////////////////////////////////////////
// Label border voxels
//////////////////////////////////////////////////////////////////////

void CVoxelObject3D::LabelBorderVoxels() {


    for (int w = 0; w < this->m_ScopeX; w++) {
        for (int h = 0; h < this->m_ScopeY; h++) {
            for (int d = 0; d < this->m_ScopeZ; d++) {

                if (this->m_CubeArray(w,h,d).flag == CT_EMPTY) continue;

                if (w == 0 || h == 0 || d == 0 || w == this->m_ScopeX - 1 || h == this->m_ScopeY - 1 || d == this->m_ScopeZ - 1) {
                    this->m_CubeArray(w,h,d).flag = CT_BORDER;
                    continue;
                }

                if (
                    (this->m_CubeArray(w-1,h,d).flag == CT_EMPTY) ||
                    (this->m_CubeArray(w+1,h,d).flag == CT_EMPTY) ||
                    (this->m_CubeArray(w,h-1,d).flag == CT_EMPTY) ||
                    (this->m_CubeArray(w,h+1,d).flag == CT_EMPTY) ||
                    (this->m_CubeArray(w,h,d-1).flag == CT_EMPTY) ||
                    (this->m_CubeArray(w,h,d+1).flag == CT_EMPTY)
                ) {
                    this->m_CubeArray(w,h,d).flag = CT_BORDER;
                    continue;
                }

                this->m_CubeArray(w,h,d).flag = CT_INNER;

            }
        }
    }

    return;
}


//////////////////////////////////////////////////////////////////////
// Recalculate centroid
//////////////////////////////////////////////////////////////////////

void CVoxelObject3D::RecalculateCentroid() {

    // create moments object
    CMoments *m = new CMoments();

    // calculate 1-dimensional moments
    int m100 = m->Moment(1, 0, 0, this);
    int m010 = m->Moment(0, 1, 0, this);
    int m001 = m->Moment(0, 0, 1, this);
    int m000 = m->Moment(0, 0, 0, this);

    // calculate centroid
    this->m_CentroidX = m100 / m000 -1;
    this->m_CentroidY = m010 / m000 -1;
    this->m_CentroidZ = m001 / m000 -1;

    delete m;

    return;
}


//////////////////////////////////////////////////////////////////////
// Recalculate principle axes
//////////////////////////////////////////////////////////////////////

void CVoxelObject3D::RecalculatePrincipalAxes() {

    /// @todo what the hell is this function for??

//     RecalculateCentroid();
// 
//     CMoments moment;
// 
// //     int IMatrix[3][3];
// 
//     SymmetricMatrix A(3);
//     DiagonalMatrix D(3);
//     Matrix V(3,3);
// 
//     A(1,1) = moment.CentralMoment(0, 2, 0, this) + moment.CentralMoment(0, 0, 2, this);
//     A(2,2) = moment.CentralMoment(2, 0, 0, this) + moment.CentralMoment(0, 0, 2, this);
//     A(3,3) = moment.CentralMoment(2, 0, 0, this) + moment.CentralMoment(0, 2, 0, this);
// 
//     A(1,2) = - moment.CentralMoment(1, 1, 0, this);
//     A(1,3) = - moment.CentralMoment(1, 0, 1, this);
//     A(2,3) = - moment.CentralMoment(0, 1, 1, this);
// 
//     A(2,1) = A(1,2);
//     A(1,3) = A(3,1);
//     A(2,3) = A(3,2);
// 
//     EigenValues(A,D,V);
// 
//     this->m_pAxisX.setX( V(1,1) );
//     this->m_pAxisX.setY( V(2,1) );
//     this->m_pAxisX.setZ( V(3,1) );
// 
//     this->m_pAxisY.setX( V(1,2) );
//     this->m_pAxisY.setY( V(2,2) );
//     this->m_pAxisY.setZ( V(3,2) );
// 
//     this->m_pAxisZ.setX( V(1,3) );
//     this->m_pAxisZ.setY( V(2,3) );
//     this->m_pAxisZ.setZ( V(3,3) );
// 
//     return;
}


//////////////////////////////////////////////////////////////////////
// Copy from another voxel object
//////////////////////////////////////////////////////////////////////

void CVoxelObject3D::Copy(CVoxelObject3D *v) {

    if (v == 0) return;

    this->drawCubes = v->drawCubes;

    this->m_CenterX = v->m_CenterX;
    this->m_CenterY = v->m_CenterY;
    this->m_CenterZ = v->m_CenterZ;

    this->m_CentroidX = v->m_CentroidX;
    this->m_CentroidY = v->m_CentroidY;
    this->m_CentroidZ = v->m_CentroidZ;

    this->m_MaxRange = v->m_MaxRange;
    this->m_MinRange = v->m_MinRange;

    this->m_ScopeX = v->m_ScopeX;
    this->m_ScopeY = v->m_ScopeY;
    this->m_ScopeZ = v->m_ScopeZ;

    this->m_CubeArray.free();
    this->m_CubeArray = v->m_CubeArray;

    this->m_CubeColorArray.free();
    this->m_CubeColorArray = v->m_CubeColorArray;

    this->m_StandardCubeColor = v->m_StandardCubeColor;

}

void CVoxelObject3D::TransformTriangleToCubes(CVertex3D *vA, CVertex3D *vB, CVertex3D *vC) {

    int cubeAw = (int)((vA->getX() - this->m_MinRange.getX()) / (this->m_MaxRange.getX() - this->m_MinRange.getX()) * (this->m_ScopeX));
    int cubeAh = (int)((vA->getY() - this->m_MinRange.getY()) / (this->m_MaxRange.getY() - this->m_MinRange.getY()) * (this->m_ScopeY));
    int cubeAd = (int)((vA->getZ() - this->m_MinRange.getZ()) / (this->m_MaxRange.getZ() - this->m_MinRange.getZ()) * (this->m_ScopeZ));

    int cubeBw = (int)((vB->getX() - this->m_MinRange.getX()) / (this->m_MaxRange.getX() - this->m_MinRange.getX()) * (this->m_ScopeX));
    int cubeBh = (int)((vB->getY() - this->m_MinRange.getY()) / (this->m_MaxRange.getY() - this->m_MinRange.getY()) * (this->m_ScopeY));
    int cubeBd = (int)((vB->getZ() - this->m_MinRange.getZ()) / (this->m_MaxRange.getZ() - this->m_MinRange.getZ()) * (this->m_ScopeZ));

    int cubeCw = (int)((vC->getX() - this->m_MinRange.getX()) / (this->m_MaxRange.getX() - this->m_MinRange.getX()) * (this->m_ScopeX));
    int cubeCh = (int)((vC->getY() - this->m_MinRange.getY()) / (this->m_MaxRange.getY() - this->m_MinRange.getY()) * (this->m_ScopeY));
    int cubeCd = (int)((vC->getZ() - this->m_MinRange.getZ()) / (this->m_MaxRange.getZ() - this->m_MinRange.getZ()) * (this->m_ScopeZ));

    this->m_CubeArray(cubeAw, cubeAh, cubeAd) = CCube(CT_INNER); //&this->Cube_inner;
    this->m_CubeArray(cubeBw, cubeBh, cubeBd) = CCube(CT_INNER); //&this->Cube_inner;
    this->m_CubeArray(cubeCw, cubeCh, cubeCd) = CCube(CT_INNER); //&this->Cube_inner;


    // check neighborhoods
    if (!this->N26(cubeAw, cubeAh, cubeAd, cubeBw, cubeBh, cubeBd) || 
        !this->N26(cubeCw, cubeCh, cubeCd, cubeBw, cubeBh, cubeBd) || 
        !this->N26(cubeAw, cubeAh, cubeAd, cubeCw, cubeCh, cubeCd)) {

        CVertex3D vAB((vA->getX() - vB->getX()) / 2.f + 
                       vB->getX(), (vA->getY() - vB->getY()) / 2.f + 
                       vB->getY(), (vA->getZ() - vB->getZ()) / 2.f + 
                       vB->getZ());
        CVertex3D vBC((vB->getX() - vC->getX()) / 2.f + 
                       vC->getX(), (vB->getY() - vC->getY()) / 2.f + 
                       vC->getY(), (vB->getZ() - vC->getZ()) / 2.f + 
                       vC->getZ());
        CVertex3D vCA((vC->getX() - vA->getX()) / 2.f + 
                       vA->getX(), (vC->getY() - vA->getY()) / 2.f + 
                       vA->getY(), (vC->getZ() - vA->getZ()) / 2.f + 
                       vA->getZ());

        this->TransformTriangleToCubes( vA, &vAB, &vCA );
        this->TransformTriangleToCubes( vB, &vBC, &vAB );
        this->TransformTriangleToCubes( vC, &vCA, &vBC );
        this->TransformTriangleToCubes( &vBC, &vAB, &vCA );

    }

}

void CVoxelObject3D::TransformTriangleMeshToVoxelSet( CsmfObject *tm, int NbOfCubes ) {

    // recalculate bounding box of triangle mesh
    tm->DoRecalculateBoundingbox();

    // set voxel object range
    this->m_MinRange = tm->getVBB_lowerCorner();
    this->m_MaxRange = tm->getVBB_upperCorner();

    // resize voxel object range
    this->m_MinRange.setX( this->m_MinRange.getX() - 0.1f );
    this->m_MinRange.setY( this->m_MinRange.getY() - 0.1f );
    this->m_MinRange.setZ( this->m_MinRange.getZ() - 0.1f );

    this->m_MaxRange.setX( this->m_MaxRange.getX() + 0.1f );
    this->m_MaxRange.setY( this->m_MaxRange.getY() + 0.1f );
    this->m_MaxRange.setZ( this->m_MaxRange.getZ() + 0.1f );

    // calculate voxel object scopes
    this->m_ScopeY = (int) (pow( NbOfCubes * 
                                pow((this->m_MaxRange.getY() - this->m_MinRange.getY()), 2) /
                                (this->m_MaxRange.getZ() - this->m_MinRange.getZ()) /
                                (this->m_MaxRange.getX() - this->m_MinRange.getX()),
                                0.333333f ));

    this->m_ScopeX = (int) ((this->m_MaxRange.getX() - this->m_MinRange.getX()) /
                           (this->m_MaxRange.getY() - this->m_MinRange.getY()) * this->m_ScopeY);

    this->m_ScopeZ = (int) ((this->m_MaxRange.getZ() - this->m_MinRange.getZ()) /
                           (this->m_MaxRange.getY() - this->m_MinRange.getY()) * this->m_ScopeY);

    // free voxel object cubes array
    this->m_CubeArray.free();

    // init voxel object cubes array with empty cubes
    this->m_CubeArray.init(this->m_ScopeX, this->m_ScopeY, this->m_ScopeZ, CT_EMPTY);

    // transform
    for (int i = 0; i < tm->getFaceCount(); i++) {

        CPolygon face = tm->getFaceAtPos( i );

        CVertex3D v1 = tm->getVertexAtPos( face.getVertexIndexAtPos( 0 ) );
        CVertex3D v2 = tm->getVertexAtPos( face.getVertexIndexAtPos( 1 ) );
        CVertex3D v3 = tm->getVertexAtPos( face.getVertexIndexAtPos( 2 ) );

        this->TransformTriangleToCubes( &v1, &v2, &v3 );
    }

    this->LabelBorderVoxels();

    this->Fill();

}


/*!
    \fn CVoxelObject3D::getMemoryUsage()
 */
double CVoxelObject3D::getMemorySize()
{
    return (this->m_ScopeX * this->m_ScopeY * this->m_ScopeZ * sizeof(CCube));
}

void CVoxelObject3D::countDefinedVoxels() {
    this->numDefVoxels = 0;
    for (int w = 0; w < this->m_ScopeX; w++) {
        for (int h = 0; h < this->m_ScopeY; h++) {
            for (int d = 0; d < this->m_ScopeZ; d++) {
                if ( this->m_CubeArray(w,h,d).flag == CT_BORDER ) this->numDefVoxels++;
            }
        }
    }
}

bool CVoxelObject3D::convertToHRLEBfile( const string& directory ) {

  int i,zRun,zCoord;
  int startCoord, endCoord;
  int definedIndex;
  short int chunkSize;
  int partChunkSize;
  CHierarchicalRLE* tmpHrleObject = NULL;

  // create an HRLE object from this voxel object
  CHierarchicalRLE* hrleObject = new CHierarchicalRLE( 3, this );

  // start and end coordinate of the first run
  startCoord = hrleObject->min;
  if ( hrleObject->run_breaks.size() >= 1 ) endCoord = hrleObject->run_breaks.at(0);
  else endCoord = hrleObject->max;

  // set file name for master file
  string masterFileName = directory;
  masterFileName += "master.dat";

  // open file stream for master file
  std::ofstream masterFile( masterFileName.c_str(), ios_base::binary & ios_base::out );

  // maximum and minmum extends for the three dimensions
  masterFile.write( "MinZ", 4);
  masterFile.write( (char*)&(hrleObject->min), 2);
  masterFile.write( "MaxZ", 4);
  masterFile.write( (char*)&(hrleObject->max), 2);
  masterFile.write( "MinY", 4);
  masterFile.write( (char*)&(hrleObject->lower_hrle->min), 2);
  masterFile.write( "MaxY", 4);
  masterFile.write( (char*)&(hrleObject->lower_hrle->max), 2);
  masterFile.write( "MinX", 4);
  masterFile.write( (char*)&(hrleObject->lower_hrle->lower_hrle->min), 2);
  masterFile.write( "MaxX", 4);
  masterFile.write( (char*)&(hrleObject->lower_hrle->lower_hrle->max), 2);

  // add runs of the z coordinate
  masterFile.write( "Runs", 4);
  chunkSize = (short int) hrleObject->run_types.size();
  masterFile.write( (char*)&(chunkSize), 2);
  for ( i = 0; i < hrleObject->run_types.size(); i++ ) {
      masterFile.write( (char*)&(hrleObject->run_types.at(i)), 2);
  }

  // add breaks of the z coordinate
  masterFile.write( "Brks", 4);
  chunkSize = (short int) hrleObject->run_breaks.size();
  masterFile.write( (char*)&(chunkSize), 2);
  for ( i = 0; i < hrleObject->run_breaks.size(); i++ ) {
      masterFile.write( (char*)&(hrleObject->run_breaks.at(i)), 2);
  }

  masterFile.close();

  for ( zRun = 0; zRun < hrleObject->run_types.size(); zRun++ ) {
    int run = hrleObject->run_types.at(zRun);
    if ( RT_DEFINED( run ) ) {
      for ( zCoord = startCoord; zCoord < endCoord; zCoord++ ) {

        // calculate defined index of the segment
        definedIndex = run + zCoord - startCoord;

        // setup filename for partfile
        string filename = directory;
        filename += "part_";
        int partIndex(definedIndex);
        std::stringstream ss;
        if (ss << partIndex)
        {
            filename += ss.str();
        }
        else
            cerr << "Error defining filename for partfile!" << endl;

        filename += ".dat";

        // get the xy plane
        tmpHrleObject = hrleObject->lower_hrle->getSegmentAt( definedIndex );

        // open file stream for part file
        std::ofstream partFile( filename.c_str(), ios_base::binary & ios_base::out );

        // add runs of the y coordinate
        partFile.write( "RunY", 4);
        partChunkSize = (int) tmpHrleObject->run_types.size();
        partFile.write( (char*)&(partChunkSize), 4);
        for ( i = 0; i < tmpHrleObject->run_types.size(); i++ ) {
            partFile.write( (char*)&(tmpHrleObject->run_types.at(i)), 4);
        }

        // add breaks of the y coordinate
        partFile.write( "BrkY", 4);
        partChunkSize = (int) tmpHrleObject->run_breaks.size();
        partFile.write( (char*)&(partChunkSize), 4);
        for ( i = 0; i < tmpHrleObject->run_breaks.size(); i++ ) {
            partFile.write( (char*)&(tmpHrleObject->run_breaks.at(i)), 2);
        }

        // add runs of the x coordinate
        partFile.write( "RunX", 4);
        partChunkSize = (int) tmpHrleObject->lower_hrle->run_types.size();
        partFile.write( (char*)&(partChunkSize), 4);
        for ( i = 0; i < tmpHrleObject->lower_hrle->run_types.size(); i++ ) {
            partFile.write( (char*)&(tmpHrleObject->lower_hrle->run_types.at(i)), 4);
        }

        // add breaks of the x coordinate
        partFile.write( "BrkX", 4);
        partChunkSize = (int) tmpHrleObject->lower_hrle->run_breaks.size();
        partFile.write( (char*)&(partChunkSize), 4);
        for ( i = 0; i < tmpHrleObject->lower_hrle->run_breaks.size(); i++ ) {
            partFile.write( (char*)&(tmpHrleObject->lower_hrle->run_breaks.at(i)), 2);
        }

        partFile.write( "iStX", 4);
        partChunkSize = (int) tmpHrleObject->lower_hrle->start_indices.size();
        partFile.write( (char*)&(partChunkSize), 4);
        for ( i = 0; i < tmpHrleObject->lower_hrle->start_indices.size(); i++ ) {
            partFile.write( (char*)&(tmpHrleObject->lower_hrle->start_indices.at(i)), 4);
        }

        partFile.close();

        delete tmpHrleObject;
      }
    }

    startCoord = endCoord;
    if ( (zRun+1) < hrleObject->run_breaks.size() ) endCoord = hrleObject->run_breaks.at( zRun+1 );
    else endCoord = hrleObject->max;
  }

  return true;
}