/***************************************************************************
 *   Copyright (C) 2007 by Holger Gerth                                    *
 *   holger.gerth@informatik.tu-chemnitz.de                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "hierarchicalrle.h"

using namespace std;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//      Public functions
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// constructor
CHierarchicalRLE::CHierarchicalRLE()
{

    this->dimension = 1;
    this->lower_hrle = NULL;

    // initialize min and max
    this->min = 0;
    this->max = 0;

    // min and max coordinate range
    this->minRange = CVertex3D(0,0,0);
    this->maxRange = CVertex3D(0,0,0);

    // for testing only
    this->accessTimeRandomAccess = 0.0;
    this->accessAverageTimeRandomAccess = 0.0;
    this->accessTimeSequentialAccess = 0.0;

}

// constructor
CHierarchicalRLE::CHierarchicalRLE( unsigned int dim )
{

    this->dimension = dim;

    if (dim > 1 ) {
        this->lower_hrle = new CHierarchicalRLE( dim -1 );
    }
    else {
        this->lower_hrle = NULL;
    }

    // initialize min and max
    this->min = 0;
    this->max = 0;

    // min and max coordinate range
    this->minRange = CVertex3D(0,0,0);
    this->maxRange = CVertex3D(0,0,0);

    // for testing only
    this->accessTimeRandomAccess = 0.0;
    this->accessAverageTimeRandomAccess = 0.0;
    this->accessTimeSequentialAccess = 0.0;

}

// constructor
CHierarchicalRLE::CHierarchicalRLE( unsigned int dim, CVoxelObject3D* vo )
{

    if (dim > 1 ) {
        this->dimension = dim;
        this->lower_hrle = new CHierarchicalRLE( dim -1 );
    }
    else {
        this->dimension = 1;
        this->lower_hrle = NULL;
    }

    // get the data from the voxel object
    this->create_hrle_from_voxel_object( vo );

    // for testing only
    this->accessTimeRandomAccess = 0.0;
    this->accessAverageTimeRandomAccess = 0.0;
    this->accessTimeSequentialAccess = 0.0;

}

// destructor
CHierarchicalRLE::~CHierarchicalRLE()
{
//     if ( this->lower_hrle != NULL ) {
//         delete this->lower_hrle;
//         this->lower_hrle = NULL;
//     }
}

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////

/*!
    \fn CHierarchicalRLE::convert_to_hrle( CVoxelObject3D* vo )
 */
// converts a voxel object into a hrle level set
bool CHierarchicalRLE::convert_to_hrle( CVoxelObject3D* vo ) {

    if (this->dimension != 3) {
        cerr << "HRLE Object has false dimension - need to be 3!" << endl;
        return false;
    }
    else {

        CVoxelObject3D* tmp_vo = new CVoxelObject3D();
        bool returnValue;

        tmp_vo->Copy( vo );

        returnValue = this->create_hrle_from_voxel_object( tmp_vo );

        delete tmp_vo;

        return returnValue;
    }

}


/*!
    \fn CHierarchicalRLE::randomAccess( vector<int> *qv, int segmentIndex)
 */
// the random access operator to access data
int CHierarchicalRLE::randomAccess( vector<int> qv, int segmentIndex)
{

    if ( qv.size() == 0 )
        cerr << "Error random access: not enough coordinates!" << endl;

    int qi = qv.back();                              // split first coordinate
    qv.pop_back();                                   // delete the coordinate from the vector
    int segmentLength;
    int dataIndex;
    unsigned int firstRun;
    unsigned int firstRunBreak;
    int indexRun;                                     // index of the run in the run codes array
    int run = 0;                                      // a run code and (if defined) the index in the defined data array
    short int firstCoordinate = this->min;            // minimum coordinate or last break before the determined run
    short int lastCoordinate = this->max;             // miximum coordinate or the break after the determined run

    // qi doesn't lie between the extends of this HRLE
    if ( (qi < this->min) || (qi >= this->max) ) {
        run = RT_POSITIVE;
        return run;
    }

    try {
      this->start_indices.at(segmentIndex);
    }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
        cerr << "Out of range exception " << e.what() << endl;
        cerr << "Random access: no segment with segment index " << segmentIndex << "(Dimension: " << this->dimension << ")" << endl;
        exit(1);
    }

    // get segment length - number of runs in this segment
    if (this->start_indices.at(segmentIndex) == this->start_indices.at(this->start_indices.size()-1))
        segmentLength = this->run_types.size() - this->start_indices.at(segmentIndex);
    else
        segmentLength = this->start_indices.at(segmentIndex+1) - this->start_indices.at(segmentIndex);

    // get the index of the first run in this segment
    firstRun = this->start_indices.at(segmentIndex);

    // get the index of the first run break of this segment
    firstRunBreak = this->start_indices.at(segmentIndex) - segmentIndex;

    // get the index of the run containing qi
    try {
    indexRun = this->accessBinSearch( qi, firstRun, firstRunBreak, segmentLength, &firstCoordinate, &lastCoordinate );
    }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
        cerr << "Out of range exception " << e.what() << endl;
        cerr << "Random access: error while binary search! Coordinate: " << qi << ", dimension: " << this->dimension << endl;
        cerr << "Segment index is: " << segmentIndex << endl;
        exit(1);
    }

    // get the run code
    try {
        run = this->run_types.at( indexRun );
    }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
        cerr << "Out of range exception " << e.what() << endl;
        cerr << "Random access: cannot get run " << indexRun << endl;
        exit(1);
    }

    if ( RT_DEFINED(run) ) {

        // defined data index = run code + offset of qi
        dataIndex = run + qi - firstCoordinate;

        // recursive traversal to the next lower hrle block
        if (this->lower_hrle != NULL) {
            return this->lower_hrle->randomAccess( qv, dataIndex );
        }
        else run = dataIndex;                                     // return the direct index into the value array
    }

    return run;

}

int CHierarchicalRLE::randomAccess( int X, int Y, int Z, int segmentIndex) {

    // create a stl vector from the given coordinates
    vector<int>     qv;
    qv.push_back(X);
    qv.push_back(Y);
    qv.push_back(Z);

    return this->randomAccess( qv, segmentIndex);


}

// same as random access but returns only the index of the segment in the X encoding, which includes the point
// dimension of the segment index depends on the coordinate vector qv
int CHierarchicalRLE::getSegmentIndex( vector<int> qv, int segmentIndex)
{
    int qi = qv.back();                          // split first coordinate
    qv.pop_back();                               // delete the coordinate from the vector
    int segmentLength;
    int dataIndex;
    unsigned int firstRun;
    unsigned int firstRunBreak;
    int indexRun;                                // index of the run in the run codes array
    int run = 0;                                 // a run code and (if defined) the index in the defined data array
    short int firstCoordinate = this->min;       // minimum coordinate or last break before the determined run
    short int lastCoordinate = this->max;        // miximum coordinate or the break after the determined run

    if ( (this->lower_hrle == NULL) || (qv.size() == 0) )
        return this->start_indices.at( segmentIndex );

    // qi doesn't lie between the extends of this HRLE
    if ( (qi < this->min) || (qi >= this->max) ) {
        run = RT_POSITIVE;
        return run;
    }

    // get segment length - number of runs in this segment
    if (this->start_indices.at(segmentIndex) == this->start_indices.at(this->start_indices.size()-1))
        segmentLength = this->run_types.size() - this->start_indices.at(segmentIndex);
    else
        segmentLength = this->start_indices.at(segmentIndex+1) - this->start_indices.at(segmentIndex);

    // get the index of the first run in this segment
    firstRun = this->start_indices.at(segmentIndex);

    // get the index of the first run break of this segment
    firstRunBreak = this->start_indices.at(segmentIndex) - segmentIndex;

    // get the index of the run containing qi
    indexRun = this->accessBinSearch( qi, firstRun, firstRunBreak, segmentLength, &firstCoordinate, &lastCoordinate );

    // get the run code
    run = this->run_types.at( indexRun );

    // search for the segment in the next lower dimension
    if ( RT_DEFINED(run)) {

        // defined data index = run code + offset of qi
        dataIndex = run + qi - firstCoordinate;

        // recursive traversal to the next lower hrle block
        return this->lower_hrle->getSegmentIndex( qv, dataIndex );
    }

    return run;

}

// returns the dilated HRLE object
CHierarchicalRLE* CHierarchicalRLE::getDilatedHRLE( vector<bool> structureElement ) {

    CHierarchicalRLE* dilatedHRLE = this->dilation( structureElement );

    if ( dilatedHRLE == NULL ) return NULL;

    dilatedHRLE->minRange = this->minRange;
    dilatedHRLE->maxRange = this->maxRange;
    dilatedHRLE->lower_hrle->minRange = this->lower_hrle->minRange;
    dilatedHRLE->lower_hrle->maxRange = this->lower_hrle->maxRange;
    dilatedHRLE->lower_hrle->lower_hrle->minRange = this->lower_hrle->lower_hrle->minRange;
    dilatedHRLE->lower_hrle->lower_hrle->maxRange = this->lower_hrle->lower_hrle->maxRange;
    return dilatedHRLE;

}

// returns the dilated HRLE object
CHierarchicalRLE* CHierarchicalRLE::getDilatedHRLE( CHierarchicalRLE* structureElement, vector<short int> mpv ) {

    CHierarchicalRLE* dilatedHRLE = this->advancedDilation( structureElement, mpv );
//     delete structureElement;

    if ( dilatedHRLE == NULL ) return NULL;

    dilatedHRLE->minRange = this->minRange;
    dilatedHRLE->maxRange = this->maxRange;
    dilatedHRLE->lower_hrle->minRange = this->lower_hrle->minRange;
    dilatedHRLE->lower_hrle->maxRange = this->lower_hrle->maxRange;
    dilatedHRLE->lower_hrle->lower_hrle->minRange = this->lower_hrle->lower_hrle->minRange;
    dilatedHRLE->lower_hrle->lower_hrle->maxRange = this->lower_hrle->lower_hrle->maxRange;
    return dilatedHRLE;

}

// returns the eroded HRLE object
CHierarchicalRLE* CHierarchicalRLE::getErodedHRLE( CHierarchicalRLE* structureElement, vector<short int> mpv ) {

    CHierarchicalRLE* erodedHRLE = this->erosion( structureElement, mpv );

    if ( erodedHRLE == NULL ) {
        erodedHRLE = new CHierarchicalRLE(this->dimension);
        erodedHRLE->run_types.push_back( RT_POSITIVE );
        erodedHRLE->start_indices.push_back(0);
        erodedHRLE->min = this->min;
        erodedHRLE->max = this->max;
        erodedHRLE->lower_hrle->min = this->lower_hrle->min;
        erodedHRLE->lower_hrle->max = this->lower_hrle->max;
        erodedHRLE->lower_hrle->lower_hrle->min = this->lower_hrle->lower_hrle->min;
        erodedHRLE->lower_hrle->lower_hrle->max = this->lower_hrle->lower_hrle->max;

        erodedHRLE->minRange = this->minRange;
        erodedHRLE->maxRange = this->maxRange;
    }
    else {
        erodedHRLE->minRange = this->minRange;
        erodedHRLE->maxRange = this->maxRange;
        erodedHRLE->lower_hrle->minRange = this->lower_hrle->minRange;
        erodedHRLE->lower_hrle->maxRange = this->lower_hrle->maxRange;
        erodedHRLE->lower_hrle->lower_hrle->minRange = this->lower_hrle->lower_hrle->minRange;
        erodedHRLE->lower_hrle->lower_hrle->maxRange = this->lower_hrle->lower_hrle->maxRange;
    }
    return erodedHRLE;

}

// wrapper for computeMedialSurface; returns the medial surface and leaves this object unchanged
CHierarchicalRLE* CHierarchicalRLE::getMedialSurface( bool bTopHold, bool bBottomHold ) {
    CHierarchicalRLE* tmpValue = new CHierarchicalRLE( this->dimension );

    tmpValue->copy( this );

    tmpValue->computeMedialSurface( bTopHold, bBottomHold );

    return tmpValue;
}

// wrapper for computeMedialAxis; returns the medial axis and leave this object unchanged
CHierarchicalRLE* CHierarchicalRLE::getMedialAxis( bool bTopHold, bool bBottomHold ) {
    CHierarchicalRLE* tmpValue = new CHierarchicalRLE( this->dimension );

    tmpValue->copy( this );

    tmpValue->computeMedialAxis( bTopHold, bBottomHold );

    return tmpValue;
}

// compute the medial surface of the object - first pass of the thinning operation
// this algorithm needs the result of the medial surface operation as input
bool CHierarchicalRLE::computeMedialSurface( bool bTopHold, bool bBottomHold ) {

    int                     i, type, numCycles = 1;

    vector<definedPoint>     pointArray;
    vector<coordValue>      delateablePoints;
    bool                    pointsDeleted;

    pointArray = this->getDefinedPoints();

    // delete border points, which are not final and don't deconnect components
    do {
        // initialize the stopping value
        pointsDeleted = false;

        // direction order: B, T, E, W, S, N
        for ( type = 1; type < 7; type++ ) {
            // for all types of border points
            for ( i = 0; i < pointArray.size(); i++ ) {

                if ( pointArray.at(i).isDeleted == false ) {
                    if ( this->isBorderPointOfDirection( pointArray.at(i).coordinate, (borderPointType)type ) ) {

                        if ( this->isFinalPoint( pointArray.at(i).coordinate ) == false ) {

                            if ( this->isSimplePoint(pointArray.at(i).coordinate) == true ) {

                                // checking plane condition holds, so point can be deleted
                                if ( this->checkingPlaneCondition( pointArray.at(i).coordinate, 2, (borderPointType)type ) == true ) {

                                    this->addToDeleteablePoints( pointArray.at(i).coordinate, &delateablePoints );

                                    pointsDeleted = true;
                                    pointArray.at(i).isDeleted = true;
                                }

                            }
                        }
                    }
                }

            }


            // delete the points
            if ( delateablePoints.size() > 0 ) {
                try {
                    if (!(this->deleteBorderPoints_bool( delateablePoints ))) {
                        cerr << "Error while deleting points!" << endl;

                    }
                }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
                    cerr << "Out of range exception " << e.what() << endl;
                    cerr << "Medial surface: error deleting points." << endl;
                    exit(1);
                }
            }

            delateablePoints.clear();
        }

    } while ( pointsDeleted );

    return true;

}

// compute the medial axis of the object - second pass of the thinning operation
// this algorithm needs the result of the medial surface operation as input
bool CHierarchicalRLE::computeMedialAxis( bool bTopHold, bool bBottomHold ) {

    int                     i, type, numCycles = 1;

    vector<definedPoint>    pointArray;
    vector<coordValue>      delateablePoints;
    bool                    pointsDeleted;

    pointArray = this->getDefinedPoints();

    // delete border points, which are not final and don't deconnect components
    do {

        // initialize the stopping value
        pointsDeleted = false;

        // direction order: B, T, E, W, S, N
        for ( type = 1; type < 7; type++ ) {

            // for all types of border points
            for ( i = 0; i < pointArray.size(); i++ ) {

                if ( pointArray.at(i).isDeleted == false ) {
                    if ( this->isBorderPointOfDirection(
                         pointArray.at(i).coordinate, (borderPointType)type ) ) {

                        if ( this->isFinalPoint( pointArray.at(i).coordinate ) == false ) {

                            if ( this->isSimplePoint(pointArray.at(i).coordinate) == true ) {

                                // checking plane condition holds, so point can be deleted
                                if ( this->checkingPlaneCondition( pointArray.at(i).coordinate, 1, (borderPointType)type ) == true ) {

                                    this->addToDeleteablePoints( pointArray.at(i).coordinate, &delateablePoints );

                                    pointsDeleted = true;
                                    pointArray.at(i).isDeleted = true;
                                }

                            }
                        }
                    }

                }

            }


            // delete the points
            if ( delateablePoints.size() > 0 ) {

                try {
                    if (!(this->deleteBorderPoints_bool( delateablePoints ))) {
                        cerr << "Error while deleting points!" << endl;

                    }
                }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
                    cerr << "Out of range exception " << e.what() << endl;
                    cerr << "Medial surface: error deleting points." << endl;
                    exit(1);
                }
            }

            delateablePoints.clear();

        }

    } while ( pointsDeleted );

    return true;

}

// one single thinning step
bool CHierarchicalRLE::thinObject( short int checkPlaneValue, borderPointType type ) {

    int                     i;

    vector<definedPoint>    pointArray;
    vector<coordValue>      delateablePoints;
    bool                    pointsDeleted = false;

    pointArray = this->getBorderPoints();

        // for all types of border points
        for ( i = 0; i < pointArray.size(); i++ ) {

            if ( pointArray.at(i).isDeleted == false ) {
                if ( this->isBorderPointOfDirection( pointArray.at(i).coordinate, (borderPointType)type ) ) {

                    if ( this->isFinalPoint( pointArray.at(i).coordinate ) == false ) {

                        if ( this->isSimplePoint(pointArray.at(i).coordinate) == true ) {

                            // checking plane condition holds, so point can be deleted
                            if ( this->checkingPlaneCondition( pointArray.at(i).coordinate,
                                 checkPlaneValue, (borderPointType)type ) == true ) {

                                pointArray.at(i).isDeleted = true;

                                if ( (pointArray.at(i).coordinate.back() != this->min) &&
                                     (pointArray.at(i).coordinate.back() != (this->max-1)) ) {
                                    this->addToDeleteablePoints( pointArray.at(i).coordinate, &delateablePoints );
                                    pointsDeleted = true;
                                }
                            }

                        }
                    }
                }
            }
        }



        // delete the points
        if ( delateablePoints.size() > 0 ) {

            try {
                if (!(this->deleteBorderPoints_bool( delateablePoints ))) {
                    cerr << "Error while deleting points!" << endl;

                }
            }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
                cerr << "Out of range exception " << e.what() << endl;
                cerr << "Medial surface: error deleting points." << endl;
                exit(1);
            }
        }
        delateablePoints.clear();

    return pointsDeleted;

}

// copies the data of value into the current H-RLE structure
bool CHierarchicalRLE::copy( CHierarchicalRLE* value ) {

    if ( value == NULL ) return false;

    if ( this->lower_hrle != NULL ) {
        if ( this->lower_hrle->copy( value->lower_hrle ) == false) return false;
    }

    // clear the arrays
    this->start_indices.clear();
    this->run_breaks.clear();
    this->run_types.clear();

    // copy data
    this->min = value->min;
    this->max = value->max;
    this->run_types = value->run_types;
    this->run_breaks = value->run_breaks;
    this->start_indices = value->start_indices;

    return true;
}

// the function is used to retrieve an RLE segment in this dimension with all its subsegments by the segment index
CHierarchicalRLE* CHierarchicalRLE::getSegmentAt( int si, bool withSubSegments ) {

    CHierarchicalRLE* returnValue;
    int firstRun, firstBreak, segmentLength, segmentEnd;
    int startCoord, endCoord;
    int runIndex = 0;
    int newRun;
    int definedIndex;
    int i,j;

    if ( withSubSegments ) returnValue = new CHierarchicalRLE( this->dimension );
    else returnValue = new CHierarchicalRLE();

    // get the index of the first run in this segment
    firstRun = this->start_indices.at( si );


    // get the index of the first run break of this segment
    firstBreak = this->start_indices.at( si ) - si;
    // end of this segment (start of the next one)
    if (this->start_indices.at( si ) == this->start_indices.at(this->start_indices.size()-1))
        segmentEnd = this->run_types.size();
    else
        segmentEnd = this->start_indices.at( si+1 );
    // get segment length - number of runs in this segment
    segmentLength = segmentEnd - firstRun;
    // start- and endcoordinates
    startCoord = this->min;
    if ( segmentLength > 1) endCoord = this->run_breaks.at( firstBreak );
    else endCoord = this->max;

    // set min, max and start index of this segment in the new block
    returnValue->min = this->min;
    returnValue->max = this->max;
    returnValue->start_indices.push_back( 0 );

    if ( withSubSegments && (this->lower_hrle != NULL) )
        returnValue->lower_hrle = new CHierarchicalRLE( this->dimension -1 );

    for ( i = firstRun; i < segmentEnd; i++ ) {

        newRun = this->run_types.at( i );

        if ( RT_DEFINED(newRun) ) {

            runIndex = newRun;

            if ( withSubSegments && (this->lower_hrle != NULL) )
                newRun = returnValue->lower_hrle->start_indices.size();
            else
                newRun = 1;

            if ( withSubSegments && (this->lower_hrle != NULL) ) {
              for ( j = startCoord; j < endCoord; j++ ) {
                definedIndex = runIndex + j - startCoord;
                returnValue->lower_hrle->addToHRLE( this->lower_hrle->getSegmentAt( definedIndex ) );
              }
            }

        }

        // add this run
        returnValue->run_types.push_back( newRun );

        // add the break, if it exists
        if ( (i+1) < segmentEnd ) returnValue->run_breaks.push_back( this->run_breaks.at(firstBreak++) );

        startCoord = endCoord;
        if ( (i+2) < segmentEnd ) endCoord = this->run_breaks.at( firstBreak );
        else endCoord = this->max;

    }

    return returnValue;

}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//      Private functions
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// convert an CVoxelObject3D into the HRLE structure
bool CHierarchicalRLE::create_hrle_from_voxel_object( CVoxelObject3D* vo ) {

    int segments = 0;
    int numRunCodes;

    this->minRange = vo->m_MinRange;
    this->maxRange = vo->m_MaxRange;

    // create x-RLE Block
    if (this->dimension == 1) {

        this->min = 0;
        this->max = vo->m_ScopeX;

        for (int z = 0; z < vo->m_ScopeZ; z++) {
            for (int y = 0; y < vo->m_ScopeY; y++) {
                numRunCodes = 0;
                segments++;

                for (int x = 0; x < vo->m_ScopeX; x++) {

                    // handle blocks which are outer
                    if (vo->m_CubeArray(x,y,z).flag == CT_EMPTY) {
                        if (numRunCodes == 0) {
                            this->run_types.push_back( RT_POSITIVE );
                            numRunCodes++;
                        }
                        else if ( RT_DEFINED(this->run_types[this->run_types.size()-1]) ) {
                            this->run_breaks.push_back(x);
                            this->run_types.push_back( RT_POSITIVE );
                            numRunCodes++;
                        }
                    }

                    // handle the border blocks
                    else if ((vo->m_CubeArray(x,y,z).flag == CT_BORDER) ||
                             (vo->m_CubeArray(x,y,z).flag == CT_INNER)) {
                        if (numRunCodes == 0) {
                            this->run_types.push_back( 1 );
                            numRunCodes++;
                        }
                        else if ( !RT_DEFINED(this->run_types[this->run_types.size()-1]) ) {
                            this->run_breaks.push_back(x);
                            this->run_types.push_back( 1 );
                            numRunCodes++;
                        }

                    }

                }

                // if no defined runs are in this segment
                if ((numRunCodes == 1) && ( !RT_DEFINED(this->run_types[this->run_types.size()-1]) )) {
                    // remove last run -> will be saved in the next higher dimension
                    this->run_types.pop_back();
                }
                else {
                    // add offset of this segment to the start_indices block
                    this->start_indices.push_back( this->run_types.size() - numRunCodes );

                    // create a 2D projection of the voxel set
                    vo->m_CubeArray(0,y,z).flag = CT_BORDER;
                }
            }
        }

    }

    // start encoding along Y axis
    else if (this->dimension == 2) {

        this->lower_hrle->create_hrle_from_voxel_object( vo );

        this->min = 0;
        this->max = vo->m_ScopeY;

        int segmentIndex = 0;

        for (int z = 0; z < vo->m_ScopeZ; z++) {
            numRunCodes = 0;
            segments++;
            for (int y = 0; y < vo->m_ScopeY; y++) {
                // handle blocks which are outer
                if (vo->m_CubeArray(0,y,z).flag == CT_EMPTY) {
                    if (numRunCodes == 0) {
                        this->run_types.push_back( RT_POSITIVE );
                        numRunCodes++;
                    }
                    else if ( RT_DEFINED(this->run_types[this->run_types.size()-1]) ) {
                        this->run_breaks.push_back(y);
                        this->run_types.push_back( RT_POSITIVE );
                        numRunCodes++;
                    }
                }

                // handle the border blocks
                else if ((vo->m_CubeArray(0,y,z).flag == CT_BORDER) ||
                         (vo->m_CubeArray(0,y,z).flag == CT_INNER)) {
                    if (numRunCodes == 0) {
                        this->run_types.push_back( segmentIndex );
                        numRunCodes++;
                    }
                    else if ( !RT_DEFINED(this->run_types[this->run_types.size()-1]) ) {
                        this->run_breaks.push_back(y);
                        this->run_types.push_back( segmentIndex );
                        numRunCodes++;
                    }

                    segmentIndex++;

                }

            }

            // if no defined runs are in this segment
            if ((numRunCodes == 1) && ( !RT_DEFINED(this->run_types[this->run_types.size()-1]) )) {
                // remove last run -> will be saved in the next higher dimension
                this->run_types.pop_back();
            }
            else {
                // add offset of this segment to the start_indices block
                this->start_indices.push_back( this->run_types.size() - numRunCodes );

                // create a 2D projection of the voxel set
                vo->m_CubeArray(0,0,z).flag = CT_BORDER;

            }
        }

    }

    // start encoding along Z axis
    else if (this->dimension == 3) {

        this->lower_hrle->create_hrle_from_voxel_object( vo );

        this->min = 0;
        this->max = vo->m_ScopeZ;
        cout << "ScopeZ: " << vo->m_ScopeZ << endl;

        int segmentIndex = 0;
        numRunCodes = 0;

        for (int z = 0; z < vo->m_ScopeZ; z++) {
            // handle blocks which are outer
            if (vo->m_CubeArray(0,0,z).flag == CT_EMPTY) {
                if (numRunCodes == 0) {
                    this->run_types.push_back( RT_POSITIVE );
                    numRunCodes++;
                }
                else if ( RT_DEFINED(this->run_types[this->run_types.size()-1]) ) {
                    this->run_breaks.push_back(z);
                    this->run_types.push_back( RT_POSITIVE );
                    numRunCodes++;
                }
            }

            // handle the border blocks
            else if ((vo->m_CubeArray(0,0,z).flag == CT_BORDER) ||
                     (vo->m_CubeArray(0,0,z).flag == CT_INNER)) {
                if (numRunCodes == 0) {
                    this->run_types.push_back( segmentIndex );
                    numRunCodes++;
                }
                else if ( !RT_DEFINED(this->run_types[this->run_types.size()-1]) ) {
                    this->run_breaks.push_back(z);
                    this->run_types.push_back( segmentIndex );
                    numRunCodes++;
                }

                segmentIndex++;

            }

        }

        this->start_indices.push_back( 0 );

    }
    else return false;

    return true;

}

// Binary Search within the run breaks array . returns index in the start_indices array
int CHierarchicalRLE::accessBinSearch( int qi, unsigned int indexFirstRun, unsigned int indexFirstBreak, int segmentLength, short int *min, short int *max )
{

    if ( segmentLength == 1 ) {
        return indexFirstRun;
    }
    if ( segmentLength == 2 ) {
        if ( qi < this->run_breaks.at(indexFirstBreak)) {
            *max = this->run_breaks.at(indexFirstBreak);
            return indexFirstRun;
        }
        else {
            *min = this->run_breaks.at(indexFirstBreak);
            return (indexFirstRun+1);
        }
    }
    else {

        // get middle element of the run breaks
        int mid = indexFirstBreak + ((segmentLength - 1) / 2);

        // recursive search in the left array
        if ( qi < this->run_breaks.at(mid)) {
           *max = this->run_breaks.at(mid);
           return this->accessBinSearch( qi, indexFirstRun, indexFirstBreak, (segmentLength+1) / 2, min, max);
        }
        // recursive search in the right array
        else {
           *min = this->run_breaks.at(mid);
           return this->accessBinSearch( qi, indexFirstRun + ((segmentLength+1) / 2), indexFirstBreak + ((segmentLength+1) / 2), segmentLength / 2, min, max);
        }
    }
}

// dilation algorithm for the rebuilding of the H-RLE Level Set
CHierarchicalRLE* CHierarchicalRLE::advancedDilation( CHierarchicalRLE* structureElement, vector<short int> qv )
{
    if (this->run_types.size() < 1) return NULL;
    if ((this->run_types.size() == 1) && (!RT_DEFINED(this->run_types.at(0))) ) return NULL;

    CHierarchicalRLE* dilatedHRLE = NULL;
    short int qi = qv.back();

    // dilate the highest RLE block which consists only of one segment
    dilatedHRLE = this->dilation1D( 0, structureElement->getSegmentAt(0, false), qi );

    // next the dilation of the whole data structure is started
    dilatedHRLE = this->advancedDilation( dilatedHRLE, structureElement, qv );

    return dilatedHRLE;

}

// dilation algorithm for the rebuilding of the H-RLE Level Set
CHierarchicalRLE* CHierarchicalRLE::advancedDilation( CHierarchicalRLE* dilatedHRLE, CHierarchicalRLE* structureElement, vector<short int> qv, int si  )
{

    int                             run, orgRun, tmpRun, tmpRunSE;
    short int                       nCoord;
    int                             startCoordinate;
    int                             endCoordinate;
    short int                       firstCoord, lastCoord, dilationStart, dilationEnd;
    int                             sePos, definedIndex;
    CHierarchicalRLE                *dilatedSubHRLE = NULL;         // the dilated lower RLE block
    CHierarchicalRLE                *tmpSubSegment = NULL;          // just one dilated subsegment
    int                             indexRunBreak = 0;
    vector<CHierarchicalRLE*>       tmpSubsegments;                 // all dilated subsegments for one coordinate qi
    CHierarchicalRLE                *dilatedSubSegment = NULL;                        // the original subsegment

    short int qi = qv.back();
    qv.pop_back();

    // number of subsegments which must be included in the dilation of the current point
    dilationStart = structureElement->max - qi -1;
    dilationEnd = qi - structureElement->min + 1;

    // start- and endcoordinate of the first run
    startCoordinate = dilatedHRLE->min;
    if ( dilatedHRLE->run_types.size() == 1 ) endCoordinate = dilatedHRLE->max;
    else endCoordinate = dilatedHRLE->run_breaks.at(0);

    // now we dilate the next lower hrle-block (if it exists)
    if ( this->lower_hrle != NULL ) {

            nCoord = qv.back();

            // for all runs in this segment
            for ( int j = 0; j < dilatedHRLE->run_types.size(); j++) {
                run = dilatedHRLE->run_types.at(j);                                         // get run from the dilated hrle segment

                // dilate the subsegments if this run is defined
                if ( RT_DEFINED(run) ) {                                                    // if this dilated run is defined

                    // update the defined run index
                    if ( dilatedSubHRLE == NULL )
                        dilatedHRLE->run_types[j] = 0;
                    else 
                        dilatedHRLE->run_types[j] = dilatedSubHRLE->start_indices.size();

                    // get dilated rle subsegments in the range ym-n to ym+n
                    for ( int ym = startCoordinate; ym < endCoordinate; ym++) {           // traverse all coordinates of this defined run

                        // delete subsegments from the last coordinate
                        tmpSubsegments.clear();

                        // first and last coordinate which affects the current coordinate
                        firstCoord = ym - dilationStart;
                        if ( firstCoord < dilatedHRLE->min ) {
                            firstCoord = dilatedHRLE->min;
                            sePos = qi + (ym - dilatedHRLE->min);
                        }
                        else sePos = structureElement->max - 1;
                        if ( (ym + dilationEnd) >= dilatedHRLE->max ) lastCoord = dilatedHRLE->max;
                        else lastCoord = ym + dilationEnd;

                        // iterate over all coordinates which affect the ym
                        for ( int coord = firstCoord; coord < lastCoord; coord++ ) {

                            dilatedSubSegment = NULL;

                            // get the run where the coordinate sePos in the structure element is included
                            short int startOfRunSE = structureElement->min;
                            tmpRunSE = structureElement->getRunByCoordinate( sePos, 0, &startOfRunSE );

                            // get the run of the coordinate in the original HRLE segment
                            short int startOfRun = this->min;
                            orgRun = this->getRunByCoordinate( coord, si, &startOfRun );

                            // get the sub structure element
                            CHierarchicalRLE* subStructureElement = NULL;
                            if (RT_DEFINED( tmpRunSE ) ) {
                                int defIndexSI = sePos + tmpRunSE - startOfRunSE;
                                subStructureElement = structureElement->lower_hrle->getSegmentAt( defIndexSI );

                            }

                            // dilate the subsegments
                            if ( RT_DEFINED( orgRun ) ) {

                                definedIndex =  orgRun + coord - startOfRun;

                                if ( subStructureElement != NULL) {

                                    // 1D dilation of the subsegment
                                    dilatedSubSegment = this->lower_hrle->dilation1D( definedIndex, subStructureElement, nCoord );

                                    // dilate the lower segments of this subsegment
                                    dilatedSubSegment = this->lower_hrle->advancedDilation(
                                                        dilatedSubSegment, subStructureElement, qv, definedIndex );

                                    // add the result to the dilated subsegments vector
                                    if ( dilatedSubSegment != NULL ) {
                                        tmpSubsegments.push_back( dilatedSubSegment );
                                    }
                                }


                            }

                            if ( subStructureElement != NULL ) {
                                delete subStructureElement;
                                subStructureElement = NULL;
                            }

                            sePos--;
                        }

                        // union of the dilated subsegments
                        tmpSubSegment = this->hrleUnion( tmpSubsegments );

                        // add the new dilated subsegment to the dilated HRLE block
                        if ( tmpSubSegment != NULL ) {
                          if ( dilatedSubHRLE == NULL ) {
                              dilatedSubHRLE = new CHierarchicalRLE( tmpSubSegment->dimension );
                              dilatedSubHRLE->addToHRLE( tmpSubSegment );
                          }
                          else
                              dilatedSubHRLE->addToHRLE( tmpSubSegment );
                        }
                   }
                }

                // start- and endcoordinate of the next run
                startCoordinate = endCoordinate;
                if ( (j+1) >= (dilatedHRLE->run_types.size()-1) ) endCoordinate = dilatedHRLE->max;
                else endCoordinate = dilatedHRLE->run_breaks.at(++indexRunBreak);
            }

        // add the dilated lower RLE block to the dilated H-RLE data structure
        dilatedHRLE->lower_hrle = dilatedSubHRLE;

        // update dimension
        dilatedHRLE->dimension = this->dimension;
    }
    else {
        dilatedHRLE->lower_hrle = NULL;
    }

    return dilatedHRLE;
}

// the 1D dilation algorithm for the advanced dilation of the HRLE object
CHierarchicalRLE* CHierarchicalRLE::dilation1D( int si, CHierarchicalRLE* structureElement, short int nCoord ) {
    CHierarchicalRLE*     dilatedHRLE = NULL;                         // the dilated hrle level set
    CHierarchicalRLE*     currentSegment;                             // the original segment
    CHierarchicalRLE*     tmpHRLE;
    CHierarchicalRLE*     tmpSI;
    int                   run, tmpRun;
    short int             startCoordinate;
    short int             endCoordinate;
    short int             firstBreak;
    short int             runs, coords, i;
    short int             difference;
    bool                  noMidsection = false;

    // get the segment
    currentSegment = this->getSegmentAt( si, false );

    if ( structureElement == NULL ) {
        return currentSegment;
    }

    // copy the upper segment of the structure element
    tmpSI = new CHierarchicalRLE();
    tmpSI->min = structureElement->min;
    tmpSI->max = structureElement->max;
    tmpSI->run_breaks = structureElement->run_breaks;
    tmpSI->run_types = structureElement->run_types;
    tmpSI->start_indices.push_back(0);

    if ( !RT_DEFINED(tmpSI->getRunByCoordinate(nCoord, 0) ) ) noMidsection = true;

    // start and end coordinate of the first run (end coordinate don't belong to the run!)
    startCoordinate = currentSegment->min;
    if ( currentSegment->run_types.size() == 1 )  endCoordinate = currentSegment->max;
    else endCoordinate = currentSegment->run_breaks.at( 0 );                        // set endCoordinate for the first run to the first break

    if ( noMidsection ) {
        dilatedHRLE = new CHierarchicalRLE();
        dilatedHRLE->min = currentSegment->min;
        dilatedHRLE->max = currentSegment->max;
        dilatedHRLE->run_types.push_back(RT_POSITIVE);
        dilatedHRLE->start_indices.push_back(0);
    }

    // dilate the defined runs of this segment and create a new RLE segment
    for (runs = 0; runs < currentSegment->run_types.size(); runs++) {
        run = currentSegment->run_types.at(runs);                                   // get run

        if ( RT_DEFINED(run) ) {
            for (coords = startCoordinate; coords < endCoordinate; coords++) {
                // setup element of structure for the current point
                difference = coords - nCoord;
                if ( difference != 0 ) {
                    for (i = 0; i < tmpSI->run_breaks.size(); i++)
                        tmpSI->run_breaks[i] += difference;
                    tmpSI->min += difference;
                    tmpSI->max += difference;
                }

                if ( noMidsection ) {
                        tmpHRLE = new CHierarchicalRLE();
                        tmpHRLE->min = tmpSI->min;
                        tmpHRLE->max = tmpSI->max;
                        tmpHRLE->run_breaks = tmpSI->run_breaks;
                        tmpHRLE->run_types = tmpSI->run_types;
                        tmpHRLE->start_indices.push_back(0);
                }
                else tmpHRLE = this->uniteSegments( currentSegment, tmpSI );

                if ( dilatedHRLE == NULL ) {
                    dilatedHRLE = tmpHRLE;
                }
                else {

                    dilatedHRLE = this->hrleUnion( dilatedHRLE, tmpHRLE );
                }

                nCoord = coords;
            }
        }

        // start coordinate of the next run
        startCoordinate = endCoordinate;
        // end coordinate of the next run
        if ((runs+1) >= currentSegment->run_breaks.size()) endCoordinate = currentSegment->max;
        else endCoordinate = currentSegment->run_breaks.at( runs+1 );
    }

    delete tmpSI;
    delete currentSegment;

    return dilatedHRLE;
}

// union of two 1D segments (for 1D dilation)
CHierarchicalRLE* CHierarchicalRLE::uniteSegments( CHierarchicalRLE* currentSegment, CHierarchicalRLE* structureElement ) {

    CHierarchicalRLE *returnValue = new CHierarchicalRLE();;      // the result
    int pos1 = 0, pos2 = 0;                                       // indices into the run arrays of the segments
    int startCoord, endCoord;
    int startCoord1, endCoord1, startCoord2, endCoord2;           // start and end coordinates of the different runs of both segments
    int runBreak1, runBreak2;
    int newRun, runFromOtherBlock;
    vector<CHierarchicalRLE*>       tmpSubsegments;
    int lastRun = RT_NEGATIVE;

    // setup the return value
      // set dimension
      returnValue->dimension = 1;
      // initialize start indices array
      returnValue->start_indices.push_back( 0 );

      // update min & max
      if ( currentSegment->min < structureElement->min ) returnValue->min = currentSegment->min;
      else returnValue->min = structureElement->min;
      if ( currentSegment->max > structureElement->max ) returnValue->max = currentSegment->max;
      else returnValue->max = structureElement->max;

    // now we need to compare the different HRLE blocks
      // initialize start and end coordinates (first run)
      startCoord1 = currentSegment->min;
      startCoord2 = structureElement->min;
      if ( currentSegment->run_types.size() < 2 ) endCoord1 = currentSegment->max;
      else endCoord1 = currentSegment->run_breaks.at( 0 );
      if ( structureElement->run_types.size() < 2 ) endCoord2 = structureElement->max;
      else endCoord2 = structureElement->run_breaks.at( 0 );

      // traverse through all runs of both H-RLE blocks
      while ( ( pos1 < currentSegment->run_types.size()) || (pos2 < structureElement->run_types.size()) ) {

          // get the breaks of both runs
          if ( pos1 >= currentSegment->run_breaks.size() ) runBreak1 = currentSegment->max;
          else runBreak1 = currentSegment->run_breaks.at(pos1);
          if ( pos2 >= structureElement->run_breaks.size() ) runBreak2 = structureElement->max;
          else runBreak2 = structureElement->run_breaks.at(pos2);

            // first segment starts before the second
            if ( ((pos2 == 0) && (startCoord1 < startCoord2)) ) {
                returnValue->run_types.push_back( currentSegment->run_types.at(pos1) );

                if ( startCoord2 < endCoord1 ) returnValue->run_breaks.push_back( startCoord2 );
                else returnValue->run_breaks.push_back( endCoord1 );
                if (endCoord1 > returnValue->run_breaks.back()) startCoord1 = returnValue->run_breaks.back();
                else runBreak1 = returnValue->run_breaks.back();
            }

            // otherwise
            else if ( ((pos1 == 0) && (startCoord2 < startCoord1)) ) {
                returnValue->run_types.push_back( structureElement->run_types.at(pos2) );

                if ( startCoord1 < endCoord2 ) returnValue->run_breaks.push_back( startCoord1 );
                else returnValue->run_breaks.push_back( endCoord2 );
                if (endCoord2 > returnValue->run_breaks.back()) startCoord2 = returnValue->run_breaks.back();
                else runBreak2 = returnValue->run_breaks.back();
            }

          // add the remaining runs and breaks of the structure element
          else if ( (pos1 >= currentSegment->run_types.size()) &&
               (pos2 < structureElement->run_types.size()) ) {

              if (runBreak2 > returnValue->run_breaks.back() ) {
                  for ( int i = pos2; i < structureElement->run_types.size(); i++ ) {
                      newRun = structureElement->run_types.at(i);

                      if ( returnValue->run_breaks.size() > 0 ) startCoord = returnValue->run_breaks.back();
                      else startCoord = returnValue->min;

                      if ( (returnValue->run_types.size() > 0) &&
                            this->sameRunType(lastRun, newRun)/* ||
                              ( RT_DEFINED(lastRun)  && RT_DEFINED(newRun) ))*/ ) {
                              returnValue->run_breaks.pop_back();
                      }

                      // if the run is defined, add the subsegments to the result
                      if ( RT_DEFINED(newRun) ) {
                          newRun = 1;

                          if ( endCoord > returnValue->max ) returnValue->max = endCoord;

                          if ( i < structureElement->run_breaks.size() ) endCoord = structureElement->run_breaks.at(i);
                          else endCoord = structureElement->max;
                      }

                      if ( (i < structureElement->run_breaks.size()) &&
                          (structureElement->run_breaks.at(i) != returnValue->max) &&
                          ( (returnValue->run_breaks.size() == 0) ||
                            (structureElement->run_breaks.at(i) > returnValue->run_breaks.back()) ) /*&&
                            !(this->sameRunType(lastRun, newRun))*/ )
                          returnValue->run_breaks.push_back( structureElement->run_breaks.at(i) );

                      if ( (returnValue->run_types.size() == 0) ||
                            (!(this->sameRunType(lastRun, newRun))) )
                          returnValue->run_types.push_back( newRun );

                      lastRun = newRun;
                  }
                  pos2 = structureElement->run_types.size();
              }
          }

          // add the remaining runs and breaks of the segment
          else if ( (pos2 >= structureElement->run_types.size()) &&
                    (pos1 < currentSegment->run_types.size()) ) {

              if (runBreak1 > returnValue->run_breaks.back()) {

                  for ( int i = pos1; i < currentSegment->run_types.size(); i++ ) {
                      newRun = currentSegment->run_types.at(i);

                      if ( returnValue->run_breaks.size() > 0 ) startCoord = returnValue->run_breaks.back();
                      else startCoord = returnValue->min;

                      if ( (returnValue->run_types.size() > 0) &&
                          ( this->sameRunType(lastRun, newRun)) ) {
                              returnValue->run_breaks.pop_back();
                      }

                      // if the run is defined, add the subsegments to the result
                      if ( RT_DEFINED(newRun) ) {
                          newRun = 1;
                          if ( i < currentSegment->run_breaks.size() ) endCoord = currentSegment->run_breaks.at(i);
                          else endCoord = currentSegment->max;
                      }

                      if ( (i < currentSegment->run_breaks.size()) &&
                          (currentSegment->run_breaks.at(i) != returnValue->max) &&
                          ( (returnValue->run_breaks.size() == 0) ||
                            (currentSegment->run_breaks.at(i) > returnValue->run_breaks.back()) ) /*&&
                            !(this->sameRunType(lastRun, newRun))*/ )
                          returnValue->run_breaks.push_back( currentSegment->run_breaks.at(i) );

                      if ( (returnValue->run_types.size() == 0) ||
                            (!(this->sameRunType(lastRun, newRun))) )
                          returnValue->run_types.push_back( newRun );

                      lastRun = newRun;
                  }
                  pos1 = currentSegment->run_types.size();
              }
          }


          // compare the runs of both HRLE blocks
          else if ( ( pos1 < currentSegment->run_types.size()) && (pos2 < structureElement->run_types.size())) {

              if ( (returnValue->run_breaks.size() == 0) ||
                   ((runBreak1 > returnValue->run_breaks.back()) &&
                    (runBreak2 > returnValue->run_breaks.back())) ) {

                  // if both runs have the same value
                  if ( this->sameRunType(currentSegment->run_types.at(pos1),
                                         structureElement->run_types.at(pos2)) ) {

                      newRun = currentSegment->run_types[pos1];

                      if ( returnValue->run_breaks.size() > 0 ) startCoord = returnValue->run_breaks.back();
                      else startCoord = returnValue->min;

                      // if the last run has the same type, don't add a new run
                      if ( (returnValue->run_types.size() > 0) &&
                           this->sameRunType(lastRun, newRun) ) {
                          returnValue->run_breaks.pop_back();
                      }

                      // if defined, use the higher break and get the subsegments
                      if ( RT_DEFINED(newRun) ) {

                          if ( runBreak1 > runBreak2 ) returnValue->run_breaks.push_back( runBreak1 );
                          else returnValue->run_breaks.push_back( runBreak2 );

                          endCoord = returnValue->run_breaks.back();

                          newRun = 1;
                      }
                      // if undefined, use the lower break
                      else {
                          if ( runBreak1 < runBreak2 ) returnValue->run_breaks.push_back( runBreak1 );
                          else returnValue->run_breaks.push_back( runBreak2 );
                      }

                      if ( (returnValue->run_types.size() == 0) ||
                           (!(this->sameRunType(lastRun, newRun)) &&
                            !( RT_DEFINED(lastRun) && RT_DEFINED(newRun))) ) {
                          // add a new run
                          returnValue->run_types.push_back( newRun );
                      }

                  }

                  // else choose at first defined run, than inner run and at last outer run
                  else {

                      if ( returnValue->run_breaks.size() > 0 ) startCoord = returnValue->run_breaks.back();
                      else startCoord = returnValue->min;

                      if ( RT_DEFINED(currentSegment->run_types[pos1]) ) {

                          newRun = 1;

                          // same run so we don't need the last break
                          if ( (returnValue->run_types.size() > 0) &&
                              (this->sameRunType(lastRun, newRun)) ) {
                              returnValue->run_breaks.pop_back();
                          }
                          returnValue->run_breaks.push_back( runBreak1 );

                          endCoord = returnValue->run_breaks.back();

                      }
                      else if ( RT_DEFINED(structureElement->run_types[pos2]) ) {

                          newRun = 1;

                          // same run so we don't need the last break
                          if ( (returnValue->run_types.size() > 0) &&
                                (this->sameRunType(lastRun, newRun)) ) {
                              returnValue->run_breaks.pop_back();
                          }

                          returnValue->run_breaks.push_back( runBreak2 );

                          endCoord = returnValue->run_breaks.back();

                      }
                      // first run is an inner run, second an outer
                      else if ( currentSegment->run_types[pos1] == RT_NEGATIVE ) {
                          newRun = RT_NEGATIVE;
                          if ( (returnValue->run_types.size() > 0) &&
                                (this->sameRunType(lastRun, newRun)) ) {
                              returnValue->run_breaks.pop_back();
                          }
                          if ( runBreak1 < runBreak2 ) returnValue->run_breaks.push_back( runBreak1 );
                          else returnValue->run_breaks.push_back( runBreak2 );
                      }
                      // second run is an inner run, first an outer
                      else if ( structureElement->run_types[pos2] == RT_NEGATIVE ) {
                          newRun = RT_NEGATIVE;
                          if ( (returnValue->run_types.size() > 0) &&
                                (this->sameRunType(lastRun, newRun)) ) {
                              returnValue->run_breaks.pop_back();
                          }
                          if ( runBreak1 < runBreak2 ) returnValue->run_breaks.push_back( runBreak1 );
                          else returnValue->run_breaks.push_back( runBreak2 );
                      }
                      // well ... that shouldn't happen :)
                      else {
                          cout << "Error during union (1D dilation)!" << endl;
                          cout << "currentSegment Type: " << currentSegment->run_types[pos1] << endl;
                          cout << "currentHRLE Type: " << structureElement->run_types[pos2] << endl;
                          break;
                      }

                      if ( (returnValue->run_types.size() == 0) ||
                           (!(this->sameRunType(lastRun, newRun))) ) {
                          // add a new run
                          returnValue->run_types.push_back( newRun );
                      }
                  }
              }
          }
          else {
              pos1 = currentSegment->run_types.size();
              pos2 = structureElement->run_types.size();
          }

          // update pos -> next run if the last break in the result is greater then the break of this run
          // if the break of the run in tempValue is greater as the last break in the result, go to the next run in the currentHRLE
          if ( returnValue->run_breaks.size() > 0 ) {
              if ( ((runBreak1 <= returnValue->run_breaks.back()) &&
                    (runBreak2 <= returnValue->run_breaks.back()))) {
                  pos1++;
                  pos2++;

                  // update start and end coordinate of the next run
                  startCoord1 = endCoord1;
                  startCoord2 = endCoord2;
                  if ( pos1 < currentSegment->run_breaks.size()) endCoord1 = currentSegment->run_breaks.at(pos1);
                  else endCoord1 = currentSegment->max;
                  if ( pos2 < structureElement->run_breaks.size()) endCoord2 = structureElement->run_breaks.at(pos2);
                  else endCoord2 = structureElement->max;
              }
              else if ( (runBreak1 > returnValue->run_breaks.back()) &&
                        (runBreak2 <= returnValue->run_breaks.back()) ) {
                  pos2++;

                  // update start and end coordinate of the next run
                  startCoord2 = endCoord2;
                  if ( pos2 < structureElement->run_breaks.size()) endCoord2 = structureElement->run_breaks.at(pos2);
                  else endCoord2 = structureElement->max;
              }
              else if ( (runBreak2 > returnValue->run_breaks.back()) &&
                        (runBreak1 <= returnValue->run_breaks.back()) ) {
                  pos1++;

                  // update start and end coordinate of the next run
                  startCoord1 = endCoord1;
                  if ( pos1 < currentSegment->run_breaks.size()) endCoord1 = currentSegment->run_breaks.at(pos1);
                  else endCoord1 = currentSegment->max;
              }
          }

          lastRun = returnValue->run_types.back();

      }

      // deleta last break if it is == maximum extend
      if ( (returnValue->run_breaks.size() > 0) &&
         (returnValue->run_breaks.back() == returnValue->max) ) returnValue->run_breaks.pop_back();

    return returnValue;

}

/*!
    \fn CHierarchicalRLE::dilation( vector<bool> structureElement )
 */
// dilation algorithm for the rebuilding of the H-RLE Level Set (old version)
CHierarchicalRLE* CHierarchicalRLE::dilation( vector<bool> structureElement )
{
    if (this->run_types.size() < 1) return NULL;
    if ((this->run_types.size() == 1) && (!RT_DEFINED(this->run_types.at(0))) ) return this;

    CHierarchicalRLE* dilatedHRLE = NULL;
    bool left = false, middle = false, right = false;
    int x,y,z,i;

    for ( int i = 0; i < structureElement.size(); i++ ) cout << structureElement.at(i) << ", ";
    cout << endl;

    // set variables for 1D dilation of the z axis
    for ( i = 0; i < 9; i++ ) {
      if ( structureElement.at(i) ) {
        left = true;
        break;
      }
    }
    for ( i = 9; i < 18; i++ ) {
      if ( structureElement.at(i) ) {
        middle = true;
        break;
      }
    }
    for ( i = 18; i < structureElement.size(); i++ ) {
      if ( structureElement.at(i) ) {
        right = true;
        break;
      }
    }

    // dilate the highest RLE block which consists only of one segment
    dilatedHRLE = this->dilateSegment( 0, left, middle, right );

    if ( dilatedHRLE == NULL ) {
        dilatedHRLE = new CHierarchicalRLE();
        dilatedHRLE->min = 0;
        dilatedHRLE->max = 1;
        int run;
        run = RT_NEGATIVE;
        dilatedHRLE->run_types.push_back( run );
        dilatedHRLE->start_indices.push_back( 0 );
        cout << "Dilation of z axis failed!!!" << endl;
    }


    // next the dilation of the whole data structure is started
    dilatedHRLE = this->dilation( dilatedHRLE, structureElement, left, middle, right );

    return dilatedHRLE;

}


/*!
    \fn CHierarchicalRLE::dilation( CHierarchicalRLE dilatedHRLE, int pointCount )
 */
// dilation algorithm for the rebuilding of the H-RLE Level Set (old version)
CHierarchicalRLE* CHierarchicalRLE::dilation( CHierarchicalRLE* dilatedHRLE, vector<bool> structureElement, bool left, bool middle, bool right, int si )
{

    vector<bool>                    leftElement, middleElement, rightElement;
    int                             x,z,i;
    int                             run;
    int                             startCoordinate;
    int                             endCoordinate;
    int                             tmpRun;
    CHierarchicalRLE                *dilatedSubHRLE = NULL;         // the dilated lower RLE block
    CHierarchicalRLE                *tmpSubSegment = NULL;          // just one dilated subsegment
    int                             indexRunBreak = 0;
    vector<CHierarchicalRLE*>       tmpSubsegments;                 // all dilated subsegments for one coordinate qi

    // start- and endcoordinate of the first run
    startCoordinate = dilatedHRLE->min;
    if ( dilatedHRLE->run_types.size() < 2 ) endCoordinate = dilatedHRLE->max;
    else endCoordinate = dilatedHRLE->run_breaks.at(0);

    // setup dilation for the y axis
    if ( this->dimension == 3) {
      for ( i = 0; i < 9; i++) {
        leftElement.push_back(structureElement.at(i));
      }
      for ( i = 9; i < 18; i++) {
        middleElement.push_back(structureElement.at(i));
      }
      for ( i = 18; i < structureElement.size(); i++) {
        rightElement.push_back(structureElement.at(i));
      }
    }

    // setup dilation for the x axis
    if ( this->dimension == 2) {
      for ( i = 0; i < 3; i++) {
        leftElement.push_back(structureElement.at(i));
      }
      for ( i = 3; i < 6; i++) {
        middleElement.push_back(structureElement.at(i));
      }
      for ( i = 6; i < structureElement.size(); i++) {
        rightElement.push_back(structureElement.at(i));
      }
    }

    // now we dilate the next lower hrle-block (if it exists)
    if ( this->lower_hrle != NULL ) {

            // for all runs in this segment
            for ( int j = 0; j < dilatedHRLE->run_types.size(); j++) {
                run = dilatedHRLE->run_types.at(j);                                                // get run from the dilated hrle segment

                // dilate the subsegments if this run is defined
                if ( RT_DEFINED(run) ) {                                                    // if this dilated run is defined

                    // update the defined run index
                    if ( dilatedSubHRLE == NULL )
                        dilatedHRLE->run_types[j] = 0;
                    else 
                        dilatedHRLE->run_types[j] = dilatedSubHRLE->start_indices.size();

                    // get dilated rle subsegments in the range ym-n to ym+n
                    for ( int ym = startCoordinate; ym < endCoordinate; ym++) {           // traverse all coordinates of this defined run
                      tmpSubsegments.clear();                                             // delete subsegments from the last coordinate

                      if ( left ) {
                        if ( ((ym+1) >= dilatedHRLE->min)/* && ((ym-1) < dilatedHRLE->max)*/ ) {
                          CHierarchicalRLE* tmpSegment = this->getDilatedSubsegment( ym+1, si, leftElement );
                          if ( tmpSegment != NULL ) tmpSubsegments.push_back( tmpSegment );
                        }
                      }
                      if ( middle ) {
                        CHierarchicalRLE* tmpSegment = this->getDilatedSubsegment( ym, si, middleElement );
                        if ( tmpSegment != NULL ) tmpSubsegments.push_back( tmpSegment );
                      }
                      if ( right ) {
                        if ( /*((ym+1) >= dilatedHRLE->min) && */((ym-1) < dilatedHRLE->max) ) {
                          CHierarchicalRLE* tmpSegment = this->getDilatedSubsegment( ym-1, si, rightElement );
                          if ( tmpSegment != NULL ) tmpSubsegments.push_back( tmpSegment );
                        }
                      }

                      // union of the dilated subsegments
                      tmpSubSegment = this->hrleUnion( tmpSubsegments );

                      // add the new dilated subsegment to the dilated HRLE block
                      if ( tmpSubSegment != NULL ) {
                        if ( dilatedSubHRLE == NULL )
                            dilatedSubHRLE = tmpSubSegment;
                        else
                            dilatedSubHRLE->addToHRLE( tmpSubSegment );
                      }
                   }
                }

                // start- and endcoordinate of the next run
                startCoordinate = endCoordinate;
                if ( (j+1) >= (dilatedHRLE->run_types.size()-1) ) endCoordinate = dilatedHRLE->max;
                else endCoordinate = dilatedHRLE->run_breaks.at(++indexRunBreak);
            }

        // add the dilated lower RLE block to the dilated H-RLE data structure
        dilatedHRLE->lower_hrle = dilatedSubHRLE;

        // update dimension
        dilatedHRLE->dimension = this->dimension;
        return dilatedHRLE;
    }

    return NULL;
}

// dilation of an n-D subsegment (old algorithm)
CHierarchicalRLE* CHierarchicalRLE::getDilatedSubsegment( int qi, int si, vector<bool> structureElement ) {

  CHierarchicalRLE* tmpSegment = NULL;
  bool              left = false, middle = false, right = false;
  int               i,z;
  int               tmpRun;

  if ((qi < this->min) || (qi > this->max)) return NULL;

  // set variables for 1D dilation of the y axis
  if ( this->dimension == 3 ) {
    for ( i = 0; i < 3; i++ ) {
      if ( structureElement.at(i) ) {
        left = true;
        break;
      }
    }
    for ( i = 3; i < 6; i++ ) {
      if ( structureElement.at(i) ) {
        middle = true;
        break;
      }
    }
    for ( i = 6; i < structureElement.size(); i++ ) {
      if ( structureElement.at(i) ) {
        right = true;
        break;
      }
    }
  }

  // set variables for 1D dilation of the x axis
  if ( this->dimension == 2 ) {
    left = structureElement.at(0);
    middle = structureElement.at(1);
    right = structureElement.at(2);
  }

  short int startOfRun = this->min;
  tmpRun = this->getRunByCoordinate( qi, si, &startOfRun );
  if ( RT_DEFINED(tmpRun) ) {                                 // check if the corresponding run is defined in the original RLE segment
      int definedIndex = tmpRun + qi - startOfRun;            // compute defined data index -> segment index of the corresponding lower hrle segment

      tmpSegment = this->lower_hrle->dilateSegment( definedIndex, left, middle, right );  // dilate the subsegment

if ( tmpSegment == NULL ) cout << "dilateSegment failed!!!" << endl;

      if ( tmpSegment != NULL ) {
        // recursive call for the next lower dimension
        if ( this->lower_hrle->lower_hrle != NULL ) {
            // dilation of the next lower dimension
            tmpSegment = this->lower_hrle->dilation( tmpSegment, structureElement, left, middle, right, definedIndex ); 
        }
      }
  }

if ( (tmpSegment == NULL) && ((left || middle || right) == false) ) cout << "Dilation of Subsegment failed!!!" << endl;

  return tmpSegment;
}

// 1D erosion algorithm
CHierarchicalRLE* CHierarchicalRLE::erosion1D( int si, CHierarchicalRLE* structureElement, short int nCoord ) {

    if ( structureElement == NULL ) return NULL;

    CHierarchicalRLE*   returnValue = new CHierarchicalRLE();
    short int           coord, startCoord, endCoord;
    int                 run, iRun, firstRun, firstBreak, segmentEnd;
    short int           difference, i;
    bool                middle = false;

    // get the index of the first run in this segment
    try {
        firstRun = this->start_indices.at(si);
    }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
        cerr << "Out of range exception " << e.what() << endl;
        cerr << "1D erosion: no segment with index " << si << endl;
        exit(1);
    }

    // get the index of the first run break of this segment
    firstBreak = this->start_indices.at(si) - si;

    // segment end
    if ( (si+1) >= this->start_indices.size() ) segmentEnd = this->run_types.size();
    else {
        try {
            segmentEnd = this->start_indices.at(si+1);
        }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
            cerr << "Out of range exception " << e.what() << endl;
            cerr << "1D erosion: error while setting up segment end" << endl;
            exit(1);
        }
    }

    // start and end coordinates of the first run
    startCoord = this->min;
    if ( (firstRun+1) >= segmentEnd ) endCoord = this->max;
    else {
        try {
            endCoord = this->run_breaks.at( firstBreak++ );
        }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
            cerr << "Out of range exception " << e.what() << endl;
            cerr << "1D erosion: no break with index " << firstBreak-1 << endl;
            exit(1);
        }
    }

    // setup return value
    returnValue->min = this->min;
    returnValue->max = this->max;
    returnValue->start_indices.push_back(0);

    run = structureElement->getRunByCoordinate( (int)nCoord );
    if ( RT_DEFINED(run) ) middle = true;

    // iterate over all runs
    for ( iRun = firstRun; iRun < segmentEnd; iRun++ ) {

        run = this->run_types.at( iRun );

        if ( RT_DEFINED( run ) || !middle ) {
            for ( coord = startCoord; coord < endCoord; coord++ ) {
                // setup element of structure for the current point
                difference = coord - nCoord;
                if ( difference != 0 ) {
                    for (i = 0; i < structureElement->run_breaks.size(); i++)
                        structureElement->run_breaks[i] += difference;
                    structureElement->min += difference;
                    structureElement->max += difference;
                }

                // check if this coordinate should remain defined
                if ( this->coordinateIsSignificant( structureElement, si ) ) {
                    if ( (returnValue->run_types.size() == 0) || (!RT_DEFINED(returnValue->run_types.back())) ) {
                        returnValue->run_types.push_back( 1 );
                    }
                    else if (RT_DEFINED(returnValue->run_types.back())) {
                        if ( returnValue->run_breaks.size() > 0 ) returnValue->run_breaks.pop_back();
                    }
                }

                // add an non defined run for this coordinate
                else {
                    if ( RT_DEFINED( run ) ) {
                        if ( returnValue->run_types.size() == 0 ) {
                            returnValue->run_types.push_back( RT_POSITIVE );
                        }
                        else if ( RT_DEFINED(returnValue->run_types.back()) ) {
                            returnValue->run_types.push_back( RT_NEGATIVE );
                        }
                        else {
                            if ( returnValue->run_breaks.size() > 0 ) {
                                try {
                                    returnValue->run_breaks.pop_back();
                                }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
                                    cerr << "Out of range exception " << e.what() << endl;
                                    cerr << "1D erosion: removing break (defined run)" << endl;
                                    exit(1);
                                }
                            }
                        }
                    }
                    else {

                        if ( returnValue->run_types.size() == 0 ) {
                            returnValue->run_types.push_back( run );
                        }

                        else if (!(this->sameRunType(returnValue->run_types.back(), run)) ) {

                            if ( (returnValue->run_types.back() == RT_NEGATIVE) && (run == RT_POSITIVE) ) {
                                try {
                                    returnValue->run_types.pop_back();
                                    if ( returnValue->run_breaks.size() > 0 ) returnValue->run_breaks.pop_back();
                                }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
                                    cerr << "Out of range exception " << e.what() << endl;
                                    cerr << "1D erosion: positive after negative run" << endl;
                                    exit(1);
                                }
                            }
                            else if ( (returnValue->run_types.back() == RT_POSITIVE) && (run == RT_NEGATIVE) ) {
                                returnValue->run_types.pop_back();
                            }
                            returnValue->run_types.push_back( run );
                        }

                        else if ( returnValue->run_breaks.size() > 0 ) {
                            try {
                                returnValue->run_breaks.pop_back();
                            }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
                                cerr << "Out of range exception " << e.what() << endl;
                                cerr << "1D erosion: removing break (undefined run)" << endl;
                                exit(1);
                            }
                        }
                    }
                }

                // update nCoord
                nCoord = coord;

                // add the break
                if ( (coord+1) < returnValue->max ) returnValue->run_breaks.push_back( coord+1);
            }
        }

        else {
            if ( returnValue->run_types.size() == 0 ) {
                returnValue->run_types.push_back( run );
            }
            else if (!(this->sameRunType(returnValue->run_types.back(), run)) ) {
                if ( (returnValue->run_types.back() == RT_NEGATIVE) && (run == RT_POSITIVE) ) {
                    try {
                        returnValue->run_types.pop_back();
                        if ( returnValue->run_breaks.size() > 0 ) returnValue->run_breaks.pop_back();
                    }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
                        cerr << "Out of range exception " << e.what() << endl;
                        cerr << "1D erosion: positive after negative run" << endl;
                        exit(1);
                    }
                }
                else if ( (returnValue->run_types.back() == RT_POSITIVE) && (run == RT_NEGATIVE) ) {
                    returnValue->run_types.pop_back();
                }
                returnValue->run_types.push_back( run );
            }
            else if ( returnValue->run_breaks.size() > 0 ) {
                try {
                    returnValue->run_breaks.pop_back();
                }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
                    cerr << "Out of range exception " << e.what() << endl;
                    cerr << "1D erosion: removing break (undefined run)" << endl;
                    exit(1);
                }
            }

            // add the break for this run
            if ( endCoord < returnValue->max ) returnValue->run_breaks.push_back( endCoord );
        }

        startCoord = endCoord;
        if ( (iRun+2) >= segmentEnd ) endCoord = this->max;
        else {
            try {
                endCoord = this->run_breaks.at( firstBreak++ );
            }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
                cerr << "Out of range exception " << e.what() << endl;
                cerr << "1D erosion: no break with index " << firstBreak-1 << endl;
                exit(1);
            }
        }

    }

    delete structureElement;
    structureElement = NULL;

    if ( (returnValue->run_types.size() == 1) && (!RT_DEFINED(returnValue->run_types.back())) ) {
        delete returnValue;
        return NULL;
    }

    if ( (returnValue->run_types.size() > 0) && (returnValue->run_types.back() == RT_NEGATIVE) ) {
        returnValue->run_types.pop_back();
        returnValue->run_types.push_back( RT_POSITIVE );
    }

    returnValue->dimension = 1;
    return returnValue;
}

// this is for the erosion algorithm
// returns true, if all defined coordinates in the structure element exist in the given (1D) segment
bool CHierarchicalRLE::coordinateIsSignificant( CHierarchicalRLE *structElement, int si ) {
    int           runSE, run, iRun;
    short int     startCoord, endCoord, coord;

    startCoord = structElement->min;
    if ( structElement->run_breaks.size() == 0 ) endCoord = structElement->max;
    else endCoord = structElement->run_breaks.at(0);

    for ( iRun = 0; iRun < structElement->run_types.size(); iRun++ ) {
        runSE = structElement->run_types.at( iRun );
        if ( RT_DEFINED( runSE ) ) {
            for ( coord = startCoord; coord < endCoord; coord++ ) {
                run = this->getRunByCoordinate( coord, si );
                if ( !RT_DEFINED( run ) ) return false;
            }
        }
        startCoord = endCoord;
        if ( (iRun+1) >= structElement->run_breaks.size() ) endCoord = structElement->max;
        else endCoord = structElement->run_breaks.at(iRun+1);
    }

    return true;
}

// dilation algorithm for the rebuilding of the H-RLE Level Set
 CHierarchicalRLE* CHierarchicalRLE::erosion( CHierarchicalRLE* structureElement, vector<short int> qv, int si )
{
    CHierarchicalRLE*             erodedHRLE = NULL;
    CHierarchicalRLE*             erodedSubSegment = NULL;
    CHierarchicalRLE*             tmpSubSegment = NULL;
    CHierarchicalRLE*             subStructureElement = NULL;
    vector<CHierarchicalRLE*>     tmpSubsegments;
    int                           run, iRun, runSE, posSE, tmpRun;
    short int                     ym, coord, startCoord, endCoord, runStartCoord, runStartCoordSE;
    int                           firstRun, firstBreak, segmentEnd;
    short int                     erosionStart, erosionEnd;
    short int                     qi = qv.back();
    int                           definedIndex, defIndexSE;
    bool                          middle = false;
    qv.pop_back();

    if ( this->lower_hrle != NULL ) {

        // get the index of the first run in this segment
        firstRun = this->start_indices.at(si);

        // get the index of the first run break of this segment
        firstBreak = this->start_indices.at(si) - si;

        // segment end
        if ( (si+1) >= this->start_indices.size() ) segmentEnd = this->run_types.size();
        else segmentEnd = this->start_indices.at(si+1);

        // start and end coordinates of the first run
        startCoord = this->min;
        if ( (firstRun+1) >= segmentEnd ) endCoord = this->max;
        else endCoord = this->run_breaks.at( firstBreak++ );

        // setup return value
        erodedHRLE = new CHierarchicalRLE();
        erodedHRLE->dimension = this->dimension;
        erodedHRLE->min = this->min;
        erodedHRLE->max = this->max;
        erodedHRLE->start_indices.push_back(0);

        run = structureElement->getRunByCoordinate(qi);
        if ( RT_DEFINED(run) ) middle = true;

        for ( iRun = firstRun; iRun < segmentEnd; iRun++ ) {

            // get the run
            try {
                run = this->run_types.at( iRun );
            }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
                cerr << "Out of range exception " << e.what() << endl;
                cerr << "Run with index " << iRun << " does not exist!" << endl;
                exit(1);
            }

            // run is defined, so check if it's coordinates are relevant
            if ( RT_DEFINED( run ) || !middle ) {

                  for ( ym = startCoord; ym < endCoord; ym++ ) {

                      // initialize current coordinate in the structure element
                      posSE = structureElement->min;

                      // setup start and end coordinates for the erosion
                      erosionStart = ym - (qi - structureElement->min);
                      erosionEnd = ym + (structureElement->max - qi);

                      // clear temporary sub segments
                      tmpSubsegments.clear();

                      for ( coord = erosionStart; coord < erosionEnd; coord++ ) {

                          tmpRun = this->getRunByCoordinate( coord, si, &runStartCoord );
                          runSE = structureElement->getRunByCoordinate( posSE, 0, &runStartCoordSE );

                          // coordinate is in the structure element defined, but not in the HRLE object -> ym not relevant
                          if ( RT_DEFINED(runSE) && !RT_DEFINED(tmpRun) ) {
                              tmpSubsegments.clear();
                              break;
                          }
                          // get eroded sub segment if both defined
                          else if ( RT_DEFINED(runSE) && RT_DEFINED(tmpRun) ) {
                              definedIndex = coord + tmpRun - runStartCoord;
                              defIndexSE = posSE + runSE - runStartCoordSE;

                              subStructureElement = structureElement->lower_hrle->getSegmentAt(defIndexSE);

                              // recursive call for the lower RLE block
                              if ( this->lower_hrle->lower_hrle != NULL ) {
                                  tmpSubSegment = this->lower_hrle->erosion(subStructureElement, qv, definedIndex);
                                  tmpSubsegments.push_back( tmpSubSegment );
                              }
                              else {
                                  tmpSubSegment = this->lower_hrle->erosion1D( definedIndex, subStructureElement, qv.back() );
                                  tmpSubsegments.push_back( tmpSubSegment );

                              }
                          }

                          posSE++;

                      }

                      // intersect the eroded sub segments
                      if ( tmpSubsegments.size() > 0 ) {

                          erodedSubSegment = this->hrleIntersection( tmpSubsegments );

                          if (  (erodedSubSegment != NULL) &&
                                (erodedSubSegment->run_types.size() == 1) &&
                                ( !RT_DEFINED(erodedSubSegment->run_types.back() )) ) {
                              delete erodedSubSegment;
                              erodedSubSegment = NULL;
                          }
                      }
                      else {
                          erodedSubSegment = NULL;
                      }

                      // if erodedSubSegment != NULL than add an defined run
                      if ( erodedSubSegment != NULL ) {

                        if ( ( erodedHRLE->run_types.size() == 0 ) || ( !RT_DEFINED(erodedHRLE->run_types.back()) ) ) {
                            if ( erodedHRLE->lower_hrle == NULL ) erodedHRLE->run_types.push_back( 0 );
                            else erodedHRLE->run_types.push_back( erodedHRLE->lower_hrle->start_indices.size() );
                        }
                        else if (RT_DEFINED(erodedHRLE->run_types.back())) {
                            if ( erodedHRLE->run_breaks.size() > 0 ) erodedHRLE->run_breaks.pop_back();
                        }

                        // add the new eroded subsegment to the eroded HRLE block
                        if ( erodedHRLE->lower_hrle == NULL ) {
                            erodedHRLE->lower_hrle = new CHierarchicalRLE(erodedSubSegment->dimension);
                            erodedHRLE->lower_hrle->addToHRLE( erodedSubSegment );
                        }
                        else {
                            erodedHRLE->lower_hrle->addToHRLE( erodedSubSegment );
                        }
                      }
                      // else this coordinate is no longer defined
                      else {
                          if ( RT_DEFINED(run) ) {
                              if ( erodedHRLE->run_types.size() == 0 ) {
                                  erodedHRLE->run_types.push_back( RT_POSITIVE );
                              }
                              else if ( !RT_DEFINED( erodedHRLE->run_types.back() ) ) {
                                  if ( erodedHRLE->run_breaks.size() > 0 ) {
                                      try {
                                          erodedHRLE->run_breaks.pop_back();
                                      }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
                                          cerr << "Out of range exception " << e.what() << endl;
                                          cerr << "Erosion: removing break in dimension" << this->dimension << endl;
                                          exit(1);
                                      }
                                  }
                              }
                              else {
                                  erodedHRLE->run_types.push_back( RT_NEGATIVE );
                              }
                          }
                          else {

                              if ( erodedHRLE->run_types.size() == 0 ) {
                                  erodedHRLE->run_types.push_back( run );
                              }

                              else if (!(this->sameRunType(erodedHRLE->run_types.back(), run)) ) {

                                  if ( (erodedHRLE->run_types.back() == RT_NEGATIVE) &&
                                       (run == RT_POSITIVE) ) {
                                      try {
                                          erodedHRLE->run_types.pop_back();
                                          if ( erodedHRLE->run_breaks.size() > 0 ) erodedHRLE->run_breaks.pop_back();
                                      }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
                                          cerr << "Out of range exception " << e.what() << endl;
                                          cerr << "Erosion: positive after negative run" << endl;
                                          exit(1);
                                      }
                                  }
                                  else if ( (erodedHRLE->run_types.back() == RT_POSITIVE) && (run == RT_NEGATIVE) ) {
                                      erodedHRLE->run_types.pop_back();
                                  }
                                  erodedHRLE->run_types.push_back( run );
                              }

                              else if ( erodedHRLE->run_breaks.size() > 0 ) {
                                  try {
                                      erodedHRLE->run_breaks.pop_back();
                                  }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
                                      cerr << "Out of range exception " << e.what() << endl;
                                      cerr << "Erosion: removing break (undefined run)" << endl;
                                      exit(1);
                                  }
                              }

                          }
                      }

                      // add the break
                      if ( (ym+1) < erodedHRLE->max ) erodedHRLE->run_breaks.push_back( ym+1 );

                  }

              }

              // add the exisiting non defined run
              else {

                  if (erodedHRLE->run_types.size() == 0)
                       erodedHRLE->run_types.push_back( run );

                  else if (!(this->sameRunType(erodedHRLE->run_types.back(), run)) ) {
                      if ( (erodedHRLE->run_types.back() == RT_NEGATIVE) && (run == RT_POSITIVE) ) {
                          try {
                              erodedHRLE->run_types.pop_back();
                              if ( erodedHRLE->run_breaks.size() > 0 ) erodedHRLE->run_breaks.pop_back();
                          }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
                              cerr << "Out of range exception " << e.what() << endl;
                              cerr << "Erosion: positive after negative run" << endl;
                              exit(1);
                          }
                      }
                      else if ( (erodedHRLE->run_types.back() == RT_POSITIVE) && (run == RT_NEGATIVE) ) {
                          erodedHRLE->run_types.pop_back();
                      }
                      erodedHRLE->run_types.push_back( run );
                  }

                  else if ( erodedHRLE->run_breaks.size() > 0 ) {
                      try {
                          erodedHRLE->run_breaks.pop_back();
                      }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
                          cerr << "Out of range exception " << e.what() << endl;
                          cerr << "Erosion: removing break (undefined run) in dimension " << this->dimension << endl;
                          exit(1);
                      }
                  }

                  // add the break for this run
                  if ( endCoord < erodedHRLE->max ) erodedHRLE->run_breaks.push_back( endCoord );

              }

              startCoord = endCoord;
              if ( (iRun+2) >= segmentEnd ) endCoord = this->max;
              else {
                  try{
                      endCoord = this->run_breaks.at( firstBreak++ );
                  } catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
                      cerr << "Out of range exception " << e.what() << endl;
                      cerr << "Erosion: updating end coordinate of run in dimension " << this->dimension << endl;
                      exit(1);
                  }
              }
        }

        if ( (erodedHRLE->run_types.size() > 0) && (erodedHRLE->run_types.back() == RT_NEGATIVE) ) {
            erodedHRLE->run_types.pop_back();
            erodedHRLE->run_types.push_back( RT_POSITIVE );
        }

        if (erodedHRLE->lower_hrle == NULL ) {
            if ( erodedHRLE != NULL ) delete erodedHRLE;
            erodedHRLE = NULL;
        }

    }

    return erodedHRLE;
}

/*!
    \fn CHierarchicalRLE::dilateSegment( int si, bool left, bool middle, bool right )
 */
// this is the 1D dilation algorithm (old version)
// only run types und run breaks are used! subsegments ans values are not considered
CHierarchicalRLE* CHierarchicalRLE::dilateSegment( int si, bool left, bool middle, bool right )
{
    CHierarchicalRLE*               dilatedHRLE = new CHierarchicalRLE();       // the dilated hrle level set
    vector<int>                     dilatedCoordinates;                         // run break array after the dilationCArray3D
    int                             run, tmpRun;
    int                             segmentStart;                               // starting index of the segment
    int                             segmentEnd;                                 // starting index of the next segment
    int                             segmentLength;
    short int                       startCoordinate;
    short int                       endCoordinate;
    int                             firstBreak;
    int                             j;

    // get start and end indices of this segment
    segmentStart = this->start_indices.at(si);
    if (segmentStart == this->start_indices.at(this->start_indices.size()-1))
        segmentEnd = this->run_types.size();
    else
        segmentEnd = this->start_indices.at(si+1);

    segmentLength = segmentEnd - segmentStart;

    firstBreak = this->start_indices.at(si) - si;

    // start and end coordinate of the first run (end coordinate don't belong to the run!)
    startCoordinate = this->min;
    if (segmentLength < 2)  endCoordinate = this->max;
    else endCoordinate = this->run_breaks.at( firstBreak );                             // set endCoordinate for the first run to the first break

    // initialize min and max
    dilatedHRLE->min = this->min;
    dilatedHRLE->max = this->max;

    // dilate the defined runs of this segment and create a new RLE segment
    for (j = segmentStart; j < segmentEnd; j++) {
        run = this->run_types.at(j);                                                    // get run

        if ( RT_DEFINED(run) ) {
            // dilate the defined run
            if (middle) {
              if (left) {
                dilatedCoordinates.push_back( startCoordinate - 1 );
              }
              else {
                dilatedCoordinates.push_back( startCoordinate );
              }
              if (right) {
                dilatedCoordinates.push_back( endCoordinate + 1 );
              }
              else {
                dilatedCoordinates.push_back( endCoordinate );
              }
            }
            else {
              if (left && !right) {
                dilatedCoordinates.push_back( startCoordinate - 1 );
                dilatedCoordinates.push_back( endCoordinate - 1);
              }
              if (right && !left) {
                dilatedCoordinates.push_back( startCoordinate + 1);
                dilatedCoordinates.push_back( endCoordinate + 1 );
              }
              if (right && left) {
                dilatedCoordinates.push_back( startCoordinate  - 1);
                dilatedCoordinates.push_back( endCoordinate + 1 );
              }
            }
        }

        startCoordinate = endCoordinate;                                                // start coordinate of the next run
        if ((j+2) >= segmentEnd) endCoordinate = this->max;                             // end coordinate of the next run
        else endCoordinate = this->run_breaks.at( ++firstBreak );
    }

    // no more define
    if ( dilatedCoordinates.size() == 0 ) {
        return NULL;
    }


    // union of the dilated runs and the original segment
    for (j = 0; j < dilatedCoordinates.size(); j += 2) {

        // start and end coordinate of this defined run
        int definedStartCoordinate = dilatedCoordinates.at(j);
        int definedEndCoordinate = dilatedCoordinates.at(j+1);

        // if the defined run starts before the minimal coordinate
        if ( j == 0 ) {
            if ( definedStartCoordinate <= this->min ) {
                tmpRun = this->getRunByCoordinate( this->min, si );
                if ( !RT_DEFINED(tmpRun) || !middle ) {
                    dilatedHRLE->min = definedStartCoordinate - 1;
                    dilatedHRLE->run_types.push_back( RT_POSITIVE );
                }
                else dilatedHRLE->min = definedStartCoordinate;
            }
            else dilatedHRLE->run_types.push_back( this->getRunByCoordinate( this->min, si ) );
        }

        // if the run is between min and max
        if ( dilatedHRLE->run_breaks.size() > 0 ) {
            // union of two defined runs
            if (definedStartCoordinate <= dilatedHRLE->run_breaks.back()) {
                dilatedHRLE->run_breaks.pop_back();
            }
            // if a non-defined run lies between the current and the previous defined run
            else {
                int runTypePos = dilatedHRLE->run_breaks.back();
                dilatedHRLE->run_types.push_back( this->getRunByCoordinate( runTypePos, si ) );

                if ( definedStartCoordinate != dilatedHRLE->min ) dilatedHRLE->run_breaks.push_back( definedStartCoordinate );
                dilatedHRLE->run_types.push_back( 1 );
            }
            dilatedHRLE->run_breaks.push_back( definedEndCoordinate );
        }
        // add defined run if it is the first defined run in this segment / block
        else {
            if ( definedStartCoordinate != dilatedHRLE->min ) dilatedHRLE->run_breaks.push_back( definedStartCoordinate );
            dilatedHRLE->run_breaks.push_back( definedEndCoordinate );
            dilatedHRLE->run_types.push_back( 1 );
        }

        // if the defined run ends behind the maximal coordinate
        if ( (j+2) >= dilatedCoordinates.size() ) {
            if ( definedEndCoordinate >= this->max ) {
                tmpRun = this->getRunByCoordinate( (this->max -1), si );
                if ( !RT_DEFINED(tmpRun) || !middle ) {
                    dilatedHRLE->max = definedEndCoordinate + 1;
                    dilatedHRLE->run_types.push_back( RT_POSITIVE );
                }
                else dilatedHRLE->max = definedEndCoordinate;
            }
            else dilatedHRLE->run_types.push_back( this->getRunByCoordinate( (this->max -1), si ) );
            if ( (dilatedHRLE->run_breaks.size() > 0) && (dilatedHRLE->run_breaks.back() == dilatedHRLE->max) ) dilatedHRLE->run_breaks.pop_back();
        }

    }

    // start index of this segment is 0 because this dilatedHRLE is an 1D RLE block
    dilatedHRLE->start_indices.push_back(0);

    return dilatedHRLE;
}

/*!
    \fn CHierarchicalRLE::getRunByCoordinate( int qi, int segmentIndex )
 */
// returns the run code of a run in this RLE block by the given coordinate and segment index
int CHierarchicalRLE::getRunByCoordinate( int qi, int segmentIndex, short int* startCoord ) {

    short int           tmp_min = this->min;
    short int           tmp_max = this->max;
    int                 segmentLength;
    unsigned int        firstRun;
    unsigned int        firstBreak;
    int                 run;

    // get segment length - number of runs in this segment
    if (this->start_indices.at(segmentIndex) == this->start_indices.at(this->start_indices.size()-1))
        segmentLength = this->run_types.size() - this->start_indices.at(segmentIndex);
    else
        segmentLength = this->start_indices.at(segmentIndex+1) - this->start_indices.at(segmentIndex);

    // get the index of the first run in this segment
    firstRun = this->start_indices.at(segmentIndex);
    // get the index of the first run break of this segment
    firstBreak = this->start_indices.at(segmentIndex) - segmentIndex;

    if ( (qi < this->min) || (qi >= this->max) ) {
        run = RT_NEGATIVE;
    }
    else {
        int indexRun = this->accessBinSearch( qi, firstRun, firstBreak, segmentLength, &tmp_min, &tmp_max);
        run = this->run_types.at( indexRun );
    }

    if (startCoord != NULL) *startCoord = tmp_min;

    return run;

}

/*!
    \fn CHierarchicalRLE::addToHRLE( CHierarchicalRLE hrleObject )
 */
// add a hrle structure to the current hrle structure
// hrleObject must be from the same dimension!!
bool CHierarchicalRLE::addToHRLE( CHierarchicalRLE *hrleObject ) {
    int i;
    int numRuns = this->run_types.size();
    int run;

    if ( hrleObject == NULL ) {
        cerr << "Error! Can't add NULL to HRLE object!" << endl;
        exit(1);
    }

    // add the values of the given hrle level set to the current level set - higher dimensions
    if ( (this->lower_hrle != NULL) && (hrleObject->lower_hrle != NULL) ) {
        // add run types and update the start indices
        for ( i = 0; i < hrleObject->run_types.size(); i++ ) {
            run = hrleObject->run_types.at(i);
            if ( RT_DEFINED(run) ) {
                run += this->lower_hrle->start_indices.size();
            }
            this->run_types.push_back( run );
        }
        // recursive call for the lower hrle block
        if ( this->lower_hrle->addToHRLE( hrleObject->lower_hrle ) == false ) {
            cerr << "Error while adding lower dimension of a HRLE object to another" << endl;
            return false;
        }
    }
    // add the values of the given hrle level set to the current level set - lowest dimension
    else if ( (this->lower_hrle == NULL) && (hrleObject->lower_hrle == NULL) ) {
        // add run types and update the start indices
        for ( i = 0; i < hrleObject->run_types.size(); i++ ) {
            run = hrleObject->run_types.at(i);
            this->run_types.push_back( run );
        }
    }
    else {
        cerr << "Error while adding a hrle level set to another! - Different dimensions!!" << endl;
        return false;
    }

    // add start indices
    for ( i = 0; i < hrleObject->start_indices.size(); i++ )
        this->start_indices.push_back( numRuns + hrleObject->start_indices.at( i ) );
    // add run breaks
    for ( i = 0; i < hrleObject->run_breaks.size(); i++ )
        this->run_breaks.push_back( hrleObject->run_breaks.at(i) );
    // update min & max
    if ( hrleObject->min < this->min ) this->min = hrleObject->min;
    if ( hrleObject->max > this->max ) this->max = hrleObject->max;

    delete hrleObject;
    return true;
}

// intersection of two N-D HRLE blocks to one 1D HRLE block
CHierarchicalRLE* CHierarchicalRLE::hrleIntersection( CHierarchicalRLE* first, CHierarchicalRLE* second ) {

    vector<CHierarchicalRLE*> hrleVector;
    hrleVector.push_back( first );
    hrleVector.push_back( second );

    return this->hrleIntersection( hrleVector );

}

// intersection of some N-D HRLE blocks to one 1D HRLE block
CHierarchicalRLE* CHierarchicalRLE::hrleIntersection( vector<CHierarchicalRLE*> hrleVector ) {

    CHierarchicalRLE              *tempValue, *currentHRLE, *returnValue, *subSegment;
    int                           pos1, pos2;
    short int                     startCoord, endCoord, coord;
    short int                     startCoord1, endCoord1, startCoord2, endCoord2;
    int                           defIndex, run;
    vector<CHierarchicalRLE*>     tmpSubsegments;

    // if there is just one HRLE block -> nothing to do, return given HRLE block
    if ( hrleVector.size() < 1 ) {
        return NULL;
    }
    else if ( hrleVector.size() == 1 ) {
        return hrleVector.at(0);
    }
    else {
        for ( int i = 0; i < hrleVector.size(); i++ ) {
            if ( hrleVector.at(i) == NULL ) {
                return NULL;
            }
        }
    }

    try {
      tempValue = hrleVector.at(0);
    }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
        cerr << "Out of range exception " << e.what() << endl;
        cerr << "Intersection: first HRLE segment does not exist!" << endl;
        exit(1);
    }

    // now we need to compare the different HRLE blocks
    for ( int i = 1; i < hrleVector.size(); i++ ) {
        try {
            currentHRLE = hrleVector.at(i);                     // the next HRLE block
        }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
            cerr << "Out of range exception " << e.what() << endl;
            cerr << "Intersection: no further HRLE segment for intersection." << endl;
            exit(1);
        }
        pos1 = 0;                                           // index into the run array of the tempValue
        pos2 = 0;                                           // index into the run array of the currentHRLE
        returnValue = new CHierarchicalRLE();

        // initialize min and max of the return value
        if ( tempValue->min > currentHRLE->min ) returnValue->min = tempValue->min;
        else returnValue->min = currentHRLE->min;
        if ( tempValue->max > currentHRLE->max ) returnValue->max = currentHRLE->max;
        else returnValue->max = tempValue->max;

        // initialize start index
        returnValue->start_indices.push_back( 0 );
        returnValue->dimension = tempValue->dimension;

        // initialize start and end coordinates (first run)
        startCoord1 = tempValue->min;
        startCoord2 = currentHRLE->min;
        if ( tempValue->run_types.size() < 2 ) endCoord1 = tempValue->max;
        else {
            try {
                endCoord1 = tempValue->run_breaks.at( 0 );
            }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
                cerr << "Out of range exception " << e.what() << endl;
                cerr << "Intersection: can't setup break for the first HRLE segment" << endl;
                exit(1);
            }
        }
        if ( currentHRLE->run_types.size() < 2 ) endCoord2 = currentHRLE->max;
        else {
            try {
                endCoord2 = currentHRLE->run_breaks.at( 0 );
            }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
                cerr << "Out of range exception " << e.what() << endl;
                cerr << "Intersection: can't setup break for the second HRLE segment" << endl;
                exit(1);
            }
        }

        // only coordinates in both runs are from interest
        while ( (endCoord1 < returnValue->min) && (pos1 < tempValue->run_types.size()) ) {
            pos1++;
            startCoord1 = endCoord1;
            if ( pos1 < tempValue->run_breaks.size() ) {
                try {
                    endCoord1 = tempValue->run_breaks.at(pos1);
                }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
                    cerr << "Out of range exception " << e.what() << endl;
                    cerr << "Intersection: updating break for pos1" << endl;
                    exit(1);
                }
            }
            else endCoord1 = tempValue->max;
        }
        while ( (endCoord2 < returnValue->min) && (pos2 < currentHRLE->run_types.size()) ) {
            pos2++;
            startCoord2 = endCoord2;
            if ( pos2 < currentHRLE->run_breaks.size() ) {
                try {
                    endCoord2 = currentHRLE->run_breaks.at(pos2);
                }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
                    cerr << "Out of range exception " << e.what() << endl;
                    cerr << "Intersection: updating break for pos2" << endl;
                    exit(1);
                }
            }
            else endCoord2 = currentHRLE->max;
        }

        // compare the two segments
        while ( (startCoord1 < returnValue->max) && (startCoord2 < returnValue->max) &&
                (pos1 < tempValue->run_types.size()) && (pos2 < currentHRLE->run_types.size()) ) {

                // if both defined, add a defined run
                if ( RT_DEFINED(tempValue->run_types.at(pos1)) && RT_DEFINED(currentHRLE->run_types.at(pos2)) ) {
                    if ( tempValue->lower_hrle == NULL ) {
                        if ( (returnValue->run_types.size() == 0) ||
                             !RT_DEFINED(returnValue->run_types.back()) ) {
                            returnValue->run_types.push_back(1);
                        }
                        else {
                            if ( returnValue->run_breaks.size() > 0 ) returnValue->run_breaks.pop_back();
                        }
                        if ( endCoord1 < endCoord2 ) endCoord = endCoord1;
                        else endCoord = endCoord2;
                        if ( endCoord > returnValue->max ) endCoord = returnValue->max;
                    }
                    // intersection of the sub segments
                    else {

                        if ( returnValue->lower_hrle == NULL ) {
                            // initialize lower RLE block of the return value
                            returnValue->lower_hrle = new CHierarchicalRLE( tempValue->lower_hrle->dimension );
                        }
                        // start and end coordinates of the defined intersection
                        if ( returnValue->run_breaks.size() == 0 ) startCoord = returnValue->min;
                        else {
                            try {
                                startCoord = returnValue->run_breaks.back();
                            }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
                                cerr << "Out of range exception " << e.what() << endl;
                                cerr << "Intersection: error setting up startCoord for intersecting defined runs" << endl;
                                exit(1);
                            }
                        }

                        if ( endCoord1 < endCoord2 ) endCoord = endCoord1;
                        else endCoord = endCoord2;
                        if ( endCoord > returnValue->max ) endCoord = returnValue->max;

                        for ( coord = startCoord; coord < endCoord; coord++ ) {
                            tmpSubsegments.clear();
                            subSegment = NULL;

                            // get the sub segment from the first HRLE structure
                            try {
                                run = tempValue->run_types.at(pos1);
                            }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
                                cerr << "Out of range exception " << e.what() << endl;
                                cerr << "Intersection: run with pos1=" << pos1 << " does not exist!" << endl;
                                exit(1);
                            }
                            defIndex = run + coord - startCoord1;
                            tmpSubsegments.push_back( tempValue->lower_hrle->getSegmentAt( defIndex ) );

                            // get the sub segment from the second HRLE structure
                            try {
                                run = currentHRLE->run_types.at(pos2);
                            }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
                                cerr << "Out of range exception " << e.what() << endl;
                                cerr << "Intersection: run with pos2=" << pos2 << " does not exist!" << endl;
                                exit(1);
                            }
                            defIndex = run + coord - startCoord2;
                            tmpSubsegments.push_back( currentHRLE->lower_hrle->getSegmentAt( defIndex ) );

                            // intersect the sub segments
                            subSegment = this->hrleIntersection( tmpSubsegments );

                            // there were no intersecting defined coordinates in both sub segments
                            if ( (subSegment->run_types.size() == 1) && (!RT_DEFINED(subSegment->run_types.back())) ) {
                                // the run is the first in the segment
                                if (returnValue->run_types.size() == 0) {
                                    returnValue->run_types.push_back( RT_POSITIVE );
                                }
                                // last run was not defined, so delete the last break because this coordinate does not belong to this run
                                else if ( !RT_DEFINED( returnValue->run_types.back() ) ) {
                                    if ( returnValue->run_breaks.size() > 0 ) returnValue->run_breaks.pop_back();
                                }
                                // last run was defined, break the last run and add an inner run
                                else {
                                    returnValue->run_breaks.push_back( coord );
                                    returnValue->run_types.push_back( RT_NEGATIVE );
                                }

                                // add the next coordinate as new break for the run
                                if (((coord+1) < returnValue->max) && ((coord+1) < endCoord))
                                    returnValue->run_breaks.push_back( coord+1 );
                            }
                            // add the intersection of the two sub segments to the lower RLE block of the return value
                            else {
                                if ( (returnValue->run_types.size() == 0) || !RT_DEFINED(returnValue->run_types.back()) ) {
                                    returnValue->run_types.push_back( returnValue->lower_hrle->start_indices.size() );
                                }
                                returnValue->lower_hrle->addToHRLE( subSegment );
                            }

                        }
                    }

                    // add the lowest break of both segments as new break in the return value
                    if (endCoord < returnValue->max) returnValue->run_breaks.push_back( endCoord );

                }
                else {
                    // if both positive, add a positive run
                    if ( (tempValue->run_types.at(pos1) == RT_POSITIVE) && (currentHRLE->run_types.at(pos2) == RT_POSITIVE) ) {
                        if ( returnValue->run_types.size() == 0 ) {
                            returnValue->run_types.push_back( RT_POSITIVE );
                        }
                        else {
                            if ( !RT_DEFINED(returnValue->run_types.back()) ) {
                                if ( returnValue->run_breaks.size() > 0 ) returnValue->run_breaks.pop_back();

                                if ( returnValue->run_types.back() == RT_NEGATIVE ) {
                                    returnValue->run_types.pop_back();
                                    returnValue->run_types.push_back( RT_POSITIVE );
                                }
                            }
                            else {
                                returnValue->run_types.push_back( RT_POSITIVE );
                            }
                        }
                    }
                    // else add a negative run, if it is not the first run
                    else {
                        if (returnValue->run_types.size() == 0) {
                            returnValue->run_types.push_back( RT_POSITIVE );
                        }
                        else if (!RT_DEFINED( returnValue->run_types.back() )) {
                            if ( returnValue->run_breaks.size() > 0 ) returnValue->run_breaks.pop_back();
                        }
                        else {
                            returnValue->run_types.push_back( RT_NEGATIVE );
                        }
                    }
                    // add the lowest break of both segments as new break to the return value
                    if ( (endCoord1 < endCoord2) && (endCoord1 < returnValue->max) ) returnValue->run_breaks.push_back( endCoord1 );
                    else if (endCoord2 < returnValue->max) returnValue->run_breaks.push_back( endCoord2 );
                }

            // update pos -> next run if the last break in the result is greater then the break of this run
            // if the break of the run in tempValue is greater as the last break in the result, go to the next run in the currentHRLE
            if ( (endCoord1 >= returnValue->max) && (endCoord2 >= returnValue->max) ) {
                startCoord1 = endCoord1;
                startCoord2 = endCoord2;
            }

            if ( returnValue->run_breaks.size() > 0 ) {

                while ( endCoord1 <= returnValue->run_breaks.back() ) {
                    pos1++;

                    // update start and end coordinate of the next run
                    startCoord1 = endCoord1;
                    if ( pos1 < tempValue->run_breaks.size()) endCoord1 = tempValue->run_breaks.at(pos1);
                    else endCoord1 = tempValue->max;

                    if (pos1 >= tempValue->run_types.size()) break;
                }

                while ( endCoord2 <= returnValue->run_breaks.back() ) {
                    pos2++;

                    // update start and end coordinate of the next run
                    startCoord2 = endCoord2;
                    if ( pos2 < currentHRLE->run_breaks.size()) endCoord2 = currentHRLE->run_breaks.at(pos2);
                    else endCoord2 = currentHRLE->max;

                    if (pos2 >= currentHRLE->run_types.size()) break;
                }

            }
            // max is reached so do a break
            else break;

        }

        delete currentHRLE;
        delete tempValue;

        // change last run to positive if it is negative
        if ( returnValue->run_types.back() == RT_NEGATIVE ) {
            returnValue->run_types.pop_back();
            returnValue->run_types.push_back( RT_POSITIVE );
        }

        // intersect the result with the next HRLE structure
        if ( (i+1) < hrleVector.size() ) {
            tempValue = returnValue;
            returnValue = NULL;
        }

    }

    return returnValue;


}

// union of two N-D HRLE blocks to one 1D HRLE block
CHierarchicalRLE* CHierarchicalRLE::hrleUnion( CHierarchicalRLE* first, CHierarchicalRLE* second ) {

    vector<CHierarchicalRLE*> hrleVector;
    hrleVector.push_back( first );
    hrleVector.push_back( second );

    return this->hrleUnion( hrleVector );

}

// union of some N-D HRLE blocks to one 1D HRLE block
CHierarchicalRLE* CHierarchicalRLE::hrleUnion( vector<CHierarchicalRLE*> hrleVector ) {
    CHierarchicalRLE*               tempValue;
    CHierarchicalRLE*               returnValue;                               // temporary for the result
    CHierarchicalRLE*               currentHRLE;
    int                             pos1, pos2;
    short int                       startCoord, endCoord;
    short int                       startCoord1, endCoord1, startCoord2, endCoord2;
    int                             runBreak1, runBreak2;
    int                             newRun, runFromOtherBlock;
    vector<CHierarchicalRLE*>       tmpSubsegments;
    int                             lastRun = RT_NEGATIVE;

    // if there is just one HRLE block -> nothing to do, return given HRLE block
    if ( hrleVector.size() < 1 ) {
        return NULL;
    }
    else if ( hrleVector.size() == 1 ) {
        return hrleVector.at(0);
    }

    tempValue = hrleVector.at(0);

    // now we need to compare the different HRLE blocks
    for ( int i = 1; i < hrleVector.size(); i++ ) {
        currentHRLE = hrleVector.at(i);                     // the next HRLE block
        pos1 = 0;                                           // index into the run array of the tempValue
        pos2 = 0;                                           // index into the run array of the currentHRLE
        returnValue = new CHierarchicalRLE();

        if ( (tempValue->lower_hrle != NULL) && (currentHRLE->lower_hrle != NULL) )
            returnValue->lower_hrle = new CHierarchicalRLE();

        // set dimension
        returnValue->dimension = tempValue->dimension;
        // initialize start indices array
        returnValue->start_indices.push_back( 0 );

        // update min
        if ( currentHRLE->min < tempValue->min ) returnValue->min = currentHRLE->min;
        else returnValue->min = tempValue->min;

        // update max
        if ( currentHRLE->max > tempValue->max ) returnValue->max = currentHRLE->max;
        else returnValue->max = tempValue->max;

        // initialize start and end coordinates (first run)
        startCoord1 = tempValue->min;
        startCoord2 = currentHRLE->min;
        if ( tempValue->run_types.size() < 2 ) endCoord1 = tempValue->max;
        else endCoord1 = tempValue->run_breaks.at( 0 );
        if ( currentHRLE->run_types.size() < 2 ) endCoord2 = currentHRLE->max;
        else endCoord2 = currentHRLE->run_breaks.at( 0 );

        // traverse through all runs of both H-RLE blocks
        while ( ( pos1 < tempValue->run_types.size()) || (pos2 < currentHRLE->run_types.size()) ) {

            // get the breaks of both runs
            if ( pos1 >= tempValue->run_breaks.size() ) runBreak1 = tempValue->max;
            else runBreak1 = tempValue->run_breaks.at(pos1);
            if ( pos2 >= currentHRLE->run_breaks.size() ) runBreak2 = currentHRLE->max;
            else runBreak2 = currentHRLE->run_breaks.at(pos2);

            // first segment starts before the second
            if ( ((pos2 == 0) && (startCoord1 < startCoord2)) ) {

                newRun = tempValue->run_types.at(pos1);

                if ( returnValue->run_breaks.size() == 0) startCoord = returnValue->min;
                else startCoord = returnValue->run_breaks.back();

                if ( startCoord2 < endCoord1 ) returnValue->run_breaks.push_back( startCoord2 );
                else returnValue->run_breaks.push_back( endCoord1 );

                if ( RT_DEFINED(newRun) &&
                     (tempValue->lower_hrle != NULL)) {
                    newRun = returnValue->lower_hrle->start_indices.size();
                    endCoord = returnValue->run_breaks.back();
                    for ( int qi = startCoord; qi < endCoord; qi++) {
                        returnValue->lower_hrle->addToHRLE( tempValue->lower_hrle->getSegmentAt(
                                                            tempValue->run_types.at(pos1) + qi - startCoord1 ));
                    }
                }

                returnValue->run_types.push_back( newRun );

                if (endCoord1 > returnValue->run_breaks.back()) startCoord1 = returnValue->run_breaks.back();
                else runBreak1 = returnValue->run_breaks.back();
            }

            // otherwise
            else if ( ((pos1 == 0) && (startCoord2 < startCoord1)) ) {

                newRun = currentHRLE->run_types.at(pos2);

                if ( returnValue->run_breaks.size() == 0) startCoord = returnValue->min;
                else startCoord = returnValue->run_breaks.back();

                if ( startCoord1 < endCoord2 ) returnValue->run_breaks.push_back( startCoord1 );
                else returnValue->run_breaks.push_back( endCoord2 );

                if ( RT_DEFINED(newRun) &&
                     (currentHRLE->lower_hrle != NULL)) {
                    newRun = returnValue->lower_hrle->start_indices.size();
                    endCoord = returnValue->run_breaks.back();
                    for ( int qi = startCoord; qi < endCoord; qi++) {
                        returnValue->lower_hrle->addToHRLE( currentHRLE->lower_hrle->getSegmentAt(
                                                            currentHRLE->run_types.at(pos2) + qi - startCoord2 ));
                    }
                }

                returnValue->run_types.push_back( newRun );

                if (endCoord2 > returnValue->run_breaks.back()) startCoord2 = returnValue->run_breaks.back();
                else runBreak2 = returnValue->run_breaks.back();
            }

            // add the remaining runs and breaks of currentHRLE
            else if ( (pos1 >= tempValue->run_types.size()) &&
                 (pos2 < currentHRLE->run_types.size()) ) {

                if (runBreak2 > returnValue->run_breaks.back()) {
                    for ( int i = pos2; i < currentHRLE->run_types.size(); i++ ) {
                        newRun = currentHRLE->run_types.at(i);

                        if ( returnValue->run_breaks.size() > 0 ) startCoord = returnValue->run_breaks.back();
                        else startCoord = returnValue->min;

                        if ( (returnValue->run_types.size() > 0) &&
                            ( this->sameRunType(lastRun, newRun) ||
                              ( RT_DEFINED(lastRun)  && RT_DEFINED(newRun) )) ) {
                                returnValue->run_breaks.pop_back();
                        }

                        // if the run is defined, add the subsegments to the result
                        if ( RT_DEFINED(newRun) ) {
                            if ( currentHRLE->lower_hrle != NULL ) newRun = returnValue->lower_hrle->start_indices.size();

                            if ( i < currentHRLE->run_breaks.size() ) endCoord = currentHRLE->run_breaks.at(i);
                            else endCoord = currentHRLE->max;

                            if ( currentHRLE->lower_hrle != NULL ) {
                                for ( int qi = startCoord; qi < endCoord; qi++) {
                                    returnValue->lower_hrle->addToHRLE(
                                        currentHRLE->lower_hrle->getSegmentAt(
                                            currentHRLE->run_types.at(i) + qi - startCoord2 )
                                    );
                                }
                            }
                        }

                        if ( (i < currentHRLE->run_breaks.size()) &&
                            (currentHRLE->run_breaks.at(i) != returnValue->max) &&
                            ( (returnValue->run_breaks.size() == 0) ||
                              (currentHRLE->run_breaks.at(i) > returnValue->run_breaks.back()) ) /*&&
                            (!(this->sameRunType(lastRun, newRun)) && !( RT_DEFINED(lastRun) && RT_DEFINED(newRun)))*/ )
                            returnValue->run_breaks.push_back( currentHRLE->run_breaks.at(i) );

                        if ( (returnValue->run_types.size() == 0) ||
                            (!(this->sameRunType(lastRun, newRun)) && !( RT_DEFINED(lastRun) && RT_DEFINED(newRun))) )
                            returnValue->run_types.push_back( newRun );

                        lastRun = newRun;
                    }
                    pos2 = currentHRLE->run_types.size();
                }
            }

            // add the remaining runs and breaks of tempValue
            else if ( (pos2 >= currentHRLE->run_types.size())&&
                      (pos1 < tempValue->run_types.size()) ) {

                if (runBreak1 > returnValue->run_breaks.back()) {
                    for ( int i = pos1; i < tempValue->run_types.size(); i++ ) {
                        newRun = tempValue->run_types[i];

                        if ( returnValue->run_breaks.size() > 0 ) startCoord = returnValue->run_breaks.back();
                        else startCoord = returnValue->min;

                        if ( (returnValue->run_types.size() > 0) &&
                            (this->sameRunType(lastRun, newRun) || ( RT_DEFINED(lastRun) && RT_DEFINED(newRun))) ) {
                                returnValue->run_breaks.pop_back();
                        }

                        // if the run is defined, add the subsegments to the result
                        if ( RT_DEFINED(newRun) ) {
                            if ( tempValue->lower_hrle != NULL ) newRun = returnValue->lower_hrle->start_indices.size();

                            if ( i < tempValue->run_breaks.size() ) endCoord = tempValue->run_breaks.at(i);
                            else endCoord = currentHRLE->max;

                            if ( tempValue->lower_hrle != NULL ) {
                                for ( int qi = startCoord; qi < endCoord; qi++) {
                                    returnValue->lower_hrle->addToHRLE( tempValue->lower_hrle->getSegmentAt( tempValue->run_types.at(i) + qi - startCoord1 ) );
                                }

                            }                    }

                        if ( (i < tempValue->run_breaks.size()) &&
                            (tempValue->run_breaks.at(i) != returnValue->max) &&
                            ( (returnValue->run_breaks.size() == 0) ||
                              (tempValue->run_breaks.at(i) > returnValue->run_breaks.back()) ) /*&&
                            (!(this->sameRunType(lastRun, newRun)) && !( RT_DEFINED(lastRun) && RT_DEFINED(newRun)))*/ )
                            returnValue->run_breaks.push_back( tempValue->run_breaks.at(i) );

                        if ( (returnValue->run_types.size() == 0) ||
                            (!(this->sameRunType(lastRun, newRun)) && !( RT_DEFINED(lastRun) && RT_DEFINED(newRun))) )
                            returnValue->run_types.push_back( newRun );

                        lastRun = newRun;
                    }
                    pos1 = tempValue->run_types.size();
                }

            }

            // compare the runs of both HRLE blocks
            else if ( ( pos1 < tempValue->run_types.size()) && (pos2 < currentHRLE->run_types.size())) {

                if ( (returnValue->run_breaks.size() == 0) ||
                     ((runBreak1 > returnValue->run_breaks.back()) &&
                      (runBreak2 > returnValue->run_breaks.back())) ) {


                    // if both runs have the same value
                  if ( this->sameRunType( tempValue->run_types.at(pos1), currentHRLE->run_types.at(pos2) ) ) {

                        newRun = tempValue->run_types.at(pos1);

                        if ( returnValue->run_breaks.size() > 0 ) startCoord = returnValue->run_breaks.back();
                        else startCoord = returnValue->min;

                        // if the last run has the same type, don't add a new run
                        if ( (returnValue->run_types.size() > 0) && this->sameRunType(lastRun, newRun) ) {
                            returnValue->run_breaks.pop_back();
                        }

                        // if defined, use the higher break and get the subsegments
                        if ( RT_DEFINED(newRun) ) {

                            if ( runBreak1 > runBreak2 ) returnValue->run_breaks.push_back( runBreak1 );
                            else returnValue->run_breaks.push_back( runBreak2 );

                            endCoord = returnValue->run_breaks.back();

                            if ( tempValue->lower_hrle != NULL ) {

                                newRun = returnValue->lower_hrle->start_indices.size();

                                for ( int qi = startCoord; qi < endCoord; qi++) {

                                    tmpSubsegments.clear();

                                    // get subsegments of the run from the first H-RLE block
                                    if ( qi < endCoord1 )
                                        try {
                                            // subsegment of the first hrle
                                            tmpSubsegments.push_back( tempValue->lower_hrle->getSegmentAt(
                                                                      tempValue->run_types.at(pos1) + qi - startCoord1 ));
                                        }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
                                            cerr << "Out of range exception " << e.what() << endl;
                                            cerr << "HRLE union - getSegment with index " << (tempValue->run_types[pos1] + qi - startCoord1)
                                                 << " in dimension " << tempValue->dimension << " of tempValue failed!" << endl;
                                            cerr << "EndCoord1 is: " << endCoord1 << endl;
                                            cerr << "qi is: " << qi << endl;
                                            cerr << "pos1 is: " << pos1 << endl;
                                            cerr << "startCoord1 is: " << startCoord1 << endl;
                                            exit(1);
                                        }
                                    else if ( qi < tempValue->max ) {
                                        short int tmpStartOtherRun = tempValue->min;
                                        runFromOtherBlock = tempValue->getRunByCoordinate( qi, 0, &tmpStartOtherRun );
                                        if ( RT_DEFINED(runFromOtherBlock) ) {
                                            tmpSubsegments.push_back(
                                                tempValue->lower_hrle->getSegmentAt(
                                                    runFromOtherBlock + qi - tmpStartOtherRun )
                                            );
                                        }
                                    }

                                    // get subsements of the run from the second H-RLE block
                                    if ( qi < endCoord2 )
                                        try {
                                            // subsegment of the second hrle
                                            tmpSubsegments.push_back( currentHRLE->lower_hrle->getSegmentAt(
                                                                      currentHRLE->run_types[pos2] + qi - startCoord2 ) );
                                        }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
                                            cerr << "Out of range exception " << e.what() << endl;
                                            cerr << "HRLE union - getSegment with index " << (currentHRLE->run_types[pos2] + qi - startCoord2) << " in dimension " << tempValue->dimension << " of currentHRLE failed!" << endl;
                                            exit(1);
                                        }
                                    else  if ( qi < currentHRLE->max ) {
                                        short int tmpStartOtherRun = currentHRLE->min;
                                        runFromOtherBlock = currentHRLE->getRunByCoordinate( qi, 0, &tmpStartOtherRun );
                                        if ( RT_DEFINED(runFromOtherBlock) ) {
                                            tmpSubsegments.push_back( currentHRLE->lower_hrle->getSegmentAt( runFromOtherBlock + qi - tmpStartOtherRun ) );
                                        }
                                    }

                                    if ( tmpSubsegments.size() >= 1 )
                                        returnValue->lower_hrle->addToHRLE( returnValue->hrleUnion( tmpSubsegments ) );     // add the union of these two segments to the resulting hrle
                                    else if ( tmpSubsegments.size() < 1 )
                                        cout << "Error in function union -> at both defined" << endl;
                                }
                            }
                        }
                        // if undefined, use the lower break
                        else {
                            if ( runBreak1 < runBreak2 ) returnValue->run_breaks.push_back( runBreak1 );
                            else returnValue->run_breaks.push_back( runBreak2 );
                        }

                        if ( (returnValue->run_types.size() == 0) ||
                             (!(this->sameRunType(lastRun, newRun)) && !( RT_DEFINED(lastRun) && RT_DEFINED(newRun))) ) {
                            // add a new run
                            returnValue->run_types.push_back( newRun );
                        }

                    }
                    // else choose at first defined run, than inner run and at last outer run
                    else {

                        if ( returnValue->run_breaks.size() > 0 ) startCoord = returnValue->run_breaks.back();
                        else startCoord = returnValue->min;

                        if ( RT_DEFINED(tempValue->run_types.at(pos1)) ) {

                            newRun = 1;

                            // same run so we don't need the last break
                            if ( (returnValue->run_types.size() > 0) &&
                                 this->sameRunType(lastRun, newRun) ) {
                                returnValue->run_breaks.pop_back();
                            }
                            returnValue->run_breaks.push_back( runBreak1 );
                            endCoord = returnValue->run_breaks.back();

                            if ( tempValue->lower_hrle != NULL ) {

                                newRun = returnValue->lower_hrle->start_indices.size();

                                for ( int qi = startCoord; qi < endCoord; qi++) {

                                    tmpSubsegments.clear();

                                    short int tmpStartOtherRun = currentHRLE->min;

                                    tmpSubsegments.push_back( tempValue->lower_hrle->getSegmentAt( tempValue->run_types[pos1] + qi - startCoord1 ) );

                                    if ( (qi >= endCoord2) && (qi < currentHRLE->max) ) {
                                        runFromOtherBlock = currentHRLE->getRunByCoordinate( qi, 0, &tmpStartOtherRun );
                                        if ( RT_DEFINED(runFromOtherBlock) ) {
                                            tmpSubsegments.push_back(
                                                currentHRLE->lower_hrle->getSegmentAt(
                                                    runFromOtherBlock + qi - tmpStartOtherRun )
                                            );
                                        }
                                    }

                                    if ( tmpSubsegments.size() >= 1 )
                                        returnValue->lower_hrle->addToHRLE( returnValue->hrleUnion( tmpSubsegments ) );     // add the union of these two segments to the resulting hrle
                                    else if ( tmpSubsegments.size() < 1 )
                                        cout << "Error in function union -> at different types; first defined" << endl;

                                }
                            }
                        }
                        else if ( RT_DEFINED(currentHRLE->run_types.at(pos2)) ) {

                            newRun = 1;

                            // same run so we don't need the last break
                            if ( (returnValue->run_types.size() > 0) &&
                                 this->sameRunType(lastRun, newRun) ) {
                                returnValue->run_breaks.pop_back();
                            }

                            returnValue->run_breaks.push_back( runBreak2 );
                            endCoord = returnValue->run_breaks.back();

                            if ( currentHRLE->lower_hrle != NULL ) {

                                newRun = returnValue->lower_hrle->start_indices.size();

                                for ( int qi = startCoord; qi < endCoord; qi++) {

                                    tmpSubsegments.clear();

                                    short int tmpStartOtherRun = tempValue->min;

                                    tmpSubsegments.push_back( currentHRLE->lower_hrle->getSegmentAt( currentHRLE->run_types.at(pos2) + qi - startCoord2 ) );

                                    if ( (qi >= endCoord1) && (qi < tempValue->max) ) {
                                        runFromOtherBlock = tempValue->getRunByCoordinate( qi, 0, &tmpStartOtherRun );
                                        if ( RT_DEFINED(runFromOtherBlock) ) {
                                            tmpSubsegments.push_back(
                                                tempValue->lower_hrle->getSegmentAt(
                                                    runFromOtherBlock + qi - tmpStartOtherRun )
                                            );
                                        }
                                    }

                                    if ( tmpSubsegments.size() >= 1 )
                                        returnValue->lower_hrle->addToHRLE( returnValue->hrleUnion( tmpSubsegments ) );     // add the union of these two segments to the resulting hrle
                                    else if ( tmpSubsegments.size() < 1 )
                                        cout << "Error in function union -> at different types; first defined" << endl;
                                }
                            }
                        }
                        // first run is an inner run, second an outer
                        else if ( tempValue->run_types[pos1] == RT_NEGATIVE ) {
                            newRun = RT_NEGATIVE;
                            if ( (returnValue->run_types.size() > 0) &&
                                 (this->sameRunType(lastRun, newRun)) ) {
                                returnValue->run_breaks.pop_back();
                            }
                            if ( runBreak1 < runBreak2 ) returnValue->run_breaks.push_back( runBreak1 );
                            else returnValue->run_breaks.push_back( runBreak2 );
                        }
                        // second run is an inner run, first an outer
                        else if ( currentHRLE->run_types.at(pos2) == RT_NEGATIVE ) {
                            newRun = RT_NEGATIVE;
                            if ( (returnValue->run_types.size() > 0) &&
                                 (this->sameRunType(lastRun, newRun)) ) {
                                returnValue->run_breaks.pop_back();
                            }
                            if ( runBreak1 < runBreak2 ) returnValue->run_breaks.push_back( runBreak1 );
                            else returnValue->run_breaks.push_back( runBreak2 );
                        }
                        // well ... that shouldn't happen :)
                        else {
                            cout << "Error during union!" << endl;
                            cout << "tempValue Type: " << tempValue->run_types[pos1] << endl;
                            cout << "currentHRLE Type: " << currentHRLE->run_types[pos2] << endl;
                        }

                        if ( (returnValue->run_types.size() == 0) || 
                             !(this->sameRunType(lastRun, newRun)) ) {
                            // add a new run
                            returnValue->run_types.push_back( newRun );
                        }
                    }
                }
            }
            else {
                pos1 = tempValue->run_types.size();
                pos2 = currentHRLE->run_types.size();
            }

            // update pos -> next run if the last break in the result is greater then the break of this run
            // if the break of the run in tempValue is greater as the last break in the result, go to the next run in the currentHRLE
            if ( returnValue->run_breaks.size() > 0 ) {

                while ( endCoord1 <= returnValue->run_breaks.back() ) {
                    pos1++;

                    // update start and end coordinate of the next run
                    startCoord1 = endCoord1;
                    if ( pos1 < tempValue->run_breaks.size()) endCoord1 = tempValue->run_breaks.at(pos1);
                    else endCoord1 = tempValue->max;

                    if (pos1 >= tempValue->run_types.size()) break;
                }

                while ( endCoord2 <= returnValue->run_breaks.back() ) {
                    pos2++;

                    // update start and end coordinate of the next run
                    startCoord2 = endCoord2;
                    if ( pos2 < currentHRLE->run_breaks.size()) endCoord2 = currentHRLE->run_breaks.at(pos2);
                    else endCoord2 = currentHRLE->max;

                    if (pos2 >= currentHRLE->run_types.size()) break;
                }

            }

            lastRun = returnValue->run_types.back();

        }

        delete currentHRLE;
        delete tempValue;
        tempValue = returnValue;

        if ( (tempValue->run_breaks.size() > 0) && (tempValue->run_breaks.back() == tempValue->max) ) tempValue->run_breaks.pop_back();

    }

    return tempValue;
}

// get the 6 direct neighbors
vector<bool> CHierarchicalRLE::neighbor_N6( vector<int> point, int segmentIndex ) {

    vector<bool>  returnValue, value;
    int           nCoord = point.back();
    int           i, segmentLength, firstRun, firstRunBreak, indexRun;
    short int     firstCoordinate, lastCoordinate, newFirstCoord;
    int           definedIndex = -1;
    int           run, tmpRun;
    point.pop_back();

    try {
        this->start_indices.at(segmentIndex);
    }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
        cerr << "Out of range exception " << e.what() << endl;
        cerr << "N6 neighborhood: no segment with index " << segmentIndex << endl;
        exit(1);
    }

    // get segment length - number of runs in this segment
    if (this->start_indices.at(segmentIndex) == this->start_indices.at(this->start_indices.size()-1))
        segmentLength = this->run_types.size() - this->start_indices.at(segmentIndex);
    else
        segmentLength = this->start_indices.at(segmentIndex+1) - this->start_indices.at(segmentIndex);

    // get the index of the first run in this segment
    firstRun = this->start_indices.at(segmentIndex);


    // get the index of the first run break of this segment
    firstRunBreak = this->start_indices.at(segmentIndex) - segmentIndex;

    firstCoordinate = this->min;
    lastCoordinate = this->max;

    // get the index of the run containing qi
    indexRun = this->accessBinSearch( nCoord, firstRun, firstRunBreak, segmentLength, &firstCoordinate, &lastCoordinate );

    // get the run code
    try {
        run = this->run_types.at(indexRun);
    }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
        cerr << "Out of range exception " << e.what() << endl;
        cerr << "N6 neighborhood: no run with index " << indexRun << endl;
        exit(1);
    }

    if ( !RT_DEFINED(run) ) {
      cerr << "Cannot get N6 neighborhood of non defined voxel!" << endl;
      return returnValue;
    }

    // higher dimensions
    if ( this->lower_hrle != NULL ) {

      // get the neighbor at nCoord - 1
      if ( (nCoord-1) >= this->min ) {

        // calculate defined data index
        if ( (nCoord-1) < firstCoordinate ) {
          newFirstCoord = this->min;
          try {
              tmpRun = this->getRunByCoordinate( nCoord-1, segmentIndex, &newFirstCoord );
          }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
              cerr << "Out of range exception " << e.what() << endl;
              cerr << "N6 neighborhood: get coordinate for nCoord-1 failed!" << indexRun << endl;
              exit(1);
          }

          if (RT_DEFINED(tmpRun)) {
            definedIndex = tmpRun + nCoord - 1 - newFirstCoord;
          }
          else {
            definedIndex = -1;
          }
        }
        else definedIndex = run + nCoord - 1 - firstCoordinate;

        // get the point via random access
        if ( definedIndex >= 0 ) {
          try {
              tmpRun = this->lower_hrle->randomAccess( point, definedIndex );
          }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
              cerr << "Out of range exception " << e.what() << endl;
              cerr << "N6 neighborhood: random access failed! (nCoord-1)" << endl;
              exit(1);
          }
          if ( RT_DEFINED(tmpRun) ) returnValue.push_back( true );
          else returnValue.push_back( false );
        }
        else returnValue.push_back( false );

      }
      else returnValue.push_back( false );

      // get the neighbor at nCoord + 1
      if ( (nCoord+1) < this->max ) {

        // calculate defines data index
        if ( (nCoord+1) >= lastCoordinate ) {
          try {
              tmpRun = this->run_types.at(indexRun+1);
          }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
              cerr << "Out of range exception " << e.what() << endl;
              cerr << "N6 neighborhood: no run with index " << (indexRun+1) << endl;
              exit(1);
          }
          if (RT_DEFINED(tmpRun))
            definedIndex = tmpRun + nCoord + 1 - lastCoordinate;
          else
            definedIndex = -1;
        }
        else definedIndex = run + nCoord + 1 - firstCoordinate;

        // get the point via random access
        if ( definedIndex >= 0 ) {
          try {
              tmpRun = this->lower_hrle->randomAccess( point, definedIndex );
          }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
              cerr << "Out of range exception " << e.what() << endl;
              cerr << "N6 neighborhood: random access failed! (nCoord+1)" << endl;
              exit(1);
          }
          if ( RT_DEFINED(tmpRun) ) returnValue.push_back( true );
          else returnValue.push_back( false );
        }
        else returnValue.push_back( false );
      }
      else returnValue.push_back( false );

      // get the neighbors from the next lower dimension
      definedIndex = run + nCoord - firstCoordinate;
      value = this->lower_hrle->neighbor_N6( point, definedIndex );
      for ( i = 0; i < value.size(); i++ ) returnValue.push_back( value.at(i) );
    }


    // lowest dimension; x axis
    else {
      // get the neighbor at nCoord-1
      if ( (nCoord-1) < firstCoordinate ) {

        if ((nCoord-1) >= this->min) {
          try {
              tmpRun = this->run_types.at(indexRun-1);
          }catch (std::out_of_range e) {
              cerr << "Out of range exception " << e.what() << endl;
              cerr << "N6 neighborhood (lowest dimension): no run with index " << (indexRun-1) << endl;
              exit(1);
          }
          if ( RT_DEFINED(tmpRun) ) returnValue.push_back( true );
          else returnValue.push_back( false );
        }
        else returnValue.push_back( false );

      }
      else returnValue.push_back( true );

      // get the neighbor at nCoord+1
      if ( (nCoord+1) >= lastCoordinate ) {

        if ((nCoord+1) < this->max) {
          try{
              tmpRun = this->run_types.at(indexRun+1);
          }catch (std::out_of_range e) {
              cerr << "Out of range exception " << e.what() << endl;
              cerr << "N6 neighborhood (lowest dimension): no run with index " << (indexRun+1) << endl;
              exit(1);
          }
          if ( RT_DEFINED(tmpRun) ) returnValue.push_back( true );
          else returnValue.push_back( false );
        }
        else returnValue.push_back( false );

      }
      else returnValue.push_back( true );
    }

    return returnValue;
}

// get the 26 neighbors
vector<bool> CHierarchicalRLE::neighbor_N26( vector<int> point, int segmentIndex ) {

    vector<bool>  returnValue;
    int           nCoord = point.back();
    int           i, stop, segmentLength, firstRun, firstRunBreak, indexRun;
    short int     firstCoordinate, lastCoordinate, newFirstCoord;
    int           definedIndex;
    int           run, tmpRun;
    point.pop_back();

    // get segment length - number of runs in this segment
    if (this->start_indices.at(segmentIndex) == this->start_indices.at(this->start_indices.size()-1))
        segmentLength = this->run_types.size() - this->start_indices.at(segmentIndex);
    else
        segmentLength = this->start_indices.at(segmentIndex+1) - this->start_indices.at(segmentIndex);

    // get the index of the first run in this segment
    firstRun = this->start_indices.at(segmentIndex);

    // get the index of the first run break of this segment
    firstRunBreak = this->start_indices.at(segmentIndex) - segmentIndex;

    firstCoordinate = this->min;
    lastCoordinate = this->max;

    // get the index of the run containing qi
    indexRun = this->accessBinSearch( nCoord, firstRun, firstRunBreak, segmentLength, &firstCoordinate, &lastCoordinate );

    // get the run code
    run = this->run_types.at(indexRun);

    if ( this->dimension == 3 ) stop = 9;
    if ( this->dimension == 2 ) stop = 3;

    // higher dimensions
    if ( this->dimension > 1 ) {

      // get neighbors at nCoord - 1
      if ((nCoord-1) >= this->min) {
        if ( (nCoord-1) >= firstCoordinate ) {
          if ( RT_DEFINED(run) ) {
            definedIndex = run + nCoord - 1 - firstCoordinate;
            vector<bool> value = this->lower_hrle->neighbor_N26( point, definedIndex );
            for ( i = 0; i < value.size(); i++ ) returnValue.push_back( value.at(i) );
          }
          else {
            for ( i = 0; i < stop; i++ ) returnValue.push_back( false );
          }
        }
        else {
          short int startCoord = this->min;
          tmpRun = this->getRunByCoordinate( nCoord-1, segmentIndex, &startCoord );
          if ( RT_DEFINED(tmpRun) ) {
            definedIndex = tmpRun + nCoord - 1 - startCoord;
            vector<bool> value = this->lower_hrle->neighbor_N26( point, definedIndex );
            for ( i = 0; i < value.size(); i++ ) returnValue.push_back( value.at(i) );
          }
          else {
            for ( i = 0; i < stop; i++ ) returnValue.push_back( false );
          }
        }
      }
      else {
        for ( i = 0; i < stop; i++ ) returnValue.push_back( false );
      }

      // get the neighbors at nCoord
      if ( RT_DEFINED(run) ) {
        definedIndex = run + nCoord - firstCoordinate;
        vector<bool> value = this->lower_hrle->neighbor_N26( point, definedIndex );
        for ( i = 0; i < value.size(); i++ ) returnValue.push_back( value.at(i) );
      }
      else {
        for ( i = 0; i < stop; i++ ) returnValue.push_back( false );
      }

      // get the neighbors at nCoord + 1
      if ((nCoord+1) < this->max) {
        if ( (nCoord+1) < lastCoordinate ) {
          if ( RT_DEFINED(run) ) {
            definedIndex = run + nCoord + 1 - firstCoordinate;
            vector<bool> value = this->lower_hrle->neighbor_N26( point, definedIndex );
            for ( i = 0; i < value.size(); i++ ) returnValue.push_back( value.at(i) );
          }
          else {
            for ( i = 0; i < stop; i++ ) returnValue.push_back( false );
          }
        }
        else {
          tmpRun = this->run_types.at(indexRun+1);
          if ( RT_DEFINED(tmpRun) ) {
            definedIndex = tmpRun + nCoord + 1 - lastCoordinate;
            vector<bool> value = this->lower_hrle->neighbor_N26( point, definedIndex );
            for ( i = 0; i < value.size(); i++ ) returnValue.push_back( value.at(i) );
          }
          else {
            for ( i = 0; i < stop; i++ ) returnValue.push_back( false );
          }
        }
      }
      else {
        for ( i = 0; i < stop; i++ ) returnValue.push_back( false );
      }

    }

    // lowest dimension
    else if ( this->dimension == 1 ) {
      // get the neighbor at nCoord-1
      if ( (nCoord-1) < firstCoordinate ) {
        if ((nCoord-1) >= this->min) {
          tmpRun = this->run_types.at(indexRun-1);
          if ( RT_DEFINED(tmpRun) ) returnValue.push_back( true );
          else returnValue.push_back( false );
        }
        else returnValue.push_back( false );
      }
      else {
        if ( RT_DEFINED(run) ) returnValue.push_back( true );
        else returnValue.push_back( false );
      }

      // get the neighbor at nCoord
      if ( RT_DEFINED(run) ) returnValue.push_back( true );
      else returnValue.push_back( false );

      // get the neighbor at nCoord+1
      if ( (nCoord+1) >= lastCoordinate ) {
        if ((nCoord+1) < this->max) {
          tmpRun = this->run_types.at(indexRun+1);
          if ( RT_DEFINED(tmpRun) ) returnValue.push_back( true );
          else returnValue.push_back( false );
        }
        else returnValue.push_back( false );
      }
      else {
        if ( RT_DEFINED(run) ) returnValue.push_back( true );
        else returnValue.push_back( false );
      }
    }
    else {
      cerr << "Error! N26 neighborhood not possible" << endl;
    }

    return returnValue;
}

// get the 18 neighbors
vector<bool> CHierarchicalRLE::neighbor_N18( vector<int> point, int segmentIndex ) {

    vector<bool>  returnValue, value;
    int           nCoord = point.back();
    int           i, segmentLength, firstRun, firstRunBreak, indexRun;
    short int     firstCoordinate, lastCoordinate, newFirstCoord;
    int           definedIndex;
    int           run, tmpRun;
    point.pop_back();

    // get segment length - number of runs in this segment
    if (this->start_indices.at(segmentIndex) == this->start_indices.at(this->start_indices.size()-1))
        segmentLength = this->run_types.size() - this->start_indices.at(segmentIndex);
    else
        segmentLength = this->start_indices.at(segmentIndex+1) - this->start_indices.at(segmentIndex);

    // get the index of the first run in this segment
    firstRun = this->start_indices.at(segmentIndex);

    // get the index of the first run break of this segment
    firstRunBreak = this->start_indices.at(segmentIndex) - segmentIndex;

    firstCoordinate = this->min;
    lastCoordinate = this->max;

    // get the index of the run containing qi
    indexRun = this->accessBinSearch( nCoord, firstRun, firstRunBreak, segmentLength, &firstCoordinate, &lastCoordinate );

    // get the run code
    run = this->run_types.at(indexRun);

//     stop = 3 * this->dimension;

    // higher dimensions
    if ( this->dimension == 3 ) {

      // get neighbors at nCoord - 1
      if ((nCoord-1) >= this->min) {
        if ( (nCoord-1) >= firstCoordinate ) {
          if ( RT_DEFINED(run) ) {
            definedIndex = run + nCoord - 1 - firstCoordinate;
            value = this->lower_hrle->neighbor_N26( point, definedIndex );
            returnValue.push_back( value.at(1) );
            for ( i = 3; i < 6; i++ ) returnValue.push_back( value.at(i) );
            returnValue.push_back( value.at(7) );
          }
          else {
            for ( i = 0; i < 5; i++ ) returnValue.push_back( false );
          }
        }
        else {
          short int startCoord = this->min;
          tmpRun = this->getRunByCoordinate( nCoord-1, segmentIndex, &startCoord );
          if ( RT_DEFINED(tmpRun) ) {
            definedIndex = tmpRun + nCoord - 1 - startCoord;
            value = this->lower_hrle->neighbor_N26( point, definedIndex );
            for ( i = 0; i < value.size(); i++ ) returnValue.push_back( value.at(i) );
          }
          else {
            for ( i = 0; i < 5; i++ ) returnValue.push_back( false );
          }
        }
      }
      else {
        for ( i = 0; i < 5; i++ ) returnValue.push_back( false );
      }

      // get the neighbors at nCoord
      if ( RT_DEFINED(run) ) {
        definedIndex = run + nCoord - firstCoordinate;
        value = this->lower_hrle->neighbor_N26( point, definedIndex );
        for ( i = 0; i < value.size(); i++ ) returnValue.push_back( value.at(i) );
      }
      else {
        for ( i = 0; i < 9; i++ ) returnValue.push_back( false );
      }

      // get the neighbors at nCoord + 1
      if ((nCoord+1) < this->max) {
        if ( (nCoord+1) < lastCoordinate ) {
          if ( RT_DEFINED(run) ) {
            definedIndex = run + nCoord + 1 - firstCoordinate;
            value = this->lower_hrle->neighbor_N26( point, definedIndex );
            for ( i = 0; i < value.size(); i++ ) returnValue.push_back( value.at(i) );
          }
          else {
            for ( i = 0; i < 5; i++ ) returnValue.push_back( false );
          }
        }
        else {
          tmpRun = this->run_types.at(indexRun+1);
          if ( RT_DEFINED(tmpRun) ) {
            definedIndex = tmpRun + nCoord + 1 - lastCoordinate;
            value = this->lower_hrle->neighbor_N26( point, definedIndex );
            returnValue.push_back( value.at(1) );
            for ( i = 3; i < 6; i++ ) returnValue.push_back( value.at(i) );
            returnValue.push_back( value.at(7) );
          }
          else {
            for ( i = 0; i < 5; i++ ) returnValue.push_back( false );
          }
        }
      }
      else {
        for ( i = 0; i < 5; i++ ) returnValue.push_back( false );
      }

    }
    else cout << "Cannot get N18 neighborhood!" << endl;

    return returnValue;

}

// count connected components in a 3x3x3 neighborhood
short int CHierarchicalRLE::countConnectedComponentsN3D_N26( vector<bool> inArray, connectivity con ) {

    vector<adjacency>           adjancenyTable;
    vector<short int>           outArray;
    vector<short int>           vArray;
    int                         i,j,k;
    int                         label = 0;
    bool                        found = false;

    adjacency       newAdjacency;
    for ( i = 0; i < inArray.size(); i++ ) {
        outArray.push_back(0);
        vArray.push_back(0);
    }

//     for ( i = 0; i < 28; i++ )
//         adjancenyTable.push_back( newAdjacency );
// 
//     if ( con == N26 ) {
// 
//         adjancenyTable[0].adjVoxels.push_back(1);
//         adjancenyTable[0].adjVoxels.push_back(3);
//         adjancenyTable[0].adjVoxels.push_back(4);
//         adjancenyTable[0].adjVoxels.push_back(9);
//         adjancenyTable[0].adjVoxels.push_back(10);
//         adjancenyTable[0].adjVoxels.push_back(12);
//         adjancenyTable[0].adjVoxels.push_back(13);
// 
//         adjancenyTable[1].adjVoxels.push_back(0);
//         adjancenyTable[1].adjVoxels.push_back(2);
//         adjancenyTable[1].adjVoxels.push_back(3);
//         adjancenyTable[1].adjVoxels.push_back(4);
//         adjancenyTable[1].adjVoxels.push_back(5);
//         adjancenyTable[1].adjVoxels.push_back(9);
//         adjancenyTable[1].adjVoxels.push_back(10);
//         adjancenyTable[1].adjVoxels.push_back(11);
//         adjancenyTable[1].adjVoxels.push_back(12);
//         adjancenyTable[1].adjVoxels.push_back(13);
//         adjancenyTable[1].adjVoxels.push_back(14);
// 
//         adjancenyTable[2].adjVoxels.push_back(1);
//         adjancenyTable[2].adjVoxels.push_back(4);
//         adjancenyTable[2].adjVoxels.push_back(5);
//         adjancenyTable[2].adjVoxels.push_back(10);
//         adjancenyTable[2].adjVoxels.push_back(11);
//         adjancenyTable[2].adjVoxels.push_back(13);
//         adjancenyTable[2].adjVoxels.push_back(14);
// 
//         adjancenyTable[3].adjVoxels.push_back(0);
//         adjancenyTable[3].adjVoxels.push_back(1);
//         adjancenyTable[3].adjVoxels.push_back(4);
//         adjancenyTable[3].adjVoxels.push_back(6);
//         adjancenyTable[3].adjVoxels.push_back(7);
//         adjancenyTable[3].adjVoxels.push_back(9);
//         adjancenyTable[3].adjVoxels.push_back(10);
//         adjancenyTable[3].adjVoxels.push_back(12);
//         adjancenyTable[3].adjVoxels.push_back(13);
//         adjancenyTable[3].adjVoxels.push_back(15);
//         adjancenyTable[3].adjVoxels.push_back(16);
// 
//         adjancenyTable[4].adjVoxels.push_back(0);
//         adjancenyTable[4].adjVoxels.push_back(1);
//         adjancenyTable[4].adjVoxels.push_back(2);
//         adjancenyTable[4].adjVoxels.push_back(3);
//         adjancenyTable[4].adjVoxels.push_back(5);
//         adjancenyTable[4].adjVoxels.push_back(6);
//         adjancenyTable[4].adjVoxels.push_back(7);
//         adjancenyTable[4].adjVoxels.push_back(8);
//         adjancenyTable[4].adjVoxels.push_back(9);
//         adjancenyTable[4].adjVoxels.push_back(10);
//         adjancenyTable[4].adjVoxels.push_back(11);
//         adjancenyTable[4].adjVoxels.push_back(12);
//         adjancenyTable[4].adjVoxels.push_back(13);
//         adjancenyTable[4].adjVoxels.push_back(14);
//         adjancenyTable[4].adjVoxels.push_back(15);
//         adjancenyTable[4].adjVoxels.push_back(16);
//         adjancenyTable[4].adjVoxels.push_back(17);
// 
//         adjancenyTable[5].adjVoxels.push_back(1);
//         adjancenyTable[5].adjVoxels.push_back(2);
//         adjancenyTable[5].adjVoxels.push_back(4);
//         adjancenyTable[5].adjVoxels.push_back(7);
//         adjancenyTable[5].adjVoxels.push_back(8);
//         adjancenyTable[5].adjVoxels.push_back(10);
//         adjancenyTable[5].adjVoxels.push_back(11);
//         adjancenyTable[5].adjVoxels.push_back(13);
//         adjancenyTable[5].adjVoxels.push_back(14);
//         adjancenyTable[5].adjVoxels.push_back(16);
//         adjancenyTable[5].adjVoxels.push_back(17);
// 
//         adjancenyTable[6].adjVoxels.push_back(3);
//         adjancenyTable[6].adjVoxels.push_back(4);
//         adjancenyTable[6].adjVoxels.push_back(7);
//         adjancenyTable[6].adjVoxels.push_back(12);
//         adjancenyTable[6].adjVoxels.push_back(13);
//         adjancenyTable[6].adjVoxels.push_back(15);
//         adjancenyTable[6].adjVoxels.push_back(16);
// 
//         adjancenyTable[7].adjVoxels.push_back(3);
//         adjancenyTable[7].adjVoxels.push_back(4);
//         adjancenyTable[7].adjVoxels.push_back(5);
//         adjancenyTable[7].adjVoxels.push_back(6);
//         adjancenyTable[7].adjVoxels.push_back(8);
//         adjancenyTable[7].adjVoxels.push_back(12);
//         adjancenyTable[7].adjVoxels.push_back(13);
//         adjancenyTable[7].adjVoxels.push_back(14);
//         adjancenyTable[7].adjVoxels.push_back(15);
//         adjancenyTable[7].adjVoxels.push_back(16);
//         adjancenyTable[7].adjVoxels.push_back(17);
// 
//         adjancenyTable[8].adjVoxels.push_back(4);
//         adjancenyTable[8].adjVoxels.push_back(5);
//         adjancenyTable[8].adjVoxels.push_back(7);
//         adjancenyTable[8].adjVoxels.push_back(13);
//         adjancenyTable[8].adjVoxels.push_back(14);
//         adjancenyTable[8].adjVoxels.push_back(16);
//         adjancenyTable[8].adjVoxels.push_back(17);
// 
//         adjancenyTable[9].adjVoxels.push_back(0);
//         adjancenyTable[9].adjVoxels.push_back(1);
//         adjancenyTable[9].adjVoxels.push_back(3);
//         adjancenyTable[9].adjVoxels.push_back(4);
//         adjancenyTable[9].adjVoxels.push_back(10);
//         adjancenyTable[9].adjVoxels.push_back(12);
//         adjancenyTable[9].adjVoxels.push_back(13);
//         adjancenyTable[9].adjVoxels.push_back(18);
//         adjancenyTable[9].adjVoxels.push_back(19);
//         adjancenyTable[9].adjVoxels.push_back(21);
//         adjancenyTable[9].adjVoxels.push_back(22);
// 
//         adjancenyTable[10].adjVoxels.push_back(0);
//         adjancenyTable[10].adjVoxels.push_back(1);
//         adjancenyTable[10].adjVoxels.push_back(2);
//         adjancenyTable[10].adjVoxels.push_back(3);
//         adjancenyTable[10].adjVoxels.push_back(4);
//         adjancenyTable[10].adjVoxels.push_back(5);
//         adjancenyTable[10].adjVoxels.push_back(9);
//         adjancenyTable[10].adjVoxels.push_back(11);
//         adjancenyTable[10].adjVoxels.push_back(12);
//         adjancenyTable[10].adjVoxels.push_back(13);
//         adjancenyTable[10].adjVoxels.push_back(14);
//         adjancenyTable[10].adjVoxels.push_back(18);
//         adjancenyTable[10].adjVoxels.push_back(19);
//         adjancenyTable[10].adjVoxels.push_back(20);
//         adjancenyTable[10].adjVoxels.push_back(21);
//         adjancenyTable[10].adjVoxels.push_back(22);
//         adjancenyTable[10].adjVoxels.push_back(23);
// 
//         adjancenyTable[11].adjVoxels.push_back(1);
//         adjancenyTable[11].adjVoxels.push_back(2);
//         adjancenyTable[11].adjVoxels.push_back(4);
//         adjancenyTable[11].adjVoxels.push_back(5);
//         adjancenyTable[11].adjVoxels.push_back(10);
//         adjancenyTable[11].adjVoxels.push_back(13);
//         adjancenyTable[11].adjVoxels.push_back(14);
//         adjancenyTable[11].adjVoxels.push_back(19);
//         adjancenyTable[11].adjVoxels.push_back(20);
//         adjancenyTable[11].adjVoxels.push_back(22);
//         adjancenyTable[11].adjVoxels.push_back(23);
// 
//         adjancenyTable[12].adjVoxels.push_back(0);
//         adjancenyTable[12].adjVoxels.push_back(1);
//         adjancenyTable[12].adjVoxels.push_back(3);
//         adjancenyTable[12].adjVoxels.push_back(4);
//         adjancenyTable[12].adjVoxels.push_back(6);
//         adjancenyTable[12].adjVoxels.push_back(7);
//         adjancenyTable[12].adjVoxels.push_back(9);
//         adjancenyTable[12].adjVoxels.push_back(10);
//         adjancenyTable[12].adjVoxels.push_back(13);
//         adjancenyTable[12].adjVoxels.push_back(15);
//         adjancenyTable[12].adjVoxels.push_back(16);
//         adjancenyTable[12].adjVoxels.push_back(18);
//         adjancenyTable[12].adjVoxels.push_back(19);
//         adjancenyTable[12].adjVoxels.push_back(21);
//         adjancenyTable[12].adjVoxels.push_back(22);
//         adjancenyTable[12].adjVoxels.push_back(24);
//         adjancenyTable[12].adjVoxels.push_back(25);
// 
//         adjancenyTable[13].adjVoxels.push_back(0);
//         adjancenyTable[13].adjVoxels.push_back(1);
//         adjancenyTable[13].adjVoxels.push_back(2);
//         adjancenyTable[13].adjVoxels.push_back(3);
//         adjancenyTable[13].adjVoxels.push_back(4);
//         adjancenyTable[13].adjVoxels.push_back(5);
//         adjancenyTable[13].adjVoxels.push_back(6);
//         adjancenyTable[13].adjVoxels.push_back(7);
//         adjancenyTable[13].adjVoxels.push_back(8);
//         adjancenyTable[13].adjVoxels.push_back(9);
//         adjancenyTable[13].adjVoxels.push_back(10);
//         adjancenyTable[13].adjVoxels.push_back(11);
//         adjancenyTable[13].adjVoxels.push_back(12);
//         adjancenyTable[13].adjVoxels.push_back(14);
//         adjancenyTable[13].adjVoxels.push_back(15);
//         adjancenyTable[13].adjVoxels.push_back(16);
//         adjancenyTable[13].adjVoxels.push_back(17);
//         adjancenyTable[13].adjVoxels.push_back(18);
//         adjancenyTable[13].adjVoxels.push_back(19);
//         adjancenyTable[13].adjVoxels.push_back(20);
//         adjancenyTable[13].adjVoxels.push_back(21);
//         adjancenyTable[13].adjVoxels.push_back(22);
//         adjancenyTable[13].adjVoxels.push_back(23);
//         adjancenyTable[13].adjVoxels.push_back(24);
//         adjancenyTable[13].adjVoxels.push_back(25);
//         adjancenyTable[13].adjVoxels.push_back(26);
// 
//         adjancenyTable[14].adjVoxels.push_back(1);
//         adjancenyTable[14].adjVoxels.push_back(2);
//         adjancenyTable[14].adjVoxels.push_back(4);
//         adjancenyTable[14].adjVoxels.push_back(5);
//         adjancenyTable[14].adjVoxels.push_back(7);
//         adjancenyTable[14].adjVoxels.push_back(8);
//         adjancenyTable[14].adjVoxels.push_back(10);
//         adjancenyTable[14].adjVoxels.push_back(11);
//         adjancenyTable[14].adjVoxels.push_back(13);
//         adjancenyTable[14].adjVoxels.push_back(16);
//         adjancenyTable[14].adjVoxels.push_back(17);
//         adjancenyTable[14].adjVoxels.push_back(19);
//         adjancenyTable[14].adjVoxels.push_back(20);
//         adjancenyTable[14].adjVoxels.push_back(22);
//         adjancenyTable[14].adjVoxels.push_back(23);
//         adjancenyTable[14].adjVoxels.push_back(25);
//         adjancenyTable[14].adjVoxels.push_back(26);
// 
//         adjancenyTable[15].adjVoxels.push_back(3);
//         adjancenyTable[15].adjVoxels.push_back(4);
//         adjancenyTable[15].adjVoxels.push_back(6);
//         adjancenyTable[15].adjVoxels.push_back(7);
//         adjancenyTable[15].adjVoxels.push_back(12);
//         adjancenyTable[15].adjVoxels.push_back(13);
//         adjancenyTable[15].adjVoxels.push_back(16);
//         adjancenyTable[15].adjVoxels.push_back(21);
//         adjancenyTable[15].adjVoxels.push_back(22);
//         adjancenyTable[15].adjVoxels.push_back(24);
//         adjancenyTable[15].adjVoxels.push_back(25);
// 
//         adjancenyTable[16].adjVoxels.push_back(3);
//         adjancenyTable[16].adjVoxels.push_back(4);
//         adjancenyTable[16].adjVoxels.push_back(5);
//         adjancenyTable[16].adjVoxels.push_back(6);
//         adjancenyTable[16].adjVoxels.push_back(7);
//         adjancenyTable[16].adjVoxels.push_back(8);
//         adjancenyTable[16].adjVoxels.push_back(12);
//         adjancenyTable[16].adjVoxels.push_back(13);
//         adjancenyTable[16].adjVoxels.push_back(14);
//         adjancenyTable[16].adjVoxels.push_back(15);
//         adjancenyTable[16].adjVoxels.push_back(17);
//         adjancenyTable[16].adjVoxels.push_back(21);
//         adjancenyTable[16].adjVoxels.push_back(22);
//         adjancenyTable[16].adjVoxels.push_back(23);
//         adjancenyTable[16].adjVoxels.push_back(24);
//         adjancenyTable[16].adjVoxels.push_back(25);
//         adjancenyTable[16].adjVoxels.push_back(26);
// 
//         adjancenyTable[17].adjVoxels.push_back(4);
//         adjancenyTable[17].adjVoxels.push_back(5);
//         adjancenyTable[17].adjVoxels.push_back(7);
//         adjancenyTable[17].adjVoxels.push_back(8);
//         adjancenyTable[17].adjVoxels.push_back(13);
//         adjancenyTable[17].adjVoxels.push_back(14);
//         adjancenyTable[17].adjVoxels.push_back(16);
//         adjancenyTable[17].adjVoxels.push_back(22);
//         adjancenyTable[17].adjVoxels.push_back(23);
//         adjancenyTable[17].adjVoxels.push_back(25);
//         adjancenyTable[17].adjVoxels.push_back(26);
// 
//         adjancenyTable[18].adjVoxels.push_back(9);
//         adjancenyTable[18].adjVoxels.push_back(10);
//         adjancenyTable[18].adjVoxels.push_back(12);
//         adjancenyTable[18].adjVoxels.push_back(13);
//         adjancenyTable[18].adjVoxels.push_back(19);
//         adjancenyTable[18].adjVoxels.push_back(21);
//         adjancenyTable[18].adjVoxels.push_back(22);
// 
//         adjancenyTable[19].adjVoxels.push_back(9);
//         adjancenyTable[19].adjVoxels.push_back(10);
//         adjancenyTable[19].adjVoxels.push_back(11);
//         adjancenyTable[19].adjVoxels.push_back(12);
//         adjancenyTable[19].adjVoxels.push_back(13);
//         adjancenyTable[19].adjVoxels.push_back(14);
//         adjancenyTable[19].adjVoxels.push_back(18);
//         adjancenyTable[19].adjVoxels.push_back(20);
//         adjancenyTable[19].adjVoxels.push_back(21);
//         adjancenyTable[19].adjVoxels.push_back(22);
//         adjancenyTable[19].adjVoxels.push_back(23);
// 
//         adjancenyTable[20].adjVoxels.push_back(10);
//         adjancenyTable[20].adjVoxels.push_back(11);
//         adjancenyTable[20].adjVoxels.push_back(13);
//         adjancenyTable[20].adjVoxels.push_back(14);
//         adjancenyTable[20].adjVoxels.push_back(19);
//         adjancenyTable[20].adjVoxels.push_back(22);
//         adjancenyTable[20].adjVoxels.push_back(23);
// 
//         adjancenyTable[21].adjVoxels.push_back(9);
//         adjancenyTable[21].adjVoxels.push_back(10);
//         adjancenyTable[21].adjVoxels.push_back(12);
//         adjancenyTable[21].adjVoxels.push_back(13);
//         adjancenyTable[21].adjVoxels.push_back(15);
//         adjancenyTable[21].adjVoxels.push_back(16);
//         adjancenyTable[21].adjVoxels.push_back(18);
//         adjancenyTable[21].adjVoxels.push_back(19);
//         adjancenyTable[21].adjVoxels.push_back(22);
//         adjancenyTable[21].adjVoxels.push_back(24);
//         adjancenyTable[21].adjVoxels.push_back(25);
// 
//         adjancenyTable[22].adjVoxels.push_back(9);
//         adjancenyTable[22].adjVoxels.push_back(10);
//         adjancenyTable[22].adjVoxels.push_back(11);
//         adjancenyTable[22].adjVoxels.push_back(12);
//         adjancenyTable[22].adjVoxels.push_back(13);
//         adjancenyTable[22].adjVoxels.push_back(14);
//         adjancenyTable[22].adjVoxels.push_back(15);
//         adjancenyTable[22].adjVoxels.push_back(16);
//         adjancenyTable[22].adjVoxels.push_back(17);
//         adjancenyTable[22].adjVoxels.push_back(18);
//         adjancenyTable[22].adjVoxels.push_back(19);
//         adjancenyTable[22].adjVoxels.push_back(20);
//         adjancenyTable[22].adjVoxels.push_back(21);
//         adjancenyTable[22].adjVoxels.push_back(23);
//         adjancenyTable[22].adjVoxels.push_back(24);
//         adjancenyTable[22].adjVoxels.push_back(25);
//         adjancenyTable[22].adjVoxels.push_back(26);
// 
//         adjancenyTable[23].adjVoxels.push_back(10);
//         adjancenyTable[23].adjVoxels.push_back(11);
//         adjancenyTable[23].adjVoxels.push_back(13);
//         adjancenyTable[23].adjVoxels.push_back(14);
//         adjancenyTable[23].adjVoxels.push_back(16);
//         adjancenyTable[23].adjVoxels.push_back(17);
//         adjancenyTable[23].adjVoxels.push_back(19);
//         adjancenyTable[23].adjVoxels.push_back(20);
//         adjancenyTable[23].adjVoxels.push_back(22);
//         adjancenyTable[23].adjVoxels.push_back(25);
//         adjancenyTable[23].adjVoxels.push_back(26);
// 
//         adjancenyTable[24].adjVoxels.push_back(12);
//         adjancenyTable[24].adjVoxels.push_back(13);
//         adjancenyTable[24].adjVoxels.push_back(15);
//         adjancenyTable[24].adjVoxels.push_back(16);
//         adjancenyTable[24].adjVoxels.push_back(21);
//         adjancenyTable[24].adjVoxels.push_back(22);
//         adjancenyTable[24].adjVoxels.push_back(25);
// 
//         adjancenyTable[25].adjVoxels.push_back(12);
//         adjancenyTable[25].adjVoxels.push_back(13);
//         adjancenyTable[25].adjVoxels.push_back(14);
//         adjancenyTable[25].adjVoxels.push_back(15);
//         adjancenyTable[25].adjVoxels.push_back(16);
//         adjancenyTable[25].adjVoxels.push_back(17);
//         adjancenyTable[25].adjVoxels.push_back(21);
//         adjancenyTable[25].adjVoxels.push_back(22);
//         adjancenyTable[25].adjVoxels.push_back(23);
//         adjancenyTable[25].adjVoxels.push_back(24);
//         adjancenyTable[25].adjVoxels.push_back(26);
// 
//         adjancenyTable[26].adjVoxels.push_back(13);
//         adjancenyTable[26].adjVoxels.push_back(14);
//         adjancenyTable[26].adjVoxels.push_back(16);
//         adjancenyTable[26].adjVoxels.push_back(17);
//         adjancenyTable[26].adjVoxels.push_back(22);
//         adjancenyTable[26].adjVoxels.push_back(23);
//         adjancenyTable[26].adjVoxels.push_back(25);
//     }
//     else if ( con == N6 ) {
//         adjancenyTable[0].adjVoxels.push_back(1);
//         adjancenyTable[0].adjVoxels.push_back(3);
//         adjancenyTable[0].adjVoxels.push_back(9);
// 
//         adjancenyTable[1].adjVoxels.push_back(0);
//         adjancenyTable[1].adjVoxels.push_back(2);
//         adjancenyTable[1].adjVoxels.push_back(4);
//         adjancenyTable[1].adjVoxels.push_back(10);
// 
//         adjancenyTable[2].adjVoxels.push_back(1);
//         adjancenyTable[2].adjVoxels.push_back(5);
//         adjancenyTable[2].adjVoxels.push_back(11);
// 
//         adjancenyTable[3].adjVoxels.push_back(0);
//         adjancenyTable[3].adjVoxels.push_back(4);
//         adjancenyTable[3].adjVoxels.push_back(6);
//         adjancenyTable[3].adjVoxels.push_back(12);
// 
//         adjancenyTable[4].adjVoxels.push_back(1);
//         adjancenyTable[4].adjVoxels.push_back(3);
//         adjancenyTable[4].adjVoxels.push_back(5);
//         adjancenyTable[4].adjVoxels.push_back(7);
//         adjancenyTable[4].adjVoxels.push_back(13);
// 
//         adjancenyTable[5].adjVoxels.push_back(2);
//         adjancenyTable[5].adjVoxels.push_back(4);
//         adjancenyTable[5].adjVoxels.push_back(8);
//         adjancenyTable[5].adjVoxels.push_back(14);
// 
//         adjancenyTable[6].adjVoxels.push_back(3);
//         adjancenyTable[6].adjVoxels.push_back(7);
//         adjancenyTable[6].adjVoxels.push_back(15);
// 
//         adjancenyTable[7].adjVoxels.push_back(4);
//         adjancenyTable[7].adjVoxels.push_back(6);
//         adjancenyTable[7].adjVoxels.push_back(8);
//         adjancenyTable[7].adjVoxels.push_back(16);
// 
//         adjancenyTable[8].adjVoxels.push_back(5);
//         adjancenyTable[8].adjVoxels.push_back(7);
//         adjancenyTable[8].adjVoxels.push_back(17);
// 
//         adjancenyTable[9].adjVoxels.push_back(0);
//         adjancenyTable[9].adjVoxels.push_back(10);
//         adjancenyTable[9].adjVoxels.push_back(12);
//         adjancenyTable[9].adjVoxels.push_back(18);
// 
//         adjancenyTable[10].adjVoxels.push_back(1);
//         adjancenyTable[10].adjVoxels.push_back(9);
//         adjancenyTable[10].adjVoxels.push_back(11);
//         adjancenyTable[10].adjVoxels.push_back(13);
//         adjancenyTable[10].adjVoxels.push_back(19);
// 
//         adjancenyTable[11].adjVoxels.push_back(2);
//         adjancenyTable[11].adjVoxels.push_back(10);
//         adjancenyTable[11].adjVoxels.push_back(14);
//         adjancenyTable[11].adjVoxels.push_back(20);
// 
//         adjancenyTable[12].adjVoxels.push_back(3);
//         adjancenyTable[12].adjVoxels.push_back(9);
//         adjancenyTable[12].adjVoxels.push_back(13);
//         adjancenyTable[12].adjVoxels.push_back(15);
//         adjancenyTable[12].adjVoxels.push_back(21);
// 
//         adjancenyTable[13].adjVoxels.push_back(4);
//         adjancenyTable[13].adjVoxels.push_back(10);
//         adjancenyTable[13].adjVoxels.push_back(12);
//         adjancenyTable[13].adjVoxels.push_back(14);
//         adjancenyTable[13].adjVoxels.push_back(16);
//         adjancenyTable[13].adjVoxels.push_back(22);
// 
//         adjancenyTable[14].adjVoxels.push_back(5);
//         adjancenyTable[14].adjVoxels.push_back(11);
//         adjancenyTable[14].adjVoxels.push_back(13);
//         adjancenyTable[14].adjVoxels.push_back(17);
//         adjancenyTable[14].adjVoxels.push_back(23);
// 
//         adjancenyTable[15].adjVoxels.push_back(6);
//         adjancenyTable[15].adjVoxels.push_back(12);
//         adjancenyTable[15].adjVoxels.push_back(16);
//         adjancenyTable[15].adjVoxels.push_back(24);
// 
//         adjancenyTable[16].adjVoxels.push_back(7);
//         adjancenyTable[16].adjVoxels.push_back(13);
//         adjancenyTable[16].adjVoxels.push_back(15);
//         adjancenyTable[16].adjVoxels.push_back(17);
//         adjancenyTable[16].adjVoxels.push_back(25);
// 
//         adjancenyTable[17].adjVoxels.push_back(8);
//         adjancenyTable[17].adjVoxels.push_back(14);
//         adjancenyTable[17].adjVoxels.push_back(16);
//         adjancenyTable[17].adjVoxels.push_back(26);
// 
//         adjancenyTable[18].adjVoxels.push_back(9);
//         adjancenyTable[18].adjVoxels.push_back(19);
//         adjancenyTable[18].adjVoxels.push_back(21);
// 
//         adjancenyTable[19].adjVoxels.push_back(10);
//         adjancenyTable[19].adjVoxels.push_back(18);
//         adjancenyTable[19].adjVoxels.push_back(20);
//         adjancenyTable[19].adjVoxels.push_back(22);
// 
//         adjancenyTable[20].adjVoxels.push_back(11);
//         adjancenyTable[20].adjVoxels.push_back(19);
//         adjancenyTable[20].adjVoxels.push_back(23);
// 
//         adjancenyTable[21].adjVoxels.push_back(12);
//         adjancenyTable[21].adjVoxels.push_back(18);
//         adjancenyTable[21].adjVoxels.push_back(22);
//         adjancenyTable[21].adjVoxels.push_back(24);
// 
//         adjancenyTable[22].adjVoxels.push_back(13);
//         adjancenyTable[22].adjVoxels.push_back(19);
//         adjancenyTable[22].adjVoxels.push_back(21);
//         adjancenyTable[22].adjVoxels.push_back(23);
//         adjancenyTable[22].adjVoxels.push_back(25);
// 
//         adjancenyTable[23].adjVoxels.push_back(14);
//         adjancenyTable[23].adjVoxels.push_back(20);
//         adjancenyTable[23].adjVoxels.push_back(22);
//         adjancenyTable[23].adjVoxels.push_back(26);
// 
//         adjancenyTable[24].adjVoxels.push_back(15);
//         adjancenyTable[24].adjVoxels.push_back(21);
//         adjancenyTable[24].adjVoxels.push_back(25);
// 
//         adjancenyTable[25].adjVoxels.push_back(16);
//         adjancenyTable[25].adjVoxels.push_back(22);
//         adjancenyTable[25].adjVoxels.push_back(24);
//         adjancenyTable[25].adjVoxels.push_back(26);
// 
//         adjancenyTable[26].adjVoxels.push_back(17);
//         adjancenyTable[26].adjVoxels.push_back(23);
//         adjancenyTable[26].adjVoxels.push_back(25);
//     }
//     else return -1;

    if ( con == N26 ) {
        if ( this->adjancenyTableN26.size() == 0 ) this->createAdjacencyTables();
        adjancenyTable = this->adjancenyTableN26;
    }
    else if ( con == N6 ) {
        if ( this->adjancenyTableN6.size() == 0 ) this->createAdjacencyTables();
        adjancenyTable = this->adjancenyTableN6;
    }
    else return -1;


    for ( i = 0; i < inArray.size(); i++ ) {
        if ( (inArray.at(i) == true) && (outArray.at(i) == 0) ) {
            outArray[i] = ++label;
            vArray[i] = 1;

            do {
                found = false;
                for ( j = 0; j < vArray.size(); j++ ) {
                    if ( vArray.at(j) == 1 ) {

                        for ( k = 0; k < adjancenyTable[j].adjVoxels.size(); k++ ) {
                            if ( (inArray.at(adjancenyTable[j].adjVoxels.at(k)) == true) &&
                                 (outArray.at(adjancenyTable[j].adjVoxels.at(k)) == 0 ) ) {
                                outArray[adjancenyTable[j].adjVoxels.at(k)] = label;
                                vArray[adjancenyTable[j].adjVoxels.at(k)] = 1;
                                found = true;
                            }
                        }

                        vArray[j] = 2;
                    }
                }
            } while (found);

        }
    }

    return label;

}



// get connected components in a small 3x3x3 neighborhood
vector<short int> CHierarchicalRLE::connectedComponentsN3D_N26( vector<bool> inArray, connectivity con ) {

    vector<adjacency>           adjancenyTable;
    vector<short int>           outArray;
    vector<short int>           vArray;
    int                         i,j,k;
    int                         label = 0;
    bool                        found = false;

    adjacency       newAdjacency;
    for ( i = 0; i < inArray.size(); i++ ) {
        outArray.push_back(0);
        vArray.push_back(0);
    }

//     for ( i = 0; i < 28; i++ )
//         adjancenyTable.push_back( newAdjacency );

//     if ( con == N26 ) {
// 
//         adjancenyTable[0].adjVoxels.push_back(1);
//         adjancenyTable[0].adjVoxels.push_back(3);
//         adjancenyTable[0].adjVoxels.push_back(4);
//         adjancenyTable[0].adjVoxels.push_back(9);
//         adjancenyTable[0].adjVoxels.push_back(10);
//         adjancenyTable[0].adjVoxels.push_back(12);
//         adjancenyTable[0].adjVoxels.push_back(13);
// 
//         adjancenyTable[1].adjVoxels.push_back(0);
//         adjancenyTable[1].adjVoxels.push_back(2);
//         adjancenyTable[1].adjVoxels.push_back(3);
//         adjancenyTable[1].adjVoxels.push_back(4);
//         adjancenyTable[1].adjVoxels.push_back(5);
//         adjancenyTable[1].adjVoxels.push_back(9);
//         adjancenyTable[1].adjVoxels.push_back(10);
//         adjancenyTable[1].adjVoxels.push_back(11);
//         adjancenyTable[1].adjVoxels.push_back(12);
//         adjancenyTable[1].adjVoxels.push_back(13);
//         adjancenyTable[1].adjVoxels.push_back(14);
// 
//         adjancenyTable[2].adjVoxels.push_back(1);
//         adjancenyTable[2].adjVoxels.push_back(4);
//         adjancenyTable[2].adjVoxels.push_back(5);
//         adjancenyTable[2].adjVoxels.push_back(10);
//         adjancenyTable[2].adjVoxels.push_back(11);
//         adjancenyTable[2].adjVoxels.push_back(13);
//         adjancenyTable[2].adjVoxels.push_back(14);
// 
//         adjancenyTable[3].adjVoxels.push_back(0);
//         adjancenyTable[3].adjVoxels.push_back(1);
//         adjancenyTable[3].adjVoxels.push_back(4);
//         adjancenyTable[3].adjVoxels.push_back(6);
//         adjancenyTable[3].adjVoxels.push_back(7);
//         adjancenyTable[3].adjVoxels.push_back(9);
//         adjancenyTable[3].adjVoxels.push_back(10);
//         adjancenyTable[3].adjVoxels.push_back(12);
//         adjancenyTable[3].adjVoxels.push_back(13);
//         adjancenyTable[3].adjVoxels.push_back(15);
//         adjancenyTable[3].adjVoxels.push_back(16);
// 
//         adjancenyTable[4].adjVoxels.push_back(0);
//         adjancenyTable[4].adjVoxels.push_back(1);
//         adjancenyTable[4].adjVoxels.push_back(2);
//         adjancenyTable[4].adjVoxels.push_back(3);
//         adjancenyTable[4].adjVoxels.push_back(5);
//         adjancenyTable[4].adjVoxels.push_back(6);
//         adjancenyTable[4].adjVoxels.push_back(7);
//         adjancenyTable[4].adjVoxels.push_back(8);
//         adjancenyTable[4].adjVoxels.push_back(9);
//         adjancenyTable[4].adjVoxels.push_back(10);
//         adjancenyTable[4].adjVoxels.push_back(11);
//         adjancenyTable[4].adjVoxels.push_back(12);
//         adjancenyTable[4].adjVoxels.push_back(13);
//         adjancenyTable[4].adjVoxels.push_back(14);
//         adjancenyTable[4].adjVoxels.push_back(15);
//         adjancenyTable[4].adjVoxels.push_back(16);
//         adjancenyTable[4].adjVoxels.push_back(17);
// 
//         adjancenyTable[5].adjVoxels.push_back(1);
//         adjancenyTable[5].adjVoxels.push_back(2);
//         adjancenyTable[5].adjVoxels.push_back(4);
//         adjancenyTable[5].adjVoxels.push_back(7);
//         adjancenyTable[5].adjVoxels.push_back(8);
//         adjancenyTable[5].adjVoxels.push_back(10);
//         adjancenyTable[5].adjVoxels.push_back(11);
//         adjancenyTable[5].adjVoxels.push_back(13);
//         adjancenyTable[5].adjVoxels.push_back(14);
//         adjancenyTable[5].adjVoxels.push_back(16);
//         adjancenyTable[5].adjVoxels.push_back(17);
// 
//         adjancenyTable[6].adjVoxels.push_back(3);
//         adjancenyTable[6].adjVoxels.push_back(4);
//         adjancenyTable[6].adjVoxels.push_back(7);
//         adjancenyTable[6].adjVoxels.push_back(12);
//         adjancenyTable[6].adjVoxels.push_back(13);
//         adjancenyTable[6].adjVoxels.push_back(15);
//         adjancenyTable[6].adjVoxels.push_back(16);
// 
//         adjancenyTable[7].adjVoxels.push_back(3);
//         adjancenyTable[7].adjVoxels.push_back(4);
//         adjancenyTable[7].adjVoxels.push_back(5);
//         adjancenyTable[7].adjVoxels.push_back(6);
//         adjancenyTable[7].adjVoxels.push_back(8);
//         adjancenyTable[7].adjVoxels.push_back(12);
//         adjancenyTable[7].adjVoxels.push_back(13);
//         adjancenyTable[7].adjVoxels.push_back(14);
//         adjancenyTable[7].adjVoxels.push_back(15);
//         adjancenyTable[7].adjVoxels.push_back(16);
//         adjancenyTable[7].adjVoxels.push_back(17);
// 
//         adjancenyTable[8].adjVoxels.push_back(4);
//         adjancenyTable[8].adjVoxels.push_back(5);
//         adjancenyTable[8].adjVoxels.push_back(7);
//         adjancenyTable[8].adjVoxels.push_back(13);
//         adjancenyTable[8].adjVoxels.push_back(14);
//         adjancenyTable[8].adjVoxels.push_back(16);
//         adjancenyTable[8].adjVoxels.push_back(17);
// 
//         adjancenyTable[9].adjVoxels.push_back(0);
//         adjancenyTable[9].adjVoxels.push_back(1);
//         adjancenyTable[9].adjVoxels.push_back(3);
//         adjancenyTable[9].adjVoxels.push_back(4);
//         adjancenyTable[9].adjVoxels.push_back(10);
//         adjancenyTable[9].adjVoxels.push_back(12);
//         adjancenyTable[9].adjVoxels.push_back(13);
//         adjancenyTable[9].adjVoxels.push_back(18);
//         adjancenyTable[9].adjVoxels.push_back(19);
//         adjancenyTable[9].adjVoxels.push_back(21);
//         adjancenyTable[9].adjVoxels.push_back(22);
// 
//         adjancenyTable[10].adjVoxels.push_back(0);
//         adjancenyTable[10].adjVoxels.push_back(1);
//         adjancenyTable[10].adjVoxels.push_back(2);
//         adjancenyTable[10].adjVoxels.push_back(3);
//         adjancenyTable[10].adjVoxels.push_back(4);
//         adjancenyTable[10].adjVoxels.push_back(5);
//         adjancenyTable[10].adjVoxels.push_back(9);
//         adjancenyTable[10].adjVoxels.push_back(11);
//         adjancenyTable[10].adjVoxels.push_back(12);
//         adjancenyTable[10].adjVoxels.push_back(13);
//         adjancenyTable[10].adjVoxels.push_back(14);
//         adjancenyTable[10].adjVoxels.push_back(18);
//         adjancenyTable[10].adjVoxels.push_back(19);
//         adjancenyTable[10].adjVoxels.push_back(20);
//         adjancenyTable[10].adjVoxels.push_back(21);
//         adjancenyTable[10].adjVoxels.push_back(22);
//         adjancenyTable[10].adjVoxels.push_back(23);
// 
//         adjancenyTable[11].adjVoxels.push_back(1);
//         adjancenyTable[11].adjVoxels.push_back(2);
//         adjancenyTable[11].adjVoxels.push_back(4);
//         adjancenyTable[11].adjVoxels.push_back(5);
//         adjancenyTable[11].adjVoxels.push_back(10);
//         adjancenyTable[11].adjVoxels.push_back(13);
//         adjancenyTable[11].adjVoxels.push_back(14);
//         adjancenyTable[11].adjVoxels.push_back(19);
//         adjancenyTable[11].adjVoxels.push_back(20);
//         adjancenyTable[11].adjVoxels.push_back(22);
//         adjancenyTable[11].adjVoxels.push_back(23);
// 
//         adjancenyTable[12].adjVoxels.push_back(0);
//         adjancenyTable[12].adjVoxels.push_back(1);
//         adjancenyTable[12].adjVoxels.push_back(3);
//         adjancenyTable[12].adjVoxels.push_back(4);
//         adjancenyTable[12].adjVoxels.push_back(6);
//         adjancenyTable[12].adjVoxels.push_back(7);
//         adjancenyTable[12].adjVoxels.push_back(9);
//         adjancenyTable[12].adjVoxels.push_back(10);
//         adjancenyTable[12].adjVoxels.push_back(13);
//         adjancenyTable[12].adjVoxels.push_back(15);
//         adjancenyTable[12].adjVoxels.push_back(16);
//         adjancenyTable[12].adjVoxels.push_back(18);
//         adjancenyTable[12].adjVoxels.push_back(19);
//         adjancenyTable[12].adjVoxels.push_back(21);
//         adjancenyTable[12].adjVoxels.push_back(22);
//         adjancenyTable[12].adjVoxels.push_back(24);
//         adjancenyTable[12].adjVoxels.push_back(25);
// 
//         adjancenyTable[13].adjVoxels.push_back(0);
//         adjancenyTable[13].adjVoxels.push_back(1);
//         adjancenyTable[13].adjVoxels.push_back(2);
//         adjancenyTable[13].adjVoxels.push_back(3);
//         adjancenyTable[13].adjVoxels.push_back(4);
//         adjancenyTable[13].adjVoxels.push_back(5);
//         adjancenyTable[13].adjVoxels.push_back(6);
//         adjancenyTable[13].adjVoxels.push_back(7);
//         adjancenyTable[13].adjVoxels.push_back(8);
//         adjancenyTable[13].adjVoxels.push_back(9);
//         adjancenyTable[13].adjVoxels.push_back(10);
//         adjancenyTable[13].adjVoxels.push_back(11);
//         adjancenyTable[13].adjVoxels.push_back(12);
//         adjancenyTable[13].adjVoxels.push_back(14);
//         adjancenyTable[13].adjVoxels.push_back(15);
//         adjancenyTable[13].adjVoxels.push_back(16);
//         adjancenyTable[13].adjVoxels.push_back(17);
//         adjancenyTable[13].adjVoxels.push_back(18);
//         adjancenyTable[13].adjVoxels.push_back(19);
//         adjancenyTable[13].adjVoxels.push_back(20);
//         adjancenyTable[13].adjVoxels.push_back(21);
//         adjancenyTable[13].adjVoxels.push_back(22);
//         adjancenyTable[13].adjVoxels.push_back(23);
//         adjancenyTable[13].adjVoxels.push_back(24);
//         adjancenyTable[13].adjVoxels.push_back(25);
//         adjancenyTable[13].adjVoxels.push_back(26);
// 
//         adjancenyTable[14].adjVoxels.push_back(1);
//         adjancenyTable[14].adjVoxels.push_back(2);
//         adjancenyTable[14].adjVoxels.push_back(4);
//         adjancenyTable[14].adjVoxels.push_back(5);
//         adjancenyTable[14].adjVoxels.push_back(7);
//         adjancenyTable[14].adjVoxels.push_back(8);
//         adjancenyTable[14].adjVoxels.push_back(10);
//         adjancenyTable[14].adjVoxels.push_back(11);
//         adjancenyTable[14].adjVoxels.push_back(13);
//         adjancenyTable[14].adjVoxels.push_back(16);
//         adjancenyTable[14].adjVoxels.push_back(17);
//         adjancenyTable[14].adjVoxels.push_back(19);
//         adjancenyTable[14].adjVoxels.push_back(20);
//         adjancenyTable[14].adjVoxels.push_back(22);
//         adjancenyTable[14].adjVoxels.push_back(23);
//         adjancenyTable[14].adjVoxels.push_back(25);
//         adjancenyTable[14].adjVoxels.push_back(26);
// 
//         adjancenyTable[15].adjVoxels.push_back(3);
//         adjancenyTable[15].adjVoxels.push_back(4);
//         adjancenyTable[15].adjVoxels.push_back(6);
//         adjancenyTable[15].adjVoxels.push_back(7);
//         adjancenyTable[15].adjVoxels.push_back(12);
//         adjancenyTable[15].adjVoxels.push_back(13);
//         adjancenyTable[15].adjVoxels.push_back(16);
//         adjancenyTable[15].adjVoxels.push_back(21);
//         adjancenyTable[15].adjVoxels.push_back(22);
//         adjancenyTable[15].adjVoxels.push_back(24);
//         adjancenyTable[15].adjVoxels.push_back(25);
// 
//         adjancenyTable[16].adjVoxels.push_back(3);
//         adjancenyTable[16].adjVoxels.push_back(4);
//         adjancenyTable[16].adjVoxels.push_back(5);
//         adjancenyTable[16].adjVoxels.push_back(6);
//         adjancenyTable[16].adjVoxels.push_back(7);
//         adjancenyTable[16].adjVoxels.push_back(8);
//         adjancenyTable[16].adjVoxels.push_back(12);
//         adjancenyTable[16].adjVoxels.push_back(13);
//         adjancenyTable[16].adjVoxels.push_back(14);
//         adjancenyTable[16].adjVoxels.push_back(15);
//         adjancenyTable[16].adjVoxels.push_back(17);
//         adjancenyTable[16].adjVoxels.push_back(21);
//         adjancenyTable[16].adjVoxels.push_back(22);
//         adjancenyTable[16].adjVoxels.push_back(23);
//         adjancenyTable[16].adjVoxels.push_back(24);
//         adjancenyTable[16].adjVoxels.push_back(25);
//         adjancenyTable[16].adjVoxels.push_back(26);
// 
//         adjancenyTable[17].adjVoxels.push_back(4);
//         adjancenyTable[17].adjVoxels.push_back(5);
//         adjancenyTable[17].adjVoxels.push_back(7);
//         adjancenyTable[17].adjVoxels.push_back(8);
//         adjancenyTable[17].adjVoxels.push_back(13);
//         adjancenyTable[17].adjVoxels.push_back(14);
//         adjancenyTable[17].adjVoxels.push_back(16);
//         adjancenyTable[17].adjVoxels.push_back(22);
//         adjancenyTable[17].adjVoxels.push_back(23);
//         adjancenyTable[17].adjVoxels.push_back(25);
//         adjancenyTable[17].adjVoxels.push_back(26);
// 
//         adjancenyTable[18].adjVoxels.push_back(9);
//         adjancenyTable[18].adjVoxels.push_back(10);
//         adjancenyTable[18].adjVoxels.push_back(12);
//         adjancenyTable[18].adjVoxels.push_back(13);
//         adjancenyTable[18].adjVoxels.push_back(19);
//         adjancenyTable[18].adjVoxels.push_back(21);
//         adjancenyTable[18].adjVoxels.push_back(22);
// 
//         adjancenyTable[19].adjVoxels.push_back(9);
//         adjancenyTable[19].adjVoxels.push_back(10);
//         adjancenyTable[19].adjVoxels.push_back(11);
//         adjancenyTable[19].adjVoxels.push_back(12);
//         adjancenyTable[19].adjVoxels.push_back(13);
//         adjancenyTable[19].adjVoxels.push_back(14);
//         adjancenyTable[19].adjVoxels.push_back(18);
//         adjancenyTable[19].adjVoxels.push_back(20);
//         adjancenyTable[19].adjVoxels.push_back(21);
//         adjancenyTable[19].adjVoxels.push_back(22);
//         adjancenyTable[19].adjVoxels.push_back(23);
// 
//         adjancenyTable[20].adjVoxels.push_back(10);
//         adjancenyTable[20].adjVoxels.push_back(11);
//         adjancenyTable[20].adjVoxels.push_back(13);
//         adjancenyTable[20].adjVoxels.push_back(14);
//         adjancenyTable[20].adjVoxels.push_back(19);
//         adjancenyTable[20].adjVoxels.push_back(22);
//         adjancenyTable[20].adjVoxels.push_back(23);
// 
//         adjancenyTable[21].adjVoxels.push_back(9);
//         adjancenyTable[21].adjVoxels.push_back(10);
//         adjancenyTable[21].adjVoxels.push_back(12);
//         adjancenyTable[21].adjVoxels.push_back(13);
//         adjancenyTable[21].adjVoxels.push_back(15);
//         adjancenyTable[21].adjVoxels.push_back(16);
//         adjancenyTable[21].adjVoxels.push_back(18);
//         adjancenyTable[21].adjVoxels.push_back(19);
//         adjancenyTable[21].adjVoxels.push_back(22);
//         adjancenyTable[21].adjVoxels.push_back(24);
//         adjancenyTable[21].adjVoxels.push_back(25);
// 
//         adjancenyTable[22].adjVoxels.push_back(9);
//         adjancenyTable[22].adjVoxels.push_back(10);
//         adjancenyTable[22].adjVoxels.push_back(11);
//         adjancenyTable[22].adjVoxels.push_back(12);
//         adjancenyTable[22].adjVoxels.push_back(13);
//         adjancenyTable[22].adjVoxels.push_back(14);
//         adjancenyTable[22].adjVoxels.push_back(15);
//         adjancenyTable[22].adjVoxels.push_back(16);
//         adjancenyTable[22].adjVoxels.push_back(17);
//         adjancenyTable[22].adjVoxels.push_back(18);
//         adjancenyTable[22].adjVoxels.push_back(19);
//         adjancenyTable[22].adjVoxels.push_back(20);
//         adjancenyTable[22].adjVoxels.push_back(21);
//         adjancenyTable[22].adjVoxels.push_back(23);
//         adjancenyTable[22].adjVoxels.push_back(24);
//         adjancenyTable[22].adjVoxels.push_back(25);
//         adjancenyTable[22].adjVoxels.push_back(26);
// 
//         adjancenyTable[23].adjVoxels.push_back(10);
//         adjancenyTable[23].adjVoxels.push_back(11);
//         adjancenyTable[23].adjVoxels.push_back(13);
//         adjancenyTable[23].adjVoxels.push_back(14);
//         adjancenyTable[23].adjVoxels.push_back(16);
//         adjancenyTable[23].adjVoxels.push_back(17);
//         adjancenyTable[23].adjVoxels.push_back(19);
//         adjancenyTable[23].adjVoxels.push_back(20);
//         adjancenyTable[23].adjVoxels.push_back(22);
//         adjancenyTable[23].adjVoxels.push_back(25);
//         adjancenyTable[23].adjVoxels.push_back(26);
// 
//         adjancenyTable[24].adjVoxels.push_back(12);
//         adjancenyTable[24].adjVoxels.push_back(13);
//         adjancenyTable[24].adjVoxels.push_back(15);
//         adjancenyTable[24].adjVoxels.push_back(16);
//         adjancenyTable[24].adjVoxels.push_back(21);
//         adjancenyTable[24].adjVoxels.push_back(22);
//         adjancenyTable[24].adjVoxels.push_back(25);
// 
//         adjancenyTable[25].adjVoxels.push_back(12);
//         adjancenyTable[25].adjVoxels.push_back(13);
//         adjancenyTable[25].adjVoxels.push_back(14);
//         adjancenyTable[25].adjVoxels.push_back(15);
//         adjancenyTable[25].adjVoxels.push_back(16);
//         adjancenyTable[25].adjVoxels.push_back(17);
//         adjancenyTable[25].adjVoxels.push_back(21);
//         adjancenyTable[25].adjVoxels.push_back(22);
//         adjancenyTable[25].adjVoxels.push_back(23);
//         adjancenyTable[25].adjVoxels.push_back(24);
//         adjancenyTable[25].adjVoxels.push_back(26);
// 
//         adjancenyTable[26].adjVoxels.push_back(13);
//         adjancenyTable[26].adjVoxels.push_back(14);
//         adjancenyTable[26].adjVoxels.push_back(16);
//         adjancenyTable[26].adjVoxels.push_back(17);
//         adjancenyTable[26].adjVoxels.push_back(22);
//         adjancenyTable[26].adjVoxels.push_back(23);
//         adjancenyTable[26].adjVoxels.push_back(25);
//     }
//     else if ( con == N6 ) {
//         adjancenyTable[0].adjVoxels.push_back(1);
//         adjancenyTable[0].adjVoxels.push_back(3);
//         adjancenyTable[0].adjVoxels.push_back(9);
// 
//         adjancenyTable[1].adjVoxels.push_back(0);
//         adjancenyTable[1].adjVoxels.push_back(2);
//         adjancenyTable[1].adjVoxels.push_back(4);
//         adjancenyTable[1].adjVoxels.push_back(10);
// 
//         adjancenyTable[2].adjVoxels.push_back(1);
//         adjancenyTable[2].adjVoxels.push_back(5);
//         adjancenyTable[2].adjVoxels.push_back(11);
// 
//         adjancenyTable[3].adjVoxels.push_back(0);
//         adjancenyTable[3].adjVoxels.push_back(4);
//         adjancenyTable[3].adjVoxels.push_back(6);
//         adjancenyTable[3].adjVoxels.push_back(12);
// 
//         adjancenyTable[4].adjVoxels.push_back(1);
//         adjancenyTable[4].adjVoxels.push_back(3);
//         adjancenyTable[4].adjVoxels.push_back(5);
//         adjancenyTable[4].adjVoxels.push_back(7);
//         adjancenyTable[4].adjVoxels.push_back(13);
// 
//         adjancenyTable[5].adjVoxels.push_back(2);
//         adjancenyTable[5].adjVoxels.push_back(4);
//         adjancenyTable[5].adjVoxels.push_back(8);
//         adjancenyTable[5].adjVoxels.push_back(14);
// 
//         adjancenyTable[6].adjVoxels.push_back(3);
//         adjancenyTable[6].adjVoxels.push_back(7);
//         adjancenyTable[6].adjVoxels.push_back(15);
// 
//         adjancenyTable[7].adjVoxels.push_back(4);
//         adjancenyTable[7].adjVoxels.push_back(6);
//         adjancenyTable[7].adjVoxels.push_back(8);
//         adjancenyTable[7].adjVoxels.push_back(16);
// 
//         adjancenyTable[8].adjVoxels.push_back(5);
//         adjancenyTable[8].adjVoxels.push_back(7);
//         adjancenyTable[8].adjVoxels.push_back(17);
// 
//         adjancenyTable[9].adjVoxels.push_back(0);
//         adjancenyTable[9].adjVoxels.push_back(10);
//         adjancenyTable[9].adjVoxels.push_back(12);
//         adjancenyTable[9].adjVoxels.push_back(18);
// 
//         adjancenyTable[10].adjVoxels.push_back(1);
//         adjancenyTable[10].adjVoxels.push_back(9);
//         adjancenyTable[10].adjVoxels.push_back(11);
//         adjancenyTable[10].adjVoxels.push_back(13);
//         adjancenyTable[10].adjVoxels.push_back(19);
// 
//         adjancenyTable[11].adjVoxels.push_back(2);
//         adjancenyTable[11].adjVoxels.push_back(10);
//         adjancenyTable[11].adjVoxels.push_back(14);
//         adjancenyTable[11].adjVoxels.push_back(20);
// 
//         adjancenyTable[12].adjVoxels.push_back(3);
//         adjancenyTable[12].adjVoxels.push_back(9);
//         adjancenyTable[12].adjVoxels.push_back(13);
//         adjancenyTable[12].adjVoxels.push_back(15);
//         adjancenyTable[12].adjVoxels.push_back(21);
// 
//         adjancenyTable[13].adjVoxels.push_back(4);
//         adjancenyTable[13].adjVoxels.push_back(10);
//         adjancenyTable[13].adjVoxels.push_back(12);
//         adjancenyTable[13].adjVoxels.push_back(14);
//         adjancenyTable[13].adjVoxels.push_back(16);
//         adjancenyTable[13].adjVoxels.push_back(22);
// 
//         adjancenyTable[14].adjVoxels.push_back(5);
//         adjancenyTable[14].adjVoxels.push_back(11);
//         adjancenyTable[14].adjVoxels.push_back(13);
//         adjancenyTable[14].adjVoxels.push_back(17);
//         adjancenyTable[14].adjVoxels.push_back(23);
// 
//         adjancenyTable[15].adjVoxels.push_back(6);
//         adjancenyTable[15].adjVoxels.push_back(12);
//         adjancenyTable[15].adjVoxels.push_back(16);
//         adjancenyTable[15].adjVoxels.push_back(24);
// 
//         adjancenyTable[16].adjVoxels.push_back(7);
//         adjancenyTable[16].adjVoxels.push_back(13);
//         adjancenyTable[16].adjVoxels.push_back(15);
//         adjancenyTable[16].adjVoxels.push_back(17);
//         adjancenyTable[16].adjVoxels.push_back(25);
// 
//         adjancenyTable[17].adjVoxels.push_back(8);
//         adjancenyTable[17].adjVoxels.push_back(14);
//         adjancenyTable[17].adjVoxels.push_back(16);
//         adjancenyTable[17].adjVoxels.push_back(26);
// 
//         adjancenyTable[18].adjVoxels.push_back(9);
//         adjancenyTable[18].adjVoxels.push_back(19);
//         adjancenyTable[18].adjVoxels.push_back(21);
// 
//         adjancenyTable[19].adjVoxels.push_back(10);
//         adjancenyTable[19].adjVoxels.push_back(18);
//         adjancenyTable[19].adjVoxels.push_back(20);
//         adjancenyTable[19].adjVoxels.push_back(22);
// 
//         adjancenyTable[20].adjVoxels.push_back(11);
//         adjancenyTable[20].adjVoxels.push_back(19);
//         adjancenyTable[20].adjVoxels.push_back(23);
// 
//         adjancenyTable[21].adjVoxels.push_back(12);
//         adjancenyTable[21].adjVoxels.push_back(18);
//         adjancenyTable[21].adjVoxels.push_back(22);
//         adjancenyTable[21].adjVoxels.push_back(24);
// 
//         adjancenyTable[22].adjVoxels.push_back(13);
//         adjancenyTable[22].adjVoxels.push_back(19);
//         adjancenyTable[22].adjVoxels.push_back(21);
//         adjancenyTable[22].adjVoxels.push_back(23);
//         adjancenyTable[22].adjVoxels.push_back(25);
// 
//         adjancenyTable[23].adjVoxels.push_back(14);
//         adjancenyTable[23].adjVoxels.push_back(20);
//         adjancenyTable[23].adjVoxels.push_back(22);
//         adjancenyTable[23].adjVoxels.push_back(26);
// 
//         adjancenyTable[24].adjVoxels.push_back(15);
//         adjancenyTable[24].adjVoxels.push_back(21);
//         adjancenyTable[24].adjVoxels.push_back(25);
// 
//         adjancenyTable[25].adjVoxels.push_back(16);
//         adjancenyTable[25].adjVoxels.push_back(22);
//         adjancenyTable[25].adjVoxels.push_back(24);
//         adjancenyTable[25].adjVoxels.push_back(26);
// 
//         adjancenyTable[26].adjVoxels.push_back(17);
//         adjancenyTable[26].adjVoxels.push_back(23);
//         adjancenyTable[26].adjVoxels.push_back(25);
//     }
//     else return outArray;

    if ( con == N26 ) {
        if ( this->adjancenyTableN26.size() == 0 ) this->createAdjacencyTables();
        adjancenyTable = this->adjancenyTableN26;
    }
    else if ( con == N6 ) {
        if ( this->adjancenyTableN6.size() == 0 ) this->createAdjacencyTables();
        adjancenyTable = this->adjancenyTableN6;
    }
    else return outArray;


    for ( i = 0; i < inArray.size(); i++ ) {
        if ( (inArray.at(i) == true) && (outArray.at(i) == 0) ) {
            outArray[i] = ++label;
            vArray[i] = 1;

            do {
                found = false;
                for ( j = 0; j < vArray.size(); j++ ) {
                    if ( vArray.at(j) == 1 ) {

                        for ( k = 0; k < adjancenyTable[j].adjVoxels.size(); k++ ) {
                            if ( (inArray.at(adjancenyTable[j].adjVoxels.at(k)) == true) &&
                                 (outArray.at(adjancenyTable[j].adjVoxels.at(k)) == 0 ) ) {
                                outArray[adjancenyTable[j].adjVoxels.at(k)] = label;
                                vArray[adjancenyTable[j].adjVoxels.at(k)] = 1;
                                found = true;
                            }
                        }

                        vArray[j] = 2;
                    }
                }
            } while (found);

        }
    }

    return outArray;
}


// check if the point is simple
bool CHierarchicalRLE::isSimplePoint( vector<int> coord ) {

    int                 i;
    vector<bool>        neighbors;

    neighbors = this->neighbor_N26( coord );

    neighbors[13] = false;

    if ( this->countConnectedComponentsN3D_N26( neighbors, N26 ) != 1 ) return false;

    neighbors[13] = true;

    for ( i = 0; i < neighbors.size(); i++ ) {
        neighbors.at(i) = !neighbors.at(i);
    }

    if ( this->countConnectedComponentsN3D_N26( neighbors, N6 ) != 1 ) return false;

    return true;
}

// check if the point is a border point of the given direction
bool CHierarchicalRLE::isBorderPointOfDirection( vector<int> coord, borderPointType type ) {

    int i;

    vector<bool> neighbors = this->neighbor_N6( coord );

    if ( type == BP_BOTTOM ) {
        if (neighbors.at(0) == false) return true;       // z-1
        else return false;
    }
    if ( type == BP_TOP ) {
        if (neighbors.at(1) == false) return true;       // z+1
        else return false;
    }
    if ( type == BP_WEST ) {
        if (neighbors.at(2) == false) return true;      // y-1
        else return false;
    }
    if ( type == BP_EAST ) {
        if (neighbors.at(3) == false) return true;      // y+1
        else return false;
    }
    if ( type == BP_NORTH ) {
        if (neighbors.at(4) == false) return true;      // x-1
        else return false;
    }
    if ( type == BP_SOUTH ) {
        if (neighbors.at(5) == false) return true;      // x+1
        else return false;
    }

    return false;

}

// check if the point is a border point
bool CHierarchicalRLE::isBorderPoint( vector<int> coord ) {

    int i;

    vector<bool> neighbors = this->neighbor_N6(coord);

    for ( i = 0; i < neighbors.size(); i++ ) {
        if ( neighbors.at(i) == false ) return true;
    }

    return false;

}

// check if the given point is a final point
bool CHierarchicalRLE::isFinalPoint( vector<int> bp ) {

    int numBlackPoints = 0;

    vector<bool> neighbors = neighbor_N26( bp );

    neighbors[13] = false;

    for ( int i = 0; i < neighbors.size(); i++ ) {
        if ( neighbors.at(i) ) numBlackPoints++;
    }

    if ( numBlackPoints < 2 ) return true;

    return false;
}


// get all border points
vector<definedPoint> CHierarchicalRLE::getBorderPoints()
{
    vector<int> parentVector;
    return this->getBorderPoints( parentVector, this );
}


vector<definedPoint> CHierarchicalRLE::getBorderPoints( vector<int> parentCoord, CHierarchicalRLE* topHRLE, int segmentIndex )
{
    vector<definedPoint>  returnValue;
    definedPoint          bp;

    int             segmentLength;
    int             firstRun = this->start_indices.at(segmentIndex);
    int             run;
    short int       startCoordinate;                            // the first coordinate of the determined run
    short int       endCoordinate;                              // and the end coordinate of the run

    // get segment length - number of runs in this segment
    if (this->start_indices.at(segmentIndex) == this->start_indices.at(this->start_indices.size()-1))
        segmentLength = this->run_types.size() - this->start_indices.at(segmentIndex);
    else
        segmentLength = this->start_indices.at(segmentIndex+1) - this->start_indices.at(segmentIndex);

    // traverse the runs of this segment
    for (int i = firstRun; i < ( firstRun + segmentLength ); i++) {
        try {
            run = this->run_types.at(i);
        } catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
            cerr << "Out of range exception " << e.what() << endl;
            cerr << "get border points: no run with index " << i << endl;
            exit(1);
        }

        // only defined runs are considered
        if ( RT_DEFINED( run ) ) {

            // get the first coordinate of the run
            if ( i == firstRun ) {
                startCoordinate = this->min;
            }
            else {
                try {
                    startCoordinate = this->run_breaks.at( i - segmentIndex -1 );
                } catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
                    cerr << "Out of range exception " << e.what() << endl;
                    cerr << "get border points: cannot setup start coordinate for run " << endl;
                    exit(1);
                }
            }

            // last coordinate of the run
            if (i == ( firstRun + segmentLength -1 )) {
                endCoordinate = this->max;
            }
            else {
                try {
                    endCoordinate = this->run_breaks.at( i - segmentIndex );
                } catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
                    cerr << "Out of range exception " << e.what() << endl;
                    cerr << "get border points: cannot setup end coordinate for run " << endl;
                    exit(1);
                }
            }

            // iterate trough all coordinates of the run
            for (int j = startCoordinate; j < endCoordinate; j++) {
                int defDataIndex = run + j - startCoordinate;    // compute the defined data index

                // concatenate the parent coordinates with current coordinate
                vector<int> currentCoordinates;

                // if this is the lowest dimension, check if the point is a border point
                if (this->lower_hrle == NULL) {

                    // coordinate order: lowest coordinate on first position
                    currentCoordinates.push_back( j );
                    for ( int k = (parentCoord.size()-1); k >= 0; k-- ) {
                        currentCoordinates.push_back( parentCoord.at(k) );
                    }

                      if ( topHRLE->isBorderPoint(currentCoordinates) ) {
                          definedPoint dp;
                          dp.coordinate = currentCoordinates;
                          dp.isDeleted = false;
                          returnValue.push_back(dp);
                      }

                }
                // recursive call to the next lower H_RLE block
                else {
                    // coordinate order: lowest coordinate on last position
                    currentCoordinates = parentCoord;
                    currentCoordinates.push_back( j );

                    vector<definedPoint> tmp;
                    tmp = this->lower_hrle->getBorderPoints( currentCoordinates,
                                                              topHRLE, defDataIndex );
                    for (int k = 0; k < tmp.size(); k++)
                        returnValue.push_back(tmp.at(k));
                }

            }

        }
    }

    return returnValue;
}

// returns all defined points of the object
vector<definedPoint> CHierarchicalRLE::getDefinedPoints()
{
    vector<int> parentVector;
    return this->getDefinedPoints( parentVector );
}

// returns all defined points of the object
vector<definedPoint> CHierarchicalRLE::getDefinedPoints( vector<int> parentCoord, int segmentIndex )
{
    vector<definedPoint>  returnValue;
    definedPoint           bp;

    int             segmentLength;
    int             firstRun = this->start_indices.at(segmentIndex);
    int             run;
    short int       startCoordinate;                            // the first coordinate of the determined run
    short int       endCoordinate;                              // and the end coordinate of the run

    // get segment length - number of runs in this segment
    if (this->start_indices.at(segmentIndex) == this->start_indices.at(this->start_indices.size()-1))
        segmentLength = this->run_types.size() - this->start_indices.at(segmentIndex);
    else
        segmentLength = this->start_indices.at(segmentIndex+1) - this->start_indices.at(segmentIndex);

    // traverse the runs of this segment
    for (int i = firstRun; i < ( firstRun + segmentLength ); i++) {
        try {
            run = this->run_types.at(i);
        } catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
            cerr << "Out of range exception " << e.what() << endl;
            cerr << "get border points: no run with index " << i << endl;
            exit(1);
        }

        // only defined runs are considered
        if ( RT_DEFINED( run ) ) {

            // get the first coordinate of the run
            if ( i == firstRun ) {
                startCoordinate = this->min;
            }
            else {
                try {
                    startCoordinate = this->run_breaks.at( i - segmentIndex -1 );
                } catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
                    cerr << "Out of range exception " << e.what() << endl;
                    cerr << "get border points: cannot setup start coordinate for run " << endl;
                    exit(1);
                }
            }

            // last coordinate of the run
            if (i == ( firstRun + segmentLength -1 )) {
                endCoordinate = this->max;
            }
            else {
                try {
                    endCoordinate = this->run_breaks.at( i - segmentIndex );
                } catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
                    cerr << "Out of range exception " << e.what() << endl;
                    cerr << "get border points: cannot setup end coordinate for run " << endl;
                    exit(1);
                }
            }

            // iterate trough all coordinates of the run
            for (int j = startCoordinate; j < endCoordinate; j++) {
                int defDataIndex = run + j - startCoordinate;    // compute the defined data index

                // concatenate the parent coordinates with current coordinate
                vector<int> currentCoordinates;

                // if this is the lowest dimension, check if the point is a border point
                if (this->lower_hrle == NULL) {

                    // coordinate order: lowest coordinate on first position
                    currentCoordinates.push_back( j );
                    for ( int k = (parentCoord.size()-1); k >= 0; k-- ) {
                        currentCoordinates.push_back( parentCoord.at(k) );
                    }

                      definedPoint dp;
                      dp.coordinate = currentCoordinates;
                      dp.isDeleted = false;
                      returnValue.push_back(dp);

                }
                // recursive call to the next lower H_RLE block
                else {
                    // coordinate order: lowest coordinate on last position
                    currentCoordinates = parentCoord;
                    currentCoordinates.push_back( j );

                    vector<definedPoint> tmp;
                    tmp = this->lower_hrle->getDefinedPoints( currentCoordinates, defDataIndex );
                    for (int k = 0; k < tmp.size(); k++)
                        returnValue.push_back(tmp.at(k));
                }

            }

        }
    }

    return returnValue;
}

// wrapper for deleteBorderPoints
bool CHierarchicalRLE::deleteBorderPoints_bool( vector<coordValue> deleteablePoints ) {

    CHierarchicalRLE* tmpHRLE = this->deleteBorderPoints( deleteablePoints );

    if ( tmpHRLE == NULL ) return false;

    try {
        this->copy( tmpHRLE );
    } catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
        cerr << "Out of range exception " << e.what() << endl;
        cerr << "Delete border points (bool): Copy failed!" << endl;
        exit(1);
    }

    if ( (tmpHRLE->run_types.size() == 1) && !RT_DEFINED(tmpHRLE->run_types.back()) ) return false;

    return true;
}

// deletes a series of defined points
// the points must be ordered via their coordinates!!
CHierarchicalRLE* CHierarchicalRLE::deleteBorderPoints( vector<coordValue> deleteablePoints ) {

    int           run, iRun;
    short int     startCoord, endCoord;
    short int     coord, dpPos = 0;
    int           definedIndex;

    CHierarchicalRLE*   tmpHRLE = NULL;                 // the H-RLE structure without the deleteable points
    CHierarchicalRLE*   tmpSubHRLE = NULL;              // the lower H-RLE segment of a defined coordinate

    if ( deleteablePoints.size() == 0 ) {
        cerr << "No points to delete!" << endl;
        return NULL;
    }

    // start and end coordinate of the first run
    startCoord = this->min;
    try {
        if ( this->run_breaks.size() == 0 ) endCoord = this->max;
        else endCoord = this->run_breaks.at(0);
    } catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
        cerr << "Out of range exception " << e.what() << endl;
        cerr << "Delete border points: Error while setting up endcoord." << endl;
        exit(1);
    }

    // initialize the temporal data structure
    tmpHRLE = new CHierarchicalRLE( this->dimension );

    // initialize tmpHRLE
    tmpHRLE->min = this->min;
    tmpHRLE->max = this->max;
    tmpHRLE->start_indices.push_back( 0 );

    // iterate through all runs
    for ( iRun = 0; iRun < this->run_types.size(); iRun++ ) {

        // get the run
        try {
            run = this->run_types.at( iRun );
        } catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
            cerr << "Out of range exception " << e.what() << endl;
            cerr << "Delete border points: Error! Run with index " << iRun << " does not exist!" << endl;
            exit(1);
        }

        // manage defined runs
        if ( RT_DEFINED( run ) ) {
            for ( coord = startCoord; coord < endCoord; coord++ ) {

                // get the sub segment if this is not the lowest dimension
                if ( this->lower_hrle != NULL ) {
                    definedIndex = run + coord - startCoord;
                    try {
                        tmpSubHRLE = this->lower_hrle->getSegmentAt( definedIndex );
                    } catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
                        cerr << "Out of range exception " << e.what() << endl;
                        cerr << "Delete border points: Error! Can't get subsegment with index " << definedIndex << endl;
                        exit(1);
                    }
                }

                // delete the points
                if ( (dpPos < deleteablePoints.size()) && (deleteablePoints.at(dpPos).coord == coord) ) {
                    // lowest dimension
                    if ( this->lower_hrle == NULL) {
                        // first run in the segment
                        if (tmpHRLE->run_types.size() == 0) {
                            tmpHRLE->run_types.push_back( RT_POSITIVE );
                        }
                        // last run was defined
                        else if ( RT_DEFINED(tmpHRLE->run_types.back()) )
                            tmpHRLE->run_types.push_back( RT_NEGATIVE );
                        // last run was not defined
                        else {
                            if ( tmpHRLE->run_breaks.size() > 0 ) tmpHRLE->run_breaks.pop_back();
                        }

//                         if ( (coord+1) < tmpHRLE->max ) tmpHRLE->run_breaks.push_back( coord + 1 );
                    }
                    // higher dimension
                    else {
                        // the lower RLE segment is not completely deleted
                        bool blubb;
                        try {
                        blubb = tmpSubHRLE->deleteBorderPoints_bool( deleteablePoints[dpPos].subCoords );
                        } catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
                            cerr << "Out of range exception " << e.what() << endl;
                            cerr << "Delete border points: delete points in subHRLE" << endl;
                            exit(1);
                        }
                        if ( blubb == true ) {

                            if ( (tmpHRLE->run_types.size() == 0) ||
                                !RT_DEFINED( tmpHRLE->run_types.back() ) ) {
                                try {
                                    tmpHRLE->run_types.push_back( tmpHRLE->lower_hrle->start_indices.size() );
                                } catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
                                    cerr << "Out of range exception " << e.what() << endl;
                                    cerr << "Delete border points: Error setting up defined index for higher dimensions." << endl;
                                    exit(1);
                                }
                            }
                            else {
                                if ( tmpHRLE->run_breaks.size() > 0 ) tmpHRLE->run_breaks.pop_back();
                            }

                            // add the sub segment to the return value
                            try {
                                tmpHRLE->lower_hrle->addToHRLE( tmpSubHRLE );
                            } catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
                                cerr << "Out of range exception " << e.what() << endl;
                                cerr << "Delete border points: Error! Can't add new subHRLE to structure." << endl;
                                exit(1);
                            }

                        }
                        // lower RLE segment completely deleted, so this coordinate is no longer defined
                        else {
                            // first run in the segment
                            if (tmpHRLE->run_types.size() == 0) {
                                tmpHRLE->run_types.push_back( RT_POSITIVE );
                            }
                            // last run was defined
                            else if ( RT_DEFINED(tmpHRLE->run_types.back()) )
                                tmpHRLE->run_types.push_back( RT_NEGATIVE );
                            // last run was not defined
                            else {
                                if ( tmpHRLE->run_breaks.size() > 0 ) tmpHRLE->run_breaks.pop_back();
                            }
                        }
                    }

                    if ( (coord+1) < tmpHRLE->max ) tmpHRLE->run_breaks.push_back( coord + 1 );

                    dpPos++;
                }
                // don't delete the point
                else {
                    if ( (tmpHRLE->run_types.size() == 0) ||
                        !RT_DEFINED( tmpHRLE->run_types.back() ) ) {
                        if ( this->lower_hrle != NULL )
                            tmpHRLE->run_types.push_back( tmpHRLE->lower_hrle->start_indices.size() );
                        else
                            tmpHRLE->run_types.push_back( 1 );
                    }
                    else {
                        if ( tmpHRLE->run_breaks.size() > 0 ) tmpHRLE->run_breaks.pop_back();
                    }
                    if ( (coord+1) < tmpHRLE->max ) tmpHRLE->run_breaks.push_back( coord + 1 );

                    // add the sub segment to the return value
                    try {
                        if ( this->lower_hrle != NULL ) tmpHRLE->lower_hrle->addToHRLE( tmpSubHRLE );
                    } catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
                        cerr << "Out of range exception " << e.what() << endl;
                        cerr << "Delete border points: Error while adding unchanged lower defined segment." << endl;
                        exit(1);
                    }
                }

            }
        }
        // manage non defined runs
        else {
            // add this run

            if ( (tmpHRLE->run_types.size() == 0) ||
                 RT_DEFINED( tmpHRLE->run_types.back() ) ) {
                tmpHRLE->run_types.push_back( run );
            }
            else {
                if ((tmpHRLE->run_types.back() == RT_NEGATIVE) && ( run == RT_POSITIVE )) {
                    tmpHRLE->run_types.pop_back();
                    tmpHRLE->run_types.push_back( RT_POSITIVE );
                }
                if ( tmpHRLE->run_breaks.size() > 0 ) tmpHRLE->run_breaks.pop_back();
            }

            if ( endCoord < tmpHRLE->max ) tmpHRLE->run_breaks.push_back( endCoord );
        }

        startCoord = endCoord;
        if ( (iRun+1) >= this->run_breaks.size() ) endCoord = this->max;
        else endCoord = this->run_breaks.at(iRun+1);

    }

    if ( (tmpHRLE->run_types.size() > 0) && (tmpHRLE->run_types.back() == RT_NEGATIVE) ) {
        tmpHRLE->run_types.pop_back();
        tmpHRLE->run_types.push_back( RT_POSITIVE );
    }

    return tmpHRLE;

}

// add points to the deletable points array - points must be sorted!
bool CHierarchicalRLE::addToDeleteablePoints( vector<int> qv, vector<coordValue> *deleteablePoints ) {

    int qi = qv.back();
    qv.pop_back();

    if ( (deleteablePoints->empty()) || (deleteablePoints->back().coord < qi) ) {
        coordValue newCoord;

        newCoord.coord = qi;

        deleteablePoints->push_back( newCoord );
    }

    if ( qv.size() > 0 )
        this->addToDeleteablePoints( qv, &(deleteablePoints->back().subCoords) );

    return true;
}

// the checking plane condition for the thinning algorithms
bool CHierarchicalRLE::checkingPlaneCondition( vector<int> bp, int numRemainingVoxels, borderPointType type ) {

    short int     numBlackPoints = 0;
    short int     x,y,z,i;
    vector<bool>  n26;
    vector<bool>  firstPlane, secondPlane;

    // get the n26 neighborhood of the point
    try {
        n26 = this->neighbor_N26( bp );
    }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
        cerr << "Out of range exception " << e.what() << endl;
        cerr << "Checking plane: error getting N26 neighborhood!" << endl;
        exit(1);
    }

    if ( n26.size() < 26 ) cerr << "Checking plane: Error! Not enough neighbors in N26!" << endl;

    if ( n26.size() == 0 ) return false;

    // direction is the z axis: get planes xz and yz
    if ( (type == BP_TOP) || (type == BP_BOTTOM) ) {
        for ( z = 0; z < 3; z++ ) {
            for ( y = 0; y < 3; y++ ) {
                for ( x = 0; x < 3; x++ ) {
                    // plane xz
                    try {
                        if ( y == 1 ) firstPlane.push_back( n26.at(x + 3 + 9*z) );
                    }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
                        cerr << "Out of range exception " << e.what() << endl;
                        cerr << "Checking plane: error! Neighbor does not exist (Plane 1, Top/Bottom)" << endl;
                        exit(1);
                    }
                    // plane yz
                    try {
                        if ( x == 1 ) secondPlane.push_back( n26.at(1 + 3*y + 9*z) );
                    }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
                        cerr << "Out of range exception " << e.what() << endl;
                        cerr << "Checking plane: error! Neighbor does not exist (Plane 2, Top/Bottom)" << endl;
                        exit(1);
                    }
                }
            }
        }
    }

    // direction is the y axis: get planes xy and yz
    if ( (type == BP_EAST) || (type == BP_WEST) ) {
        for ( z = 0; z < 3; z++ ) {
            for ( y = 0; y < 3; y++ ) {
                for ( x = 0; x < 3; x++ ) {
                    // plane xy
                    try {
                        if ( z == 1 ) firstPlane.push_back( n26.at(x + 3*y + 9) );
                    }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
                        cerr << "Out of range exception " << e.what() << endl;
                        cerr << "Checking plane: error! Neighbor does not exist (Plane 1, North/South)" << endl;
                        exit(1);
                    }
                    // plane yz
                    try {
                        if ( x == 1 ) secondPlane.push_back( n26.at(1 + 3*y + 9*z) );
                    }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
                        cerr << "Out of range exception " << e.what() << endl;
                        cerr << "Checking plane: error! Neighbor does not exist (Plane 2, North/South)" << endl;
                        exit(1);
                    }
                }
            }
        }
    }

    // direction is the x axis: get planes xy and xz
    if ( (type == BP_NORTH) || (type == BP_SOUTH) ) {
        for ( z = 0; z < 3; z++ ) {
            for ( y = 0; y < 3; y++ ) {
                for ( x = 0; x < 3; x++ ) {
                    // plane xy
                    try {
                        if ( z == 1 ) firstPlane.push_back( n26.at(x + 3*y + 9) );
                    }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
                        cerr << "Out of range exception " << e.what() << endl;
                        cerr << "Checking plane: error! Neighbor does not exist (Plane 1, East/West)" << endl;
                        exit(1);
                    }
                    // plane xz
                    try {
                        if ( y == 1 ) secondPlane.push_back( n26.at(x + 3 + 9*z) );
                    }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
                        cerr << "Out of range exception " << e.what() << endl;
                        cerr << "Checking plane: error! Neighbor does not exist (Plane 2, East/West)" << endl;
                        exit(1);
                    }
                }
            }
        }
    }

    // set middle point to false
    firstPlane[4] = false;
    secondPlane[4] = false;

    // check if enough black voxels would remain after the deletion of the point
    numBlackPoints = 0;
    for ( i = 0; i < firstPlane.size(); i++ )
        if ( firstPlane.at(i) ) numBlackPoints++;
    if ((numBlackPoints) < numRemainingVoxels) return false;

    numBlackPoints = 0;
    for ( i = 0; i < secondPlane.size(); i++ )
        if ( secondPlane.at(i) ) numBlackPoints++;
    if ((numBlackPoints) < numRemainingVoxels) return false;

    // check if the deletion of the point disconnects the other points
    try {
        if ( this->connectedComponets2D(firstPlane) != 1 ) return false;
    }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
        cerr << "Out of range exception " << e.what() << endl;
        cerr << "Checking plane: error while counting connected components of plane 1." << endl;
        exit(1);
    }
    try {
        if ( this->connectedComponets2D(secondPlane) != 1 ) return false;
    }catch (std::out_of_range e) {  // pre-defined logical exception class is thrown
        cerr << "Out of range exception " << e.what() << endl;
        cerr << "Checking plane: error while counting connected components of plane 2." << endl;
        exit(1);
    }

    return true;

}

// counts the number of connected components in a 3x3 plane
short int CHierarchicalRLE::connectedComponets2D( vector<bool> array2D ) {

    vector<adjacency>           adjancenyTable;
    vector<short int>           outArray;
    vector<short int>           vArray;
    int                         i,j,k;
    int                         label = 0;
    bool                        found = false;

    adjacency                   newAdjacency;

    for ( i = 0; i < array2D.size(); i++ ) {
        outArray.push_back(0);
        vArray.push_back(0);
    }

    if ( this->adjancenyTableN8.size() == 0 )
        this->createAdjacencyTables();

    adjancenyTable = this->adjancenyTableN8;

//     for ( i = 0; i < 9; i++ )
//         adjancenyTable.push_back( newAdjacency );

/*
    adjancenyTable[0].adjVoxels.push_back(1);
    adjancenyTable[0].adjVoxels.push_back(3);
    adjancenyTable[0].adjVoxels.push_back(4);

    adjancenyTable[1].adjVoxels.push_back(0);
    adjancenyTable[1].adjVoxels.push_back(2);
    adjancenyTable[1].adjVoxels.push_back(3);
    adjancenyTable[1].adjVoxels.push_back(4);
    adjancenyTable[1].adjVoxels.push_back(5);

    adjancenyTable[2].adjVoxels.push_back(1);
    adjancenyTable[2].adjVoxels.push_back(4);
    adjancenyTable[2].adjVoxels.push_back(5);

    adjancenyTable[3].adjVoxels.push_back(0);
    adjancenyTable[3].adjVoxels.push_back(1);
    adjancenyTable[3].adjVoxels.push_back(4);
    adjancenyTable[3].adjVoxels.push_back(6);
    adjancenyTable[3].adjVoxels.push_back(7);

    adjancenyTable[4].adjVoxels.push_back(0);
    adjancenyTable[4].adjVoxels.push_back(1);
    adjancenyTable[4].adjVoxels.push_back(2);
    adjancenyTable[4].adjVoxels.push_back(3);
    adjancenyTable[4].adjVoxels.push_back(5);
    adjancenyTable[4].adjVoxels.push_back(6);
    adjancenyTable[4].adjVoxels.push_back(7);
    adjancenyTable[4].adjVoxels.push_back(8);

    adjancenyTable[5].adjVoxels.push_back(1);
    adjancenyTable[5].adjVoxels.push_back(2);
    adjancenyTable[5].adjVoxels.push_back(4);
    adjancenyTable[5].adjVoxels.push_back(7);
    adjancenyTable[5].adjVoxels.push_back(8);

    adjancenyTable[6].adjVoxels.push_back(3);
    adjancenyTable[6].adjVoxels.push_back(4);
    adjancenyTable[6].adjVoxels.push_back(7);

    adjancenyTable[7].adjVoxels.push_back(3);
    adjancenyTable[7].adjVoxels.push_back(4);
    adjancenyTable[7].adjVoxels.push_back(5);
    adjancenyTable[7].adjVoxels.push_back(6);
    adjancenyTable[7].adjVoxels.push_back(8);

    adjancenyTable[8].adjVoxels.push_back(4);
    adjancenyTable[8].adjVoxels.push_back(5);
    adjancenyTable[8].adjVoxels.push_back(7);
*/


    for ( i = 0; i < array2D.size(); i++ ) {
        if ( (array2D.at(i) == true) && (outArray.at(i) == 0) ) {
            outArray[i] = ++label;
            vArray[i] = 1;

            do {
                found = false;
                for ( j = 0; j < vArray.size(); j++ ) {
                    if ( vArray.at(j) == 1 ) {

                        for ( k = 0; k < adjancenyTable[j].adjVoxels.size(); k++ ) {
                            if ( (array2D.at(adjancenyTable[j].adjVoxels.at(k)) == true) &&
                                 (outArray.at(adjancenyTable[j].adjVoxels.at(k)) == 0 ) ) {
                                outArray[adjancenyTable[j].adjVoxels.at(k)] = label;
                                vArray[adjancenyTable[j].adjVoxels.at(k)] = 1;
                                found = true;
                            }
                        }

                        vArray[j] = 2;
                    }
                }
            } while (found);

        }
    }

    return label;

}

// check if two runs have the same type
bool CHierarchicalRLE::sameRunType( int a, int b ) {
  // both defined
  if ( RT_DEFINED(a) && RT_DEFINED(b) ) return true;

  // at least one is non defined
  else if ( (!RT_DEFINED(a) && !RT_DEFINED(b)) && (a == b) ) return true;

  return false;
}

// create adjacency tables for connected componets algorithms
void CHierarchicalRLE::createAdjacencyTables() {


    adjacency       newAdjacency;
    int             i;

    for ( i = 0; i < 28; i++ )
        this->adjancenyTableN26.push_back( newAdjacency );

    this->adjancenyTableN26[0].adjVoxels.push_back(1);
    this->adjancenyTableN26[0].adjVoxels.push_back(3);
    this->adjancenyTableN26[0].adjVoxels.push_back(4);
    this->adjancenyTableN26[0].adjVoxels.push_back(9);
    this->adjancenyTableN26[0].adjVoxels.push_back(10);
    this->adjancenyTableN26[0].adjVoxels.push_back(12);
    this->adjancenyTableN26[0].adjVoxels.push_back(13);

    this->adjancenyTableN26[1].adjVoxels.push_back(0);
    this->adjancenyTableN26[1].adjVoxels.push_back(2);
    this->adjancenyTableN26[1].adjVoxels.push_back(3);
    this->adjancenyTableN26[1].adjVoxels.push_back(4);
    this->adjancenyTableN26[1].adjVoxels.push_back(5);
    this->adjancenyTableN26[1].adjVoxels.push_back(9);
    this->adjancenyTableN26[1].adjVoxels.push_back(10);
    this->adjancenyTableN26[1].adjVoxels.push_back(11);
    this->adjancenyTableN26[1].adjVoxels.push_back(12);
    this->adjancenyTableN26[1].adjVoxels.push_back(13);
    this->adjancenyTableN26[1].adjVoxels.push_back(14);

    this->adjancenyTableN26[2].adjVoxels.push_back(1);
    this->adjancenyTableN26[2].adjVoxels.push_back(4);
    this->adjancenyTableN26[2].adjVoxels.push_back(5);
    this->adjancenyTableN26[2].adjVoxels.push_back(10);
    this->adjancenyTableN26[2].adjVoxels.push_back(11);
    this->adjancenyTableN26[2].adjVoxels.push_back(13);
    this->adjancenyTableN26[2].adjVoxels.push_back(14);

    this->adjancenyTableN26[3].adjVoxels.push_back(0);
    this->adjancenyTableN26[3].adjVoxels.push_back(1);
    this->adjancenyTableN26[3].adjVoxels.push_back(4);
    this->adjancenyTableN26[3].adjVoxels.push_back(6);
    this->adjancenyTableN26[3].adjVoxels.push_back(7);
    this->adjancenyTableN26[3].adjVoxels.push_back(9);
    this->adjancenyTableN26[3].adjVoxels.push_back(10);
    this->adjancenyTableN26[3].adjVoxels.push_back(12);
    this->adjancenyTableN26[3].adjVoxels.push_back(13);
    this->adjancenyTableN26[3].adjVoxels.push_back(15);
    this->adjancenyTableN26[3].adjVoxels.push_back(16);

    this->adjancenyTableN26[4].adjVoxels.push_back(0);
    this->adjancenyTableN26[4].adjVoxels.push_back(1);
    this->adjancenyTableN26[4].adjVoxels.push_back(2);
    this->adjancenyTableN26[4].adjVoxels.push_back(3);
    this->adjancenyTableN26[4].adjVoxels.push_back(5);
    this->adjancenyTableN26[4].adjVoxels.push_back(6);
    this->adjancenyTableN26[4].adjVoxels.push_back(7);
    this->adjancenyTableN26[4].adjVoxels.push_back(8);
    this->adjancenyTableN26[4].adjVoxels.push_back(9);
    this->adjancenyTableN26[4].adjVoxels.push_back(10);
    this->adjancenyTableN26[4].adjVoxels.push_back(11);
    this->adjancenyTableN26[4].adjVoxels.push_back(12);
    this->adjancenyTableN26[4].adjVoxels.push_back(13);
    this->adjancenyTableN26[4].adjVoxels.push_back(14);
    this->adjancenyTableN26[4].adjVoxels.push_back(15);
    this->adjancenyTableN26[4].adjVoxels.push_back(16);
    this->adjancenyTableN26[4].adjVoxels.push_back(17);

    this->adjancenyTableN26[5].adjVoxels.push_back(1);
    this->adjancenyTableN26[5].adjVoxels.push_back(2);
    this->adjancenyTableN26[5].adjVoxels.push_back(4);
    this->adjancenyTableN26[5].adjVoxels.push_back(7);
    this->adjancenyTableN26[5].adjVoxels.push_back(8);
    this->adjancenyTableN26[5].adjVoxels.push_back(10);
    this->adjancenyTableN26[5].adjVoxels.push_back(11);
    this->adjancenyTableN26[5].adjVoxels.push_back(13);
    this->adjancenyTableN26[5].adjVoxels.push_back(14);
    this->adjancenyTableN26[5].adjVoxels.push_back(16);
    this->adjancenyTableN26[5].adjVoxels.push_back(17);

    this->adjancenyTableN26[6].adjVoxels.push_back(3);
    this->adjancenyTableN26[6].adjVoxels.push_back(4);
    this->adjancenyTableN26[6].adjVoxels.push_back(7);
    this->adjancenyTableN26[6].adjVoxels.push_back(12);
    this->adjancenyTableN26[6].adjVoxels.push_back(13);
    this->adjancenyTableN26[6].adjVoxels.push_back(15);
    this->adjancenyTableN26[6].adjVoxels.push_back(16);

    this->adjancenyTableN26[7].adjVoxels.push_back(3);
    this->adjancenyTableN26[7].adjVoxels.push_back(4);
    this->adjancenyTableN26[7].adjVoxels.push_back(5);
    this->adjancenyTableN26[7].adjVoxels.push_back(6);
    this->adjancenyTableN26[7].adjVoxels.push_back(8);
    this->adjancenyTableN26[7].adjVoxels.push_back(12);
    this->adjancenyTableN26[7].adjVoxels.push_back(13);
    this->adjancenyTableN26[7].adjVoxels.push_back(14);
    this->adjancenyTableN26[7].adjVoxels.push_back(15);
    this->adjancenyTableN26[7].adjVoxels.push_back(16);
    this->adjancenyTableN26[7].adjVoxels.push_back(17);

    this->adjancenyTableN26[8].adjVoxels.push_back(4);
    this->adjancenyTableN26[8].adjVoxels.push_back(5);
    this->adjancenyTableN26[8].adjVoxels.push_back(7);
    this->adjancenyTableN26[8].adjVoxels.push_back(13);
    this->adjancenyTableN26[8].adjVoxels.push_back(14);
    this->adjancenyTableN26[8].adjVoxels.push_back(16);
    this->adjancenyTableN26[8].adjVoxels.push_back(17);

    this->adjancenyTableN26[9].adjVoxels.push_back(0);
    this->adjancenyTableN26[9].adjVoxels.push_back(1);
    this->adjancenyTableN26[9].adjVoxels.push_back(3);
    this->adjancenyTableN26[9].adjVoxels.push_back(4);
    this->adjancenyTableN26[9].adjVoxels.push_back(10);
    this->adjancenyTableN26[9].adjVoxels.push_back(12);
    this->adjancenyTableN26[9].adjVoxels.push_back(13);
    this->adjancenyTableN26[9].adjVoxels.push_back(18);
    this->adjancenyTableN26[9].adjVoxels.push_back(19);
    this->adjancenyTableN26[9].adjVoxels.push_back(21);
    this->adjancenyTableN26[9].adjVoxels.push_back(22);

    this->adjancenyTableN26[10].adjVoxels.push_back(0);
    this->adjancenyTableN26[10].adjVoxels.push_back(1);
    this->adjancenyTableN26[10].adjVoxels.push_back(2);
    this->adjancenyTableN26[10].adjVoxels.push_back(3);
    this->adjancenyTableN26[10].adjVoxels.push_back(4);
    this->adjancenyTableN26[10].adjVoxels.push_back(5);
    this->adjancenyTableN26[10].adjVoxels.push_back(9);
    this->adjancenyTableN26[10].adjVoxels.push_back(11);
    this->adjancenyTableN26[10].adjVoxels.push_back(12);
    this->adjancenyTableN26[10].adjVoxels.push_back(13);
    this->adjancenyTableN26[10].adjVoxels.push_back(14);
    this->adjancenyTableN26[10].adjVoxels.push_back(18);
    this->adjancenyTableN26[10].adjVoxels.push_back(19);
    this->adjancenyTableN26[10].adjVoxels.push_back(20);
    this->adjancenyTableN26[10].adjVoxels.push_back(21);
    this->adjancenyTableN26[10].adjVoxels.push_back(22);
    this->adjancenyTableN26[10].adjVoxels.push_back(23);

    this->adjancenyTableN26[11].adjVoxels.push_back(1);
    this->adjancenyTableN26[11].adjVoxels.push_back(2);
    this->adjancenyTableN26[11].adjVoxels.push_back(4);
    this->adjancenyTableN26[11].adjVoxels.push_back(5);
    this->adjancenyTableN26[11].adjVoxels.push_back(10);
    this->adjancenyTableN26[11].adjVoxels.push_back(13);
    this->adjancenyTableN26[11].adjVoxels.push_back(14);
    this->adjancenyTableN26[11].adjVoxels.push_back(19);
    this->adjancenyTableN26[11].adjVoxels.push_back(20);
    this->adjancenyTableN26[11].adjVoxels.push_back(22);
    this->adjancenyTableN26[11].adjVoxels.push_back(23);

    this->adjancenyTableN26[12].adjVoxels.push_back(0);
    this->adjancenyTableN26[12].adjVoxels.push_back(1);
    this->adjancenyTableN26[12].adjVoxels.push_back(3);
    this->adjancenyTableN26[12].adjVoxels.push_back(4);
    this->adjancenyTableN26[12].adjVoxels.push_back(6);
    this->adjancenyTableN26[12].adjVoxels.push_back(7);
    this->adjancenyTableN26[12].adjVoxels.push_back(9);
    this->adjancenyTableN26[12].adjVoxels.push_back(10);
    this->adjancenyTableN26[12].adjVoxels.push_back(13);
    this->adjancenyTableN26[12].adjVoxels.push_back(15);
    this->adjancenyTableN26[12].adjVoxels.push_back(16);
    this->adjancenyTableN26[12].adjVoxels.push_back(18);
    this->adjancenyTableN26[12].adjVoxels.push_back(19);
    this->adjancenyTableN26[12].adjVoxels.push_back(21);
    this->adjancenyTableN26[12].adjVoxels.push_back(22);
    this->adjancenyTableN26[12].adjVoxels.push_back(24);
    this->adjancenyTableN26[12].adjVoxels.push_back(25);

    this->adjancenyTableN26[13].adjVoxels.push_back(0);
    this->adjancenyTableN26[13].adjVoxels.push_back(1);
    this->adjancenyTableN26[13].adjVoxels.push_back(2);
    this->adjancenyTableN26[13].adjVoxels.push_back(3);
    this->adjancenyTableN26[13].adjVoxels.push_back(4);
    this->adjancenyTableN26[13].adjVoxels.push_back(5);
    this->adjancenyTableN26[13].adjVoxels.push_back(6);
    this->adjancenyTableN26[13].adjVoxels.push_back(7);
    this->adjancenyTableN26[13].adjVoxels.push_back(8);
    this->adjancenyTableN26[13].adjVoxels.push_back(9);
    this->adjancenyTableN26[13].adjVoxels.push_back(10);
    this->adjancenyTableN26[13].adjVoxels.push_back(11);
    this->adjancenyTableN26[13].adjVoxels.push_back(12);
    this->adjancenyTableN26[13].adjVoxels.push_back(14);
    this->adjancenyTableN26[13].adjVoxels.push_back(15);
    this->adjancenyTableN26[13].adjVoxels.push_back(16);
    this->adjancenyTableN26[13].adjVoxels.push_back(17);
    this->adjancenyTableN26[13].adjVoxels.push_back(18);
    this->adjancenyTableN26[13].adjVoxels.push_back(19);
    this->adjancenyTableN26[13].adjVoxels.push_back(20);
    this->adjancenyTableN26[13].adjVoxels.push_back(21);
    this->adjancenyTableN26[13].adjVoxels.push_back(22);
    this->adjancenyTableN26[13].adjVoxels.push_back(23);
    this->adjancenyTableN26[13].adjVoxels.push_back(24);
    this->adjancenyTableN26[13].adjVoxels.push_back(25);
    this->adjancenyTableN26[13].adjVoxels.push_back(26);

    this->adjancenyTableN26[14].adjVoxels.push_back(1);
    this->adjancenyTableN26[14].adjVoxels.push_back(2);
    this->adjancenyTableN26[14].adjVoxels.push_back(4);
    this->adjancenyTableN26[14].adjVoxels.push_back(5);
    this->adjancenyTableN26[14].adjVoxels.push_back(7);
    this->adjancenyTableN26[14].adjVoxels.push_back(8);
    this->adjancenyTableN26[14].adjVoxels.push_back(10);
    this->adjancenyTableN26[14].adjVoxels.push_back(11);
    this->adjancenyTableN26[14].adjVoxels.push_back(13);
    this->adjancenyTableN26[14].adjVoxels.push_back(16);
    this->adjancenyTableN26[14].adjVoxels.push_back(17);
    this->adjancenyTableN26[14].adjVoxels.push_back(19);
    this->adjancenyTableN26[14].adjVoxels.push_back(20);
    this->adjancenyTableN26[14].adjVoxels.push_back(22);
    this->adjancenyTableN26[14].adjVoxels.push_back(23);
    this->adjancenyTableN26[14].adjVoxels.push_back(25);
    this->adjancenyTableN26[14].adjVoxels.push_back(26);

    this->adjancenyTableN26[15].adjVoxels.push_back(3);
    this->adjancenyTableN26[15].adjVoxels.push_back(4);
    this->adjancenyTableN26[15].adjVoxels.push_back(6);
    this->adjancenyTableN26[15].adjVoxels.push_back(7);
    this->adjancenyTableN26[15].adjVoxels.push_back(12);
    this->adjancenyTableN26[15].adjVoxels.push_back(13);
    this->adjancenyTableN26[15].adjVoxels.push_back(16);
    this->adjancenyTableN26[15].adjVoxels.push_back(21);
    this->adjancenyTableN26[15].adjVoxels.push_back(22);
    this->adjancenyTableN26[15].adjVoxels.push_back(24);
    this->adjancenyTableN26[15].adjVoxels.push_back(25);

    this->adjancenyTableN26[16].adjVoxels.push_back(3);
    this->adjancenyTableN26[16].adjVoxels.push_back(4);
    this->adjancenyTableN26[16].adjVoxels.push_back(5);
    this->adjancenyTableN26[16].adjVoxels.push_back(6);
    this->adjancenyTableN26[16].adjVoxels.push_back(7);
    this->adjancenyTableN26[16].adjVoxels.push_back(8);
    this->adjancenyTableN26[16].adjVoxels.push_back(12);
    this->adjancenyTableN26[16].adjVoxels.push_back(13);
    this->adjancenyTableN26[16].adjVoxels.push_back(14);
    this->adjancenyTableN26[16].adjVoxels.push_back(15);
    this->adjancenyTableN26[16].adjVoxels.push_back(17);
    this->adjancenyTableN26[16].adjVoxels.push_back(21);
    this->adjancenyTableN26[16].adjVoxels.push_back(22);
    this->adjancenyTableN26[16].adjVoxels.push_back(23);
    this->adjancenyTableN26[16].adjVoxels.push_back(24);
    this->adjancenyTableN26[16].adjVoxels.push_back(25);
    this->adjancenyTableN26[16].adjVoxels.push_back(26);

    this->adjancenyTableN26[17].adjVoxels.push_back(4);
    this->adjancenyTableN26[17].adjVoxels.push_back(5);
    this->adjancenyTableN26[17].adjVoxels.push_back(7);
    this->adjancenyTableN26[17].adjVoxels.push_back(8);
    this->adjancenyTableN26[17].adjVoxels.push_back(13);
    this->adjancenyTableN26[17].adjVoxels.push_back(14);
    this->adjancenyTableN26[17].adjVoxels.push_back(16);
    this->adjancenyTableN26[17].adjVoxels.push_back(22);
    this->adjancenyTableN26[17].adjVoxels.push_back(23);
    this->adjancenyTableN26[17].adjVoxels.push_back(25);
    this->adjancenyTableN26[17].adjVoxels.push_back(26);

    this->adjancenyTableN26[18].adjVoxels.push_back(9);
    this->adjancenyTableN26[18].adjVoxels.push_back(10);
    this->adjancenyTableN26[18].adjVoxels.push_back(12);
    this->adjancenyTableN26[18].adjVoxels.push_back(13);
    this->adjancenyTableN26[18].adjVoxels.push_back(19);
    this->adjancenyTableN26[18].adjVoxels.push_back(21);
    this->adjancenyTableN26[18].adjVoxels.push_back(22);

    this->adjancenyTableN26[19].adjVoxels.push_back(9);
    this->adjancenyTableN26[19].adjVoxels.push_back(10);
    this->adjancenyTableN26[19].adjVoxels.push_back(11);
    this->adjancenyTableN26[19].adjVoxels.push_back(12);
    this->adjancenyTableN26[19].adjVoxels.push_back(13);
    this->adjancenyTableN26[19].adjVoxels.push_back(14);
    this->adjancenyTableN26[19].adjVoxels.push_back(18);
    this->adjancenyTableN26[19].adjVoxels.push_back(20);
    this->adjancenyTableN26[19].adjVoxels.push_back(21);
    this->adjancenyTableN26[19].adjVoxels.push_back(22);
    this->adjancenyTableN26[19].adjVoxels.push_back(23);

    this->adjancenyTableN26[20].adjVoxels.push_back(10);
    this->adjancenyTableN26[20].adjVoxels.push_back(11);
    this->adjancenyTableN26[20].adjVoxels.push_back(13);
    this->adjancenyTableN26[20].adjVoxels.push_back(14);
    this->adjancenyTableN26[20].adjVoxels.push_back(19);
    this->adjancenyTableN26[20].adjVoxels.push_back(22);
    this->adjancenyTableN26[20].adjVoxels.push_back(23);

    this->adjancenyTableN26[21].adjVoxels.push_back(9);
    this->adjancenyTableN26[21].adjVoxels.push_back(10);
    this->adjancenyTableN26[21].adjVoxels.push_back(12);
    this->adjancenyTableN26[21].adjVoxels.push_back(13);
    this->adjancenyTableN26[21].adjVoxels.push_back(15);
    this->adjancenyTableN26[21].adjVoxels.push_back(16);
    this->adjancenyTableN26[21].adjVoxels.push_back(18);
    this->adjancenyTableN26[21].adjVoxels.push_back(19);
    this->adjancenyTableN26[21].adjVoxels.push_back(22);
    this->adjancenyTableN26[21].adjVoxels.push_back(24);
    this->adjancenyTableN26[21].adjVoxels.push_back(25);

    this->adjancenyTableN26[22].adjVoxels.push_back(9);
    this->adjancenyTableN26[22].adjVoxels.push_back(10);
    this->adjancenyTableN26[22].adjVoxels.push_back(11);
    this->adjancenyTableN26[22].adjVoxels.push_back(12);
    this->adjancenyTableN26[22].adjVoxels.push_back(13);
    this->adjancenyTableN26[22].adjVoxels.push_back(14);
    this->adjancenyTableN26[22].adjVoxels.push_back(15);
    this->adjancenyTableN26[22].adjVoxels.push_back(16);
    this->adjancenyTableN26[22].adjVoxels.push_back(17);
    this->adjancenyTableN26[22].adjVoxels.push_back(18);
    this->adjancenyTableN26[22].adjVoxels.push_back(19);
    this->adjancenyTableN26[22].adjVoxels.push_back(20);
    this->adjancenyTableN26[22].adjVoxels.push_back(21);
    this->adjancenyTableN26[22].adjVoxels.push_back(23);
    this->adjancenyTableN26[22].adjVoxels.push_back(24);
    this->adjancenyTableN26[22].adjVoxels.push_back(25);
    this->adjancenyTableN26[22].adjVoxels.push_back(26);

    this->adjancenyTableN26[23].adjVoxels.push_back(10);
    this->adjancenyTableN26[23].adjVoxels.push_back(11);
    this->adjancenyTableN26[23].adjVoxels.push_back(13);
    this->adjancenyTableN26[23].adjVoxels.push_back(14);
    this->adjancenyTableN26[23].adjVoxels.push_back(16);
    this->adjancenyTableN26[23].adjVoxels.push_back(17);
    this->adjancenyTableN26[23].adjVoxels.push_back(19);
    this->adjancenyTableN26[23].adjVoxels.push_back(20);
    this->adjancenyTableN26[23].adjVoxels.push_back(22);
    this->adjancenyTableN26[23].adjVoxels.push_back(25);
    this->adjancenyTableN26[23].adjVoxels.push_back(26);

    this->adjancenyTableN26[24].adjVoxels.push_back(12);
    this->adjancenyTableN26[24].adjVoxels.push_back(13);
    this->adjancenyTableN26[24].adjVoxels.push_back(15);
    this->adjancenyTableN26[24].adjVoxels.push_back(16);
    this->adjancenyTableN26[24].adjVoxels.push_back(21);
    this->adjancenyTableN26[24].adjVoxels.push_back(22);
    this->adjancenyTableN26[24].adjVoxels.push_back(25);

    this->adjancenyTableN26[25].adjVoxels.push_back(12);
    this->adjancenyTableN26[25].adjVoxels.push_back(13);
    this->adjancenyTableN26[25].adjVoxels.push_back(14);
    this->adjancenyTableN26[25].adjVoxels.push_back(15);
    this->adjancenyTableN26[25].adjVoxels.push_back(16);
    this->adjancenyTableN26[25].adjVoxels.push_back(17);
    this->adjancenyTableN26[25].adjVoxels.push_back(21);
    this->adjancenyTableN26[25].adjVoxels.push_back(22);
    this->adjancenyTableN26[25].adjVoxels.push_back(23);
    this->adjancenyTableN26[25].adjVoxels.push_back(24);
    this->adjancenyTableN26[25].adjVoxels.push_back(26);

    this->adjancenyTableN26[26].adjVoxels.push_back(13);
    this->adjancenyTableN26[26].adjVoxels.push_back(14);
    this->adjancenyTableN26[26].adjVoxels.push_back(16);
    this->adjancenyTableN26[26].adjVoxels.push_back(17);
    this->adjancenyTableN26[26].adjVoxels.push_back(22);
    this->adjancenyTableN26[26].adjVoxels.push_back(23);
    this->adjancenyTableN26[26].adjVoxels.push_back(25);

    for ( i = 0; i < 28; i++ )
        this->adjancenyTableN6.push_back( newAdjacency );

    this->adjancenyTableN6[0].adjVoxels.push_back(1);
    this->adjancenyTableN6[0].adjVoxels.push_back(3);
    this->adjancenyTableN6[0].adjVoxels.push_back(9);

    this->adjancenyTableN6[1].adjVoxels.push_back(0);
    this->adjancenyTableN6[1].adjVoxels.push_back(2);
    this->adjancenyTableN6[1].adjVoxels.push_back(4);
    this->adjancenyTableN6[1].adjVoxels.push_back(10);

    this->adjancenyTableN6[2].adjVoxels.push_back(1);
    this->adjancenyTableN6[2].adjVoxels.push_back(5);
    this->adjancenyTableN6[2].adjVoxels.push_back(11);

    this->adjancenyTableN6[3].adjVoxels.push_back(0);
    this->adjancenyTableN6[3].adjVoxels.push_back(4);
    this->adjancenyTableN6[3].adjVoxels.push_back(6);
    this->adjancenyTableN6[3].adjVoxels.push_back(12);

    this->adjancenyTableN6[4].adjVoxels.push_back(1);
    this->adjancenyTableN6[4].adjVoxels.push_back(3);
    this->adjancenyTableN6[4].adjVoxels.push_back(5);
    this->adjancenyTableN6[4].adjVoxels.push_back(7);
    this->adjancenyTableN6[4].adjVoxels.push_back(13);

    this->adjancenyTableN6[5].adjVoxels.push_back(2);
    this->adjancenyTableN6[5].adjVoxels.push_back(4);
    this->adjancenyTableN6[5].adjVoxels.push_back(8);
    this->adjancenyTableN6[5].adjVoxels.push_back(14);

    this->adjancenyTableN6[6].adjVoxels.push_back(3);
    this->adjancenyTableN6[6].adjVoxels.push_back(7);
    this->adjancenyTableN6[6].adjVoxels.push_back(15);

    this->adjancenyTableN6[7].adjVoxels.push_back(4);
    this->adjancenyTableN6[7].adjVoxels.push_back(6);
    this->adjancenyTableN6[7].adjVoxels.push_back(8);
    this->adjancenyTableN6[7].adjVoxels.push_back(16);

    this->adjancenyTableN6[8].adjVoxels.push_back(5);
    this->adjancenyTableN6[8].adjVoxels.push_back(7);
    this->adjancenyTableN6[8].adjVoxels.push_back(17);

    this->adjancenyTableN6[9].adjVoxels.push_back(0);
    this->adjancenyTableN6[9].adjVoxels.push_back(10);
    this->adjancenyTableN6[9].adjVoxels.push_back(12);
    this->adjancenyTableN6[9].adjVoxels.push_back(18);

    this->adjancenyTableN6[10].adjVoxels.push_back(1);
    this->adjancenyTableN6[10].adjVoxels.push_back(9);
    this->adjancenyTableN6[10].adjVoxels.push_back(11);
    this->adjancenyTableN6[10].adjVoxels.push_back(13);
    this->adjancenyTableN6[10].adjVoxels.push_back(19);

    this->adjancenyTableN6[11].adjVoxels.push_back(2);
    this->adjancenyTableN6[11].adjVoxels.push_back(10);
    this->adjancenyTableN6[11].adjVoxels.push_back(14);
    this->adjancenyTableN6[11].adjVoxels.push_back(20);

    this->adjancenyTableN6[12].adjVoxels.push_back(3);
    this->adjancenyTableN6[12].adjVoxels.push_back(9);
    this->adjancenyTableN6[12].adjVoxels.push_back(13);
    this->adjancenyTableN6[12].adjVoxels.push_back(15);
    this->adjancenyTableN6[12].adjVoxels.push_back(21);

    this->adjancenyTableN6[13].adjVoxels.push_back(4);
    this->adjancenyTableN6[13].adjVoxels.push_back(10);
    this->adjancenyTableN6[13].adjVoxels.push_back(12);
    this->adjancenyTableN6[13].adjVoxels.push_back(14);
    this->adjancenyTableN6[13].adjVoxels.push_back(16);
    this->adjancenyTableN6[13].adjVoxels.push_back(22);

    this->adjancenyTableN6[14].adjVoxels.push_back(5);
    this->adjancenyTableN6[14].adjVoxels.push_back(11);
    this->adjancenyTableN6[14].adjVoxels.push_back(13);
    this->adjancenyTableN6[14].adjVoxels.push_back(17);
    this->adjancenyTableN6[14].adjVoxels.push_back(23);

    this->adjancenyTableN6[15].adjVoxels.push_back(6);
    this->adjancenyTableN6[15].adjVoxels.push_back(12);
    this->adjancenyTableN6[15].adjVoxels.push_back(16);
    this->adjancenyTableN6[15].adjVoxels.push_back(24);

    this->adjancenyTableN6[16].adjVoxels.push_back(7);
    this->adjancenyTableN6[16].adjVoxels.push_back(13);
    this->adjancenyTableN6[16].adjVoxels.push_back(15);
    this->adjancenyTableN6[16].adjVoxels.push_back(17);
    this->adjancenyTableN6[16].adjVoxels.push_back(25);

    this->adjancenyTableN6[17].adjVoxels.push_back(8);
    this->adjancenyTableN6[17].adjVoxels.push_back(14);
    this->adjancenyTableN6[17].adjVoxels.push_back(16);
    this->adjancenyTableN6[17].adjVoxels.push_back(26);

    this->adjancenyTableN6[18].adjVoxels.push_back(9);
    this->adjancenyTableN6[18].adjVoxels.push_back(19);
    this->adjancenyTableN6[18].adjVoxels.push_back(21);

    this->adjancenyTableN6[19].adjVoxels.push_back(10);
    this->adjancenyTableN6[19].adjVoxels.push_back(18);
    this->adjancenyTableN6[19].adjVoxels.push_back(20);
    this->adjancenyTableN6[19].adjVoxels.push_back(22);

    this->adjancenyTableN6[20].adjVoxels.push_back(11);
    this->adjancenyTableN6[20].adjVoxels.push_back(19);
    this->adjancenyTableN6[20].adjVoxels.push_back(23);

    this->adjancenyTableN6[21].adjVoxels.push_back(12);
    this->adjancenyTableN6[21].adjVoxels.push_back(18);
    this->adjancenyTableN6[21].adjVoxels.push_back(22);
    this->adjancenyTableN6[21].adjVoxels.push_back(24);

    this->adjancenyTableN6[22].adjVoxels.push_back(13);
    this->adjancenyTableN6[22].adjVoxels.push_back(19);
    this->adjancenyTableN6[22].adjVoxels.push_back(21);
    this->adjancenyTableN6[22].adjVoxels.push_back(23);
    this->adjancenyTableN6[22].adjVoxels.push_back(25);

    this->adjancenyTableN6[23].adjVoxels.push_back(14);
    this->adjancenyTableN6[23].adjVoxels.push_back(20);
    this->adjancenyTableN6[23].adjVoxels.push_back(22);
    this->adjancenyTableN6[23].adjVoxels.push_back(26);

    this->adjancenyTableN6[24].adjVoxels.push_back(15);
    this->adjancenyTableN6[24].adjVoxels.push_back(21);
    this->adjancenyTableN6[24].adjVoxels.push_back(25);

    this->adjancenyTableN6[25].adjVoxels.push_back(16);
    this->adjancenyTableN6[25].adjVoxels.push_back(22);
    this->adjancenyTableN6[25].adjVoxels.push_back(24);
    this->adjancenyTableN6[25].adjVoxels.push_back(26);

    this->adjancenyTableN6[26].adjVoxels.push_back(17);
    this->adjancenyTableN6[26].adjVoxels.push_back(23);
    this->adjancenyTableN6[26].adjVoxels.push_back(25);

    for ( i = 0; i < 9; i++ )
        this->adjancenyTableN8.push_back( newAdjacency );

    this->adjancenyTableN8[0].adjVoxels.push_back(1);
    this->adjancenyTableN8[0].adjVoxels.push_back(3);
    this->adjancenyTableN8[0].adjVoxels.push_back(4);

    this->adjancenyTableN8[1].adjVoxels.push_back(0);
    this->adjancenyTableN8[1].adjVoxels.push_back(2);
    this->adjancenyTableN8[1].adjVoxels.push_back(3);
    this->adjancenyTableN8[1].adjVoxels.push_back(4);
    this->adjancenyTableN8[1].adjVoxels.push_back(5);

    this->adjancenyTableN8[2].adjVoxels.push_back(1);
    this->adjancenyTableN8[2].adjVoxels.push_back(4);
    this->adjancenyTableN8[2].adjVoxels.push_back(5);

    this->adjancenyTableN8[3].adjVoxels.push_back(0);
    this->adjancenyTableN8[3].adjVoxels.push_back(1);
    this->adjancenyTableN8[3].adjVoxels.push_back(4);
    this->adjancenyTableN8[3].adjVoxels.push_back(6);
    this->adjancenyTableN8[3].adjVoxels.push_back(7);

    this->adjancenyTableN8[4].adjVoxels.push_back(0);
    this->adjancenyTableN8[4].adjVoxels.push_back(1);
    this->adjancenyTableN8[4].adjVoxels.push_back(2);
    this->adjancenyTableN8[4].adjVoxels.push_back(3);
    this->adjancenyTableN8[4].adjVoxels.push_back(5);
    this->adjancenyTableN8[4].adjVoxels.push_back(6);
    this->adjancenyTableN8[4].adjVoxels.push_back(7);
    this->adjancenyTableN8[4].adjVoxels.push_back(8);

    this->adjancenyTableN8[5].adjVoxels.push_back(1);
    this->adjancenyTableN8[5].adjVoxels.push_back(2);
    this->adjancenyTableN8[5].adjVoxels.push_back(4);
    this->adjancenyTableN8[5].adjVoxels.push_back(7);
    this->adjancenyTableN8[5].adjVoxels.push_back(8);

    this->adjancenyTableN8[6].adjVoxels.push_back(3);
    this->adjancenyTableN8[6].adjVoxels.push_back(4);
    this->adjancenyTableN8[6].adjVoxels.push_back(7);

    this->adjancenyTableN8[7].adjVoxels.push_back(3);
    this->adjancenyTableN8[7].adjVoxels.push_back(4);
    this->adjancenyTableN8[7].adjVoxels.push_back(5);
    this->adjancenyTableN8[7].adjVoxels.push_back(6);
    this->adjancenyTableN8[7].adjVoxels.push_back(8);

    this->adjancenyTableN8[8].adjVoxels.push_back(4);
    this->adjancenyTableN8[8].adjVoxels.push_back(5);
    this->adjancenyTableN8[8].adjVoxels.push_back(7);

}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//      Methodes for testing only
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/*!
    \fn CHierarchicalRLE::draw()
 */
// draw the level set as cubes - only for 3d level sets implemented
void CHierarchicalRLE::drawCubes()
{
    // for comnputing the average access time of the random access operator
    long            numGridPoints = 0;

    if (this->dimension == 3) {

        int minX = this->lower_hrle->lower_hrle->min;
        int maxX = this->lower_hrle->lower_hrle->max;
        int minY = this->lower_hrle->min;
        int maxY = this->lower_hrle->max;
        int minZ = this->min;
        int maxZ = this->max;

        float CubeW = (this->maxRange.getX() - this->minRange.getX()) / (float)(maxX - minX);
        float CubeH = (this->maxRange.getY() - this->minRange.getY()) / (float)(maxY - minY);
        float CubeD = (this->maxRange.getZ() - this->minRange.getZ()) / (float)(maxZ - minZ);

        for (int z = minZ; z < maxZ; z++) {
            for (int y = minY; y < maxY; y++) {
                for (int x = minX; x < maxX; x++) {
                    numGridPoints++;

                    int run = this->randomAccess( x, y, z );

                    if (RT_DEFINED(run)) {

                        // draw cubes and cube edges
                        // -------------------------

                        CVertex3D point1( this->minRange.getX() + x * CubeW,
                                          this->minRange.getY() + y * CubeH,
                                          this->minRange.getZ() + z * CubeD );
                        CVertex3D point2( this->minRange.getX() + (x+1) * CubeW,
                                          this->minRange.getY() + (y+1) * CubeH,
                                          this->minRange.getZ() + (z+1) * CubeD);

                        this->drawACube(&point1, &point2, 1.0);

                    }
                }
            }
        }

    }
}

// draw one single cube
void CHierarchicalRLE::drawACube(CVertex3D *p1, CVertex3D *p2, double colorValue) {

    // Fl�hen zeichnen
    // ----------------
        // Farbe festlegen
        glColor3f(0.0, 0.0, 1.0 * colorValue);

        // Offset fr den Cube (bessere Darstellung der Kanten)
        glEnable(GL_POLYGON_OFFSET_FILL);
        glPolygonOffset(1.f, 1.f);

            glBegin(GL_POLYGON);
                glNormal3f(  -1.0,  0.0,  0.0);
                glVertex3f(  p1->getX(), p1->getY(), p1->getZ());
                glVertex3f(  p1->getX(), p1->getY(), p2->getZ());
                glVertex3f(  p1->getX(), p2->getY(), p2->getZ());
                glVertex3f(  p1->getX(), p2->getY(), p1->getZ());
            glEnd();

            glBegin(GL_POLYGON);
                glNormal3d( 1.0,  0.0,  0.0);
                glVertex3d( p2->getX(), p2->getY(), p1->getZ());
                glVertex3d( p2->getX(), p2->getY(), p2->getZ());
                glVertex3d( p2->getX(), p1->getY(), p2->getZ());
                glVertex3d( p2->getX(), p1->getY(), p1->getZ());
            glEnd();

            glBegin(GL_POLYGON);
                glNormal3d(  0.0,  -1.0,  0.0);
                glVertex3d( p1->getX(), p1->getY(), p1->getZ());
                glVertex3d( p2->getX(), p1->getY(), p1->getZ());
                glVertex3d( p2->getX(), p1->getY(), p2->getZ());
                glVertex3d( p1->getX(), p1->getY(), p2->getZ());
            glEnd();

            glBegin(GL_POLYGON);
                glNormal3d(  0.0, 1.0,  0.0);
                glVertex3d( p2->getX(), p2->getY(), p1->getZ());
                glVertex3d( p1->getX(), p2->getY(), p1->getZ());
                glVertex3d( p1->getX(), p2->getY(), p2->getZ());
                glVertex3d( p2->getX(), p2->getY(), p2->getZ());
            glEnd();

            glBegin(GL_POLYGON);
                glNormal3d(  0.0,  0.0, -1.0);
                glVertex3d( p1->getX(), p1->getY(), p1->getZ());
                glVertex3d( p1->getX(), p2->getY(), p1->getZ());
                glVertex3d( p2->getX(), p2->getY(), p1->getZ());
                glVertex3d( p2->getX(), p1->getY(), p1->getZ());
            glEnd();

            glBegin(GL_POLYGON);
                glNormal3d(  0.0,  0.0, 1.0);
                glVertex3d( p2->getX(), p1->getY(), p2->getZ());
                glVertex3d( p2->getX(), p2->getY(), p2->getZ());
                glVertex3d( p1->getX(), p2->getY(), p2->getZ());
                glVertex3d( p1->getX(), p1->getY(), p2->getZ());
            glEnd();

        glDisable(GL_POLYGON_OFFSET_FILL);

        // draw lines

        glColor3d(0.2, 0.2, 0.2);

        glBegin(GL_LINES);
            glVertex3d(p1->getX(), p1->getY(), p1->getZ());
            glVertex3d(p1->getX(), p1->getY(), p2->getZ());
        glEnd();

        glBegin(GL_LINES);
            glVertex3d(p1->getX(), p1->getY(), p1->getZ());
            glVertex3d(p2->getX(), p1->getY(), p1->getZ());
        glEnd();

        glBegin(GL_LINES);
            glVertex3d(p1->getX(), p1->getY(), p1->getZ());
            glVertex3d(p1->getX(), p2->getY(), p1->getZ());
        glEnd();

        glBegin(GL_LINES);
            glVertex3d(p2->getX(), p1->getY(), p1->getZ());
            glVertex3d(p2->getX(), p2->getY(), p1->getZ());
        glEnd();

        glBegin(GL_LINES);
            glVertex3d(p2->getX(), p1->getY(), p1->getZ());
            glVertex3d(p2->getX(), p1->getY(), p2->getZ());
        glEnd();

        glBegin(GL_LINES);
            glVertex3d(p1->getX(), p2->getY(), p1->getZ());
            glVertex3d(p2->getX(), p2->getY(), p1->getZ());
        glEnd();

        glBegin(GL_LINES);
            glVertex3d(p1->getX(), p2->getY(), p1->getZ());
            glVertex3d(p1->getX(), p2->getY(), p2->getZ());
        glEnd();

        glBegin(GL_LINES);
            glVertex3d(p2->getX(), p2->getY(), p1->getZ());
            glVertex3d(p2->getX(), p2->getY(), p2->getZ());
        glEnd();

        glBegin(GL_LINES);
            glVertex3d(p2->getX(), p2->getY(), p2->getZ());
            glVertex3d(p1->getX(), p2->getY(), p2->getZ());
        glEnd();

        glBegin(GL_LINES);
            glVertex3d(p2->getX(), p2->getY(), p2->getZ());
            glVertex3d(p2->getX(), p1->getY(), p2->getZ());
        glEnd();

        glBegin(GL_LINES);
            glVertex3d(p1->getX(), p2->getY(), p2->getZ());
            glVertex3d(p1->getX(), p1->getY(), p2->getZ());
        glEnd();

        glBegin(GL_LINES);
            glVertex3d(p1->getX(), p1->getY(), p2->getZ());
            glVertex3d(p2->getX(), p1->getY(), p2->getZ());
        glEnd();


    return;
}

// print values of the data structures to standard out
void CHierarchicalRLE::printDataStructure() {

    cout << "Ausgabe der Datenstruktur, Dimension: " << this->dimension << endl;
    int i;
    cout << "Min: " << this->min << endl;
    cout << "Max: " << this->max << endl;
    cout << "Start Indices (" << this->start_indices.size() << "): ";
    for (i = 0; i < this->start_indices.size(); i++) cout << this->start_indices.at(i) << " | ";
    cout << endl;
    cout << "Run Types (" << this->run_types.size() << "): ";
    for (i = 0; i < this->run_types.size(); i++) {
        if ((i % 20) == 0) cout << endl;
        if ( RT_DEFINED(this->run_types[i]) ) cout << "d "<< this->run_types[i] <<" | ";
        else if ( this->run_types[i] == RT_NEGATIVE ) cout << "- | ";
        else if ( this->run_types[i] == RT_POSITIVE ) cout << "+ | ";
        else cout << " Error!! " << this->run_types[i];
    }
    cout << endl;
    cout << "Run Breaks (" << this->run_breaks.size() << "): ";
    for (i = 0; i < this->run_breaks.size(); i++) cout << this->run_breaks.at(i) << " | ";
    cout << endl;

    if (this->lower_hrle != NULL) this->lower_hrle->printDataStructure();
}

// print values of a coordinate vector
void CHierarchicalRLE::printCoordValue( vector<coordValue> value ) {

    for ( int i = 0; i < value.size(); i++ ) {

        cout << value.at(i).coord << ", ";
        if (value.at(i).subCoords.size() > 0)
            this->printCoordValue(value.at(i).subCoords);

    }

    cout << endl;

}

